﻿//================================================================================
//  
//      AudioSGForm.cs      オーディオ信号発生器画面
//  
//        (c)2012/08/28  by OjisanKoubou
//  
//================================================================================
//  
//  このプログラムは Ojisankoubou( OGAWA Ichiro ) が著作権を持っています。
//  個人の研究目的には、修正、改造など自由に使用できます。
//  発表される際は著作権表示部分を消さないようにしてください。
//  このソフトウェアの全部もしくは一部を製品に組み込んだり販売したりするなどの
//  商業的利用はできません。
//

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading;
using System.Windows.Forms;

namespace APB_3
{
    public partial class AudioSGForm : Form
    {
        private static int wn = 0;
        // 
        // 0x0400 DDS0 FREQ0[ 7.. 0]    変換周波数
        // 0x0401 DDS0 FREQ0[15.. 8]
        // 0x0402 DDS0 FREQ0[23..16]
        // 0x0403 DDS0 FREQ0[31..24]
        //
        // 0x0404 DDS1 FREQ1[ 7.. 0]    L ch
        // 0x0405 DDS1 FREQ1[15.. 8]
        // 0x0406 DDS1 FREQ1[23..16]
        // 0x0407 DDS1 WAVE1            0:SINE 1:SQUARE 2:SAW 3:TRIAGL
        //
        // 0x0408 DDS2 FREQ2[ 7.. 0]    R ch
        // 0x0409 DDS2 FREQ2[15.. 8]
        // 0x040A DDS2 FREQ2[23..16]
        // 0x040B DDS2 WAVE2            0:SINE 1:SQUARE 2:SAW 3:TRIAGL
        // 


        private bool modifying = true;   // 変更中は２重動作しないようにする


        public AudioSGForm()
        {
            InitializeComponent();
            wn++;
            this.Text = "オーディオ信号発生器 " + wn.ToString();
            Apb3.mainf.LogPrint("\n\nAudioSignalForm:" + wn.ToString());

            comboBox_FreqL.Items.AddRange(def.audiosgfreqstrtbl);
            comboBox_WaveL.Items.AddRange(def.audiosgwavestrtbl);
            comboBox_DeltaFreqL.Items.AddRange(def.audiosgfreqstepstrtbl);
            
            comboBox_FreqR.Items.AddRange(def.audiosgfreqstrtbl);
            comboBox_WaveR.Items.AddRange(def.audiosgwavestrtbl);
            comboBox_PhaseR.Items.AddRange(def.audiosgphasestrtbl);
            comboBox_DeltaFreqR.Items.AddRange(def.audiosgfreqstepstrtbl);

            comboBox_FreqL.SelectedIndex = 12;      // 1kHz
            comboBox_WaveL.SelectedIndex = 0;       // サイン
            comboBox_DeltaFreqL.SelectedIndex = 6;  // 100Hz
            
            comboBox_FreqR.SelectedIndex = 12;      // 1kHz
            comboBox_WaveR.SelectedIndex = 0;       // 
            comboBox_PhaseR.SelectedIndex = 10;      // 0 degree
            comboBox_DeltaFreqR.SelectedIndex = 6;  // 

            modifying = false;
            setting();
        }


        //
        // 設定が変更された
        //
        private void SettingChanged(object sender, EventArgs e)
        {
            if (!modifying)
            {
                Apb3.mainf.LogPrint("\nAudioSignalForm.SettingChanged");
                setting();
            }
        }


        //
        // 設定
        //
        private void setting()
        {
            if (Apb3.Open())
            {
                modifying = true;
                
                double freqL = util.ToDouble(comboBox_FreqL.Text);
                freqL = System.Math.Min(freqL, 48828);  // fs/2=48.828kHz(100M/2048)  fs=97.656kHz
                freqL = System.Math.Max(freqL, 0);
                int waveL = (int)comboBox_WaveL.SelectedIndex;
                Apb3.dds2( freqL, waveL );  // dds2 
                
                double freqR = util.ToDouble(comboBox_FreqR.Text);
                freqR = System.Math.Min(freqR, 48828);  // fs/2=48.828kHz(100M/2048)  fs=97.656kHz
                freqR = System.Math.Max(freqR, 0);
                double fratio = freqR / freqL;
                int fratio_n = (int)(fratio + 0.4999);
                if ((checkBox_Fratio.Checked) && (fratio > 0.999) && (System.Math.Abs(fratio - fratio_n) < 0.0001))
                {
                    freqR = (int)( freqL / Apb3.ddsdeltaf + 0.5 ) * fratio_n * Apb3.ddsdeltaf;  // 正確な整数倍になるよう補正
                }
                int waveR = (int)comboBox_WaveR.SelectedIndex;
                double phaseR = util.ToDouble(comboBox_PhaseR.Text);
                Apb3.dds3( freqR, waveR, phaseR );
                
Apb3.mainf.LogPrint("\nAudioSignalForm.setting  freqL=" + freqL.ToString() + "  waveL=" + waveL.ToString() + "  freqR=" + freqR.ToString() + "  waveR=" + waveR.ToString());
                
                // DAC 入力を DDS2/DDS3 にする 0:off 1:DDS2 2:DDS3 3:DEMOD 4:I/Q 5:任意 6:オーディオ入力 7:固定パターン
                Apb3.uda1345_OutSelect( (checkBox_OutputL.Checked ? 1 : 0), (checkBox_OutputR.Checked ? 2 : 0) );
                
                modifying = false;
                Apb3.Close();
            }
        }


        //
        // L ch 周波数マイナスボタン
        //
        private void button_MinusFreqL_Click(object sender, EventArgs e)
        {
            double deltafreq = util.ToDouble(comboBox_DeltaFreqL.Text);
            double freq = util.ToDouble(comboBox_FreqL.Text) - deltafreq;
            comboBox_FreqL.Text = " "+util.Double2String(freq);
        }


        //
        // L ch 周波数プラスボタン
        //
        private void button_PlusFreqL_Click(object sender, EventArgs e)
        {
            double deltafreq = util.ToDouble(comboBox_DeltaFreqL.Text);
            double freq = util.ToDouble(comboBox_FreqL.Text) + deltafreq;
            comboBox_FreqL.Text = " " + util.Double2String(freq);
        }


        //
        // R ch 周波数マイナスボタン
        //
        private void button_MinusFreqR_Click(object sender, EventArgs e)
        {
            double deltafreq = util.ToDouble(comboBox_DeltaFreqR.Text);
            double freq = util.ToDouble(comboBox_FreqR.Text) - deltafreq;
            comboBox_FreqR.Text = " " + util.Double2String(freq);
        }


        //
        // R ch 周波数プラスボタン
        //
        private void button_PlusFreqR_Click(object sender, EventArgs e)
        {
            double deltafreq = util.ToDouble(comboBox_DeltaFreqR.Text);
            double freq = util.ToDouble(comboBox_FreqR.Text) + deltafreq;
            comboBox_FreqR.Text = " " + util.Double2String(freq);
        }

        private void AudioSGForm_Click(object sender, EventArgs e)
        {
            this.Focus();               // どこをクリックされてもフォーカスがくるように
        }

        private void AudioSGForm_MouseDown(object sender, MouseEventArgs e)
        {
            this.Focus();               // どこをクリックされてもフォーカスがくるように
        }








    }
}
