﻿//================================================================================
//  
//       DebugForm.cs      デバッグ用画面
//  
//        (c)2012/08/28  by OjisanKoubou
//  
//================================================================================
//  
//  このプログラムは Ojisankoubou( OGAWA Ichiro ) が著作権を持っています。
//  個人の研究目的には、修正、改造など自由に使用できます。
//  発表される際は著作権表示部分を消さないようにしてください。
//  このソフトウェアの全部もしくは一部を製品に組み込んだり販売したりするなどの
//  商業的利用はできません。
//

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using FTD2XX_NET;


namespace APB_3
{
    public partial class DebugForm : Form
    {
        private static int wn = 0;
        public MainForm mainf;                      // 
        
        private Byte[] Rbuf = new Byte[256];
        
        
        //
        //　コンストラクタ
        //
        public DebugForm()
        {
            InitializeComponent();
            mainf = Apb3.mainf;
            wn++;
            this.Text = "デバッグ " + wn.ToString();
        }
        
        
        //
        //  ボタン　接続されているＦＴ２３２Ｈ情報
        //
        private void button_info_Click(object sender, EventArgs e)
        {
            Print( Apb3.Ft232h.FtInfo() );
        }
        
        
        //
        //  ボタン　ＦＰＧＡコンフィグ
        //
        private void button_config_Click(object sender, EventArgs e)
        {
            if( Apb3.DeviceInit() ) {
                Print("\n\nConfigured");
                Print("\nFPGA version="+Apb3.FpgaSignature());
            }
            else
            {
                Print("\n\nERROR: Configure failed");
            }
        }
        
        
        //
        //  ボタン　ＦＰＧＡ ＩＤ 読みだし
        //
        private void button_id_Click(object sender, EventArgs e)
        {
            if (Apb3.Open())
            {
                int numBytesRead = Apb3.Fpga.FpgaRead( 0, 16, Rbuf );
                Print( "\n\n");
                for( int i=0; i<16; i++ ) Print( Rbuf[i].ToString("X2"));
                Print("\n" + Encoding.ASCII.GetString(Rbuf));
                Apb3.Close();
            }
        }
        
        
        //
        //  ボタン　ＦＰＧＡから読み出し
        //
        private void button_read_Click(object sender, EventArgs e)
        {
            if (Apb3.Open())
            {
                int adrs = (int)util.ToInt32(textBox_ReadAdrs.Text, 16);   // 
                int size = (int)util.ToInt32(textBox_ReadSize.Text, 16);   // 
                if (size > 256) size = 256;
                if (size == 0) size = 16;
                
                Apb3.Fpga.FpgaRead( adrs, size, Rbuf );
                for (int j = 0; j < size; )
                {
                    Print("\n" + adrs.ToString("X4") + " ");
                    for (int i = 0; i < 16; i++)
                    {
                        Print(Rbuf[j].ToString("X2") + " ");
                        j++;
                        if (j >= size) break;
                    }
                    adrs += 16;
                }

                Apb3.Close();
            }
        }
        
        
        //
        //  ボタン　ＦＰＧＡ書き込み
        //
        private void button_write_Click(object sender, EventArgs e)
        {
            if (Apb3.Open())
            {
                int adrs = (int)util.ToInt32(textBox_WriteAdrs.Text, 16);   // 
                int size = (int)util.ToInt32(textBox_WriteSize.Text, 16);   // 
                int data = (int)util.ToInt32(textBox_WriteData.Text, 16);   // 
                
                Apb3.Fpga.FpgaWrite( adrs, size, data );
                
                Apb3.Close();
            }
        }
        
        
        //
        //  ボタン　ADC1610S の全レジスターを読み出す
        //
        private void button_ADCRead_Click(object sender, EventArgs e)
        {
            if (Apb3.Open())
            {
                Print("\n\nADC1610S Registers");
                for( int i=0; i<13; i++ ) {
                    int adcreg = Apb3.Adc1610.reg[i];   // 
                    int data = Apb3.Adc1610.AdcRegRead( adcreg );
                    Print("\n" + adcreg.ToString("X4") + ": " + data.ToString("X2"));
                }
                
                Apb3.Close();
            }
        }
        
        
        //
        //  ボタン　ADC1610S のレジスターに書き込む
        //
        private void button_ADCWrite_Click(object sender, EventArgs e)
        {
            if (Apb3.Open())
            {
                int adcreg = (int)util.ToInt32(textBox_ADCWriteAdrs.Text, 16);   // 
                int data    = (int)util.ToInt32(textBox_ADCWriteData.Text, 16);   // 
                
                Apb3.Adc1610.AdcRegWrite( adcreg, data );
                Print("\n\nADC1610S Register "+adcreg.ToString("X4")+" = "+data.ToString("X2"));
                
                Apb3.Close();
            }
        }
        
        
        //
        //  ボタン　ADC1610S のレジスターセットアップ
        //
        private void button_ADCSetup_Click(object sender, EventArgs e)
        {
            if (Apb3.Open())
            {
                Print("\n\nADC1610S Setup");
                Apb3.Adc1610.AdcInit();
                for( int i=0; i<13; i++ ) {
                    int adcreg = Apb3.Adc1610.reg[i];   // 
                    int  data = Apb3.Adc1610.AdcRegRead( adcreg );
                    Print("\n" + adcreg.ToString("X4") + ": " + data.ToString("X2"));
                }
                
                Apb3.Close();
            }
        }


        //
        //  ボタン　ADC1610S データ入力の IODELAY2 遅延設定
        //
        private void button_DlySet_Click(object sender, EventArgs e)
        {
            if (Apb3.Open())
            {
                int dly = (int)util.ToInt32(textBox_Dly.Text, 16);
                Print("\n\nIODelay2 設定 = 0x" + dly.ToString( "X2") );
                Apb3.Adc1610.AdcIoDelaySetting( dly );
                Apb3.Close();
            }
        }
        
        
        //
        //  ボタン　ADC1610S データ入力の IODELAY2 遅延自動設定
        //
        private void button_DlyErrTbl_Click(object sender, EventArgs e)
        {
            if (Apb3.Open())
            {
                Apb3.Adc1610.AdcInit();  //ADC1610S のレジスタ初期化と遅延自動設定
                Print("\n\nIODelay2 自動設定 = 0x" + Apb3.adccaldly.ToString( "X2") );
                Apb3.Close();
            }
        }
        
        
        //
        //  ボタン　ADC1610S データ入力の IODELAY2 遅延リセット
        //
        private void button_DlyRst_Click(object sender, EventArgs e)
        {
            if (Apb3.Open())
            {
                Apb3.Close();
            }
        }
        
        
        //
        //  ボタン　DataFlash ID 読み出し
        //
        private void button_DFID_Click(object sender, EventArgs e)
        {
            if (Apb3.Open())
            {
                Print("\n\nDF ID=" + Apb3.Df.DfReadID());
                Apb3.Close();
            }
        }
        
        
        //
        //  ボタン　DataFlash 読み出し
        //
        private void button_DFRead_Click(object sender, EventArgs e)
        {
            if (Apb3.Open())
            {
                int adrs = (int)util.ToInt32(textBox_DFReadAdrs.Text, 16);   // 
                int size = (int)util.ToInt32(textBox_DFReadSize.Text, 16);   // 
                Print("\n\nDF Read  adrs=" + adrs.ToString("X6") + " size=" + size.ToString("X2") );
                byte[] buf = new byte[size];
                Apb3.Df.DfRead( adrs, size, buf );
                for( int i=0; i<size; i++ ) {
                    if( (i & 0x0F)==0 ) Print( "\n"+ adrs.ToString("X6") );
                    Print( " "+buf[i].ToString("X2") );
                    adrs++;
                }
                Apb3.Close();
            }
        }
        
        
        //
        //  ボタン　DataFlash 書き込み
        //
        private void button_DFWrite_Click(object sender, EventArgs e)
        {
            if (Apb3.Open())
            {
                Apb3.Df.DfUnProtectAll();
                int adrs = (int)util.ToInt32(textBox_DFWriteAdrs.Text, 16);   // 
                int data = (int)util.ToInt32(textBox_DFWriteData.Text, 16);   // 
                int size = (int)util.ToInt32(textBox_DFWriteSize.Text, 16);   // 
                Print( "\n\nDF Write  adrs=" + adrs.ToString("X6") + " size=" + size.ToString("X2") + " data=" + data.ToString("X4") );
                byte[] buf = new byte[256];
                for( int i=0; i<256; i+=4 ) {
                    buf[i+0] = (byte)(data >> 24);
                    buf[i+1] = (byte)(data >> 16);
                    buf[i+2] = (byte)(data >> 8);
                    buf[i+3] = (byte)(data);
                }
                Apb3.Df.DfWrite( adrs, size, buf );
                Apb3.Close();
            }
        }
        
        
        //
        //  ボタン　DataFlash チップイレーズ
        //
        private void button_DFChipErase_Click(object sender, EventArgs e)
        {
            if (Apb3.Open())
            {
                Print( "\n\nDF Chip Erase" );
                Apb3.Df.DfChipErase();
                Apb3.Close();
            }
        }
        
        
        //
        //  ボタン　DataFlash コンフィグデータ書き込み
        //
        private void button_DFWConfig_Click(object sender, EventArgs e)
        {
            if (Apb3.Open())
            {
                Print( "\n\nDF Write Config data\n" );
                Apb3.OpenConfigFile();
                if (Apb3.filesize != 0)
                {
                    // bit ファイルの先頭の不要な部分を削除（Spartan6 ではこの部分があると SPI コンフィグ失敗する）
                    int offset;
                    for( offset=0; offset<256; offset++ ) {        // コンフィグ有効データ開始位置を探す
                        if( Apb3.filebuf[offset] == 0xFF ) break;
                    }
                    if( offset == 256 ) {  // コンフィグ有効データが見つからなかった
                        Print(": ERROR   Invalid config file");
                        Apb3.Close();
                        return;
                    }
                    int size = 0;
                    int remain = Apb3.filesize-offset;
                    int dfadrs = 0;
                    byte[] buf = new byte[32768];
                    Apb3.Df.DfChipErase();
                    Apb3.Df.DfUnProtectAll();
                    while( remain != 0 ) {
                        Print( "." );
                        Thread.Sleep(1);
                        size = ( remain >= 32768 )? 32768: remain;
                        Buffer.BlockCopy( Apb3.filebuf, (int)offset, buf, 0, (int)size ); // データコピー
                        Apb3.Df.DfWrite( dfadrs, size, buf );
                        remain -= size;
                        offset += size;
                        dfadrs += size;
                    }
                    Print( " Done" );
                    Apb3.Df.DfProtectAll();
                }else{
                    Print(": ERROR  OpenFile failed");
                }
                Apb3.Close();
            }
        }
        
        
        //
        //  ボタン　UDA1345  出力選択
        //
        private void button_UDA1345_Click(object sender, EventArgs e)
        {
            if (Apb3.Open())
            {
                int lch = (int)util.ToInt32(textBox_UDA1345_Lch.Text, 16);   // 
                int rch = (int)util.ToInt32(textBox_UDA1345_Rch.Text, 16);   // 
                Print( "\n\nUDA1345  出力選択  Lch=" + lch.ToString("X1") + " Rch=" + rch.ToString("X1") );
                Apb3.uda1345_OutSelect( lch, rch );  // DAC 出力選択
                Apb3.Close();
            }
        }
        
        
        //
        //  ボタン　DAC Test  テストパターンとゼロを 12.5MHz で出力
        //
        private void button_DacTest_Click(object sender, EventArgs e)
        {
            if( Apb3.Open() ) {
                int test = (int)util.ToInt32(textBox_DacTest.Text);   // 
                Apb3.ad9707Write( (test+1)<<4 );  // テストモードでテストパターンを出力する
                Print("\n\nDAC test bit=" + (test).ToString());
                Apb3.Close();
            }
        }
        
        
        //
        //  画面文字列出力
        //
        private void Print(string str)
        {
            richTextBox1.SelectionStart = richTextBox1.Text.Length;
            richTextBox1.SelectedText = str;
            richTextBox1.ScrollToCaret();
        }
        
        
        private void DebugForm_Click(object sender, EventArgs e)
        {
            this.Focus();               // どこをクリックされてもフォーカスがくるように
        }
        
        
    }
}
