﻿//================================================================================
//  
//        FreqCounterForm.cs        周波数カウンター画面
//  
//        (c)2011/07/12  by OjisanKoubou
//  
//================================================================================
//  
//  このプログラムは Ojisankoubou( OGAWA Ichiro ) が著作権を持っています。
//  個人の研究目的には、修正、改造など自由に使用できます。
//  発表される際は著作権表示部分を消さないようにしてください。
//  このソフトウェアの全部もしくは一部を製品に組み込んだり販売したりするなどの
//  商業的利用はおやめください。
//

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading;
using System.Windows.Forms;

namespace APB_3
{
    public partial class FreqCounterForm : Form
    {
        private static int wn = 0;
        private Thread Tmeasure;
        private bool measuring = false;

        private int source = 0;
        private double interval = 0;
        private bool seq = true;
        private bool showtime = true;
        //private bool showdetail = true;
        private bool zerofreqmode = false;
        
        private double freq = 0;
        private double zerofreq = 0;
        private string freqstr = "";

        private delegate void DelegateType();     // 別スレッドのコントロールを操作するのに使う


        public FreqCounterForm()
        {
            InitializeComponent();
            wn++;
            this.Text = "周波数カウンター " + wn.ToString();
            richTextBox1.Font = new Font("ＭＳ ゴシック", richTextBox1.SelectionFont.Size); // 等幅フォント
            comboBox_input.Items.AddRange(def.FCinputstrtbl);
            comboBox_interval.Items.AddRange(def.FCintervalstrtbl);

            comboBox_input.SelectedIndex = 8;       // AD
            comboBox_interval.SelectedIndex = 0;    // 0.5s
        }

        //
        //
        //
        private void button_startstop_Click(object sender, EventArgs e)
        {
            if (measuring)
            {
                measuring = false;  // 中断
            }
            else
            {
                richTextBox1.SelectedText = "\n";
                setting();
                Tmeasure = new Thread(new ThreadStart(tMeasure));
                Tmeasure.IsBackground = true;
                Tmeasure.Start();                 // 測定スレッドを開始する
            }
        }

        //
        //
        //
        private void button_zerofreq_Click(object sender, EventArgs e)
        {
            zerofreqmode = !zerofreqmode;
            button_zerofreq.Text = zerofreqmode? def.zerofreq_x:def.zerofreq;
            label_zerofreq.Text  = "ZERO = "+freqstr;
            label_zerofreq.Visible = zerofreqmode;
            zerofreq = freq;
        }


        private void comboBox_interval_SelectedIndexChanged(object sender, EventArgs e)
        {
            interval = def.FCintervaltbl[comboBox_interval.SelectedIndex];
        }


        private void checkBox_seq_CheckedChanged(object sender, EventArgs e)
        {
            seq       = checkBox_seq.Checked;
        }


        private void checkBox_time_CheckedChanged(object sender, EventArgs e)
        {
            showtime  = checkBox_time.Checked;
        }


        // 設定
        private void setting()
        {
            source    = comboBox_input.SelectedIndex;
        }



        // 測定スレッド
        private void tMeasure()
        {
            if (Apb3.Open())
            {
                try
                {
                    measuring = true;
                    btnStartText(def.stop, clr.MeasBtnExec);
                    
                    int mescnt = FreqCounter.Mescnt(source, 0.2);
                    if (mescnt < 0) {
                        if( mescnt == error.usb_disconnect ) {
                            Apb3.mainf.LogPrint("\ntMeasure: usb disconnect");
                            Apb3.CloseConnection();     // 一度クローズする
                        }else{                          //タイムアウト
                            Apb3.mainf.LogPrint("\ntMeasure: timeout_1");
                        }
                        error.ErrorMessageBox( mescnt );  // 測定中にエラー
                        return;
                    }
                    
                    FreqCounter.Start(source, mescnt, 1);  // 分周設定して本番の連続周波数測定 
                    Stopwatch sw = new Stopwatch();
                    sw.Start();
                    double targettime = interval;
                    
                    bool update = false;
                    
                    while (measuring) // 測定開始・中止ボタンを押されるまで
                    {
                        freq = FreqCounter.SeqMeasure( update );
                        if (freq < 0) {                     // エラー
                            if( (int)freq == error.usb_disconnect ) {
                                Apb3.mainf.LogPrint("\ntMeasure: usb disconnect");
                                Apb3.CloseConnection();     // 一度クローズする
                            }else{                          //タイムアウト
                                Apb3.mainf.LogPrint("\ntMeasure: timeout_1");
                            }
                            error.ErrorMessageBox( (int)freq );  // 測定中にエラー
                            return;
                        }
                        
                        if( zerofreqmode ) freq -= zerofreq;
                        
                        double elapsed = (double)sw.ElapsedTicks / (double)Stopwatch.Frequency;
                        if( elapsed > targettime ) {
                            if( !update ) {
                                update = true;
                                continue;
                            }
                            else
                            {
                                update = false;
                                targettime += interval;
                                
                                if      (freq >= 2e7) freqstr = freq.ToString("#0 000 000.0 Hz");
                                else if (freq >= 2e6) freqstr = freq.ToString("# 000 000.00 Hz");
                                else if (freq >= 2e5) freqstr = freq.ToString("#00 000.000 Hz");
                                else if (freq >= 2e4) freqstr = freq.ToString("#0 000.0000 Hz");
                                else if (freq >= 2e3) freqstr = freq.ToString("# 000.00000 Hz");
                                else if (freq >= 2e2) freqstr = freq.ToString("#00.000000 Hz");
                                else if (freq >= 2e1) freqstr = freq.ToString("#0.0000000 Hz");
                                else                  freqstr = freq.ToString("#0.00000000 Hz");
                                
                                string str = freq.ToString()+" ";
                                if(showtime) str += elapsed.ToString("#.000")+" ";
                                
                                try
                                {
                                    Invoke((DelegateType)delegate
                                    {
                                        textBox_freq.Text = freqstr.PadLeft(19);
                                        richTextBox1.SelectedText = str +"\n";
                                        richTextBox1.ScrollToCaret();
                                    });
                                }
                                catch
                                {
                                    
                                }
                                
                                if( !seq ) return;
                            }
                        }
                        else
                        {
                            Thread.Sleep(100);
                        }
                    }
                }
                finally
                {
                    measuring = false;
                    btnStartText(def.measure, clr.MeasBtnNor);
                    Apb3.Close();
                }
            }
        }



        // 測定開始ボタンのテキストを変更
        public void btnStartText(string str, Color clr)
        {
            Invoke((DelegateType)delegate
            {
                button_startstop.Text = str;
                button_startstop.BackColor = clr;
            });
        }


        //
        //
        //
        private void Print(string str)
        {
            richTextBox1.SelectedText = str;
            richTextBox1.ScrollToCaret();
        }


        // マウスボタンが押された
        protected void mouseDown(object sender, MouseEventArgs e) {
            this.Focus();                       // どこをクリックされてもフォーカスがくるように
        }
        
        
    }
}
