﻿//================================================================================
//  
//        FreqResponseGraphWave.cs      周波数特性グラフの波形クラス
//  
//        (c)2009/07/26  by OjisanKoubou
//  
//================================================================================
//  
//  このプログラムは Ojisankoubou( OGAWA Ichiro ) が著作権を持っています。
//  個人の研究目的には、修正、改造など自由に使用できます。
//  発表される際は著作権表示部分を消さないようにしてください。
//  このソフトウェアの全部もしくは一部を製品に組み込んだり販売したりするなどの
//  商業的利用はできません。
//

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;
using System.Drawing;


namespace APB_3 {
    
    //-------------------------------------------------------------------------
    // FreqResponseGraphWave　波形クラス（表示する波形１個分のデータ）
    //-------------------------------------------------------------------------
    public class FreqResponseGraphWave {
        public int nsweep = 0;                      // 何スイープ分のデータが入っているか
        public ArrayList org = new ArrayList();     // オリジナル測定結果
        public ArrayList cal = new ArrayList();     // 校正用リファレンス
        public ArrayList wave = new ArrayList();    // 校正後（ オリジナル － リファレンス ）
        public double offset = 0;                   // 0dB オフセット
        public int scaleno = 0;                     // スケール番号 0:左側　1:右側
        public Pen pen = new Pen(clr.getcolor());   // 波形表示ペン
        public bool isphase = false;                // 位相データ？
        public bool visible = true;                 // 表示する・しない
        
        
        // コンストラクター
        public FreqResponseGraphWave() {
        }
        
        public FreqResponseGraphWave(int scaleno) {
            this.scaleno = scaleno;
        }
        
        public FreqResponseGraphWave(int scaleno, bool isphase) {
            this.scaleno = scaleno;
            this.isphase = isphase;
        }
        
        
        // リファレンス補正をやり直す（リファレンスを変更したあとに実行する）
        public void ApplyRef() {
            wave.Clear();
            foreach (FreqResponseGraphData frs in org) {
                Hosei(frs.freq, frs.data);
            }
        }
        
        
        // 新しいデータを追加（リファレンス補正もする）
        public void Add(double freq, double data) {
            org.Add(new FreqResponseGraphData(freq, data));
            Hosei(freq, data);
        }
        
        
        // リファレンス補正して追加
        private void Hosei(double freq, double data) {
            double data1 = data;
            double f1 = -1, d1 = 0;
            double f2 = -1, d2 = 0;
            foreach (FreqResponseGraphData frd in cal) {
                if (frd.freq <= freq) {
                    f1 = frd.freq;
                    d1 = frd.data;
                }
                else if (f2 < 0) {
                    f2 = frd.freq;
                    d2 = frd.data;
                }
                else {
                    break;
                }
            }
            if ((f1 > 0) && (f2 < 0)) {
                f2 = 9999e6;
                d2 = d1;
            }
            else if ((f1 < 0) && (f2 != 0)) {
                f1 = 0;
                d1 = d2;
            }
            data1 = data - (d2 - d1) / (f2 - f1) * (freq - f1) - d1 - offset;
            if( isphase ) {
                while (data1 > 180) data1 -= 360;   // 位相の場合の補正。
                while (data1 < -180) data1 += 360;
            }
            wave.Add(new FreqResponseGraphData(freq, data1));
        }
        
        
        // ntrace 分を残して波形を削除
        public void DelOldTraces( int ntrace ) {
            while( ntrace < nsweep ) Del1stTrace();
            System.GC.Collect();
        }
        
        
        // 1番最初の波形を削除
        public void Del1stTrace() {
            if( wave.Count == 0 ) {
                nsweep=0;
                return;
            }
            int count=1;
            double f1 = ((FreqResponseGraphData)(wave[0])).freq;
            for( ; count< wave.Count; count++ ) {           // 次のトレースまでのデータ個数
                double f2 = ((FreqResponseGraphData)(wave[count])).freq;
                if( f1 > f2 ) break;
                f1 = f2;
            }
            if( count == wave.Count ) {
                nsweep = 0;
            } else {
                nsweep--;
            }
            wave.RemoveRange( 0, count );
            org.RemoveRange( 0, count );
        }
        
        
        // 実際のトレース数を調べて nsweep 数との整合性を取る
        public void tracecount() {
            nsweep = 0;
            if( wave.Count == 0 ) return;
            nsweep = 1;
            int count=1;
            double f1 = ((FreqResponseGraphData)(wave[0])).freq;
            for( ; count< wave.Count; count++ ) {           // 次のトレースまでのデータ個数
                double f2 = ((FreqResponseGraphData)(wave[count])).freq;
                if( f1 > f2 ) nsweep++;
                f1 = f2;
            }
        }
        
        
        // 波形クリア
        public void WaveClear() {
            wave.Clear();
            org.Clear();
            nsweep = 0;
            System.GC.Collect(2);
        }
        
        
        // リファレンスをクリア
        public void CalClear() {
            cal.Clear();
            ApplyRef();
        }
        
        
        // 現測定値をリファレンスにする
        public void ToCal() {
            cal = new ArrayList();
            cal = (ArrayList)(org.Clone());
            if( cal.Count == 0 ) return;
            
            int count=1;
            double f1 = ((FreqResponseGraphData)(cal[0])).freq;
            int n = nsweep;
            while( n>1 ) {
                for( ; count< cal.Count; count++ ) {           // 次のトレースまでのデータ個数
                    double f2 = ((FreqResponseGraphData)(cal[count])).freq;
                    if( f1 > f2 ) {
                        n--;
                        break;
                    }
                    f1 = f2;
                }
            }
            cal.RemoveRange( 0, count ); // 最新の１トレースを残して削除
            
            if( isphase ) {
                double prev = ((FreqResponseGraphData)(cal[0])).data;
                for (int i = 1; i < cal.Count; i++) {
                    double diff = ((FreqResponseGraphData)(cal[i])).data - prev;
                    prev = ((FreqResponseGraphData)(cal[i])).data;
                    if (diff < -180) {
                        diff += 360;
                    }
                    else if (diff > 180) {
                        diff -= 360;
                    }
                    ((FreqResponseGraphData)(cal[i])).data = ((FreqResponseGraphData)(cal[i - 1])).data + diff;
                }
            }
            ApplyRef();
        }
        
        
        // オフセットを設定
        public void SetOffset(double offset) {
            this.offset = offset;
            ApplyRef();
        }
        
        
        // 色を設定
        public void SetColor(Color color) {
            pen.Color = color;
        }
        
        
        // 色を返す
        public Color GetColor() {
            return pen.Color;
        }
        
        
        // スイープ数を１増やす
        public void IncNsweep() {
            nsweep++;
        }
        
        
        // スイープ数を返す
        public int GetNsweep() {
            return nsweep;
        }
        
        
    }

}

