﻿//================================================================================
//  
//        HfSGForm.cs           高周波信号発生器画面
//  
//        (c)2009/07/26  by OjisanKoubou
//  
//================================================================================
//  
//  このプログラムは Ojisankoubou( OGAWA Ichiro ) が著作権を持っています。
//  個人の研究目的には、修正、改造など自由に使用できます。
//  発表される際は著作権表示部分を消さないようにしてください。
//  このソフトウェアの全部もしくは一部を製品に組み込んだり販売したりするなど商業的利用はおやめください。
//

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace APB_3 {
    public partial class HfSGForm : Form {
        private static int wn=0;
        private MainForm mainf;
        private bool initializing=true;
        
        public HfSGForm() {
            wn++;
            mainf = Apb3.mainf;
            
            InitializeComponent();
            
            this.Text = "信号発生器 " + wn.ToString();
            
            groupBox1.MouseDown += new System.Windows.Forms.MouseEventHandler(this.mouseDown);
            groupBox2.MouseDown += new System.Windows.Forms.MouseEventHandler(this.mouseDown);
            groupBox3.MouseDown += new System.Windows.Forms.MouseEventHandler(this.mouseDown);

            comboBox_freq.Items.AddRange(def.hfsgfreqstrtbl);
            comboBox_freqstep.Items.AddRange(def.hfsgfreqstepstrtbl);
            comboBox_level.Items.AddRange(def.hfsglevelstrtbl);
            comboBox_levelstep.Items.AddRange(def.hfsglevelstepstrtbl);
            comboBox_ammodlevel.Items.AddRange(def.hfsgmodlevelamstrtbl);
            comboBox_ammodfreq.Items.AddRange(def.audiosgfreqstrtbl);
            comboBox_amwave.Items.AddRange(def.audiosgwavestrtbl);
            comboBox_fmdeviation.Items.AddRange(def.hfsgmodlevelfmstrtbl);
            comboBox_fmmodfreq.Items.AddRange(def.audiosgfreqstrtbl);
            comboBox_fmwave.Items.AddRange(def.audiosgwavestrtbl);

            comboBox_freq.SelectedIndex         = 2;        // 10MHz
            comboBox_freqstep.SelectedIndex     = 3;        // 1MHz
            comboBox_level.SelectedIndex        = 0;        // 0dB
            comboBox_levelstep.SelectedIndex    = 3;        // 1dB
            checkBox_dsb.Checked                = false;    // 無変調
            checkBox_am.Checked                 = false;    // 無変調
            comboBox_ammodlevel.SelectedIndex   = 0;        // 100%
            comboBox_ammodfreq.SelectedIndex    = 12;       // 1kHz
            comboBox_amwave.SelectedIndex       = 0;        // サイン波
            checkBox_fm.Checked                 = false;    // 無変調
            comboBox_fmdeviation.SelectedIndex  = 9;        // ±49 kHz
            comboBox_fmmodfreq.SelectedIndex    = 12;       // 1kHz
            comboBox_fmwave.SelectedIndex       = 0;        // サイン波
            
            setting();
            initializing = false;
            
        }
        
        
        private void button_finish_Click(object sender, EventArgs e) {
            Close();
        }
        
        
        // マウスボタンが押された
        protected void mouseDown(object sender, MouseEventArgs e) {
            this.Focus();                       // どこをクリックされてもフォーカスがくるように
//            setting();
        }
        
        
        // 設定を変更した
        private void setting_Changed(object sender, EventArgs e) {
            mainf.LogPrint("\nHfSGForm.SettingChanged");
            if( !initializing ) setting();
        }
        
        private void setting() {
            if( Apb3.Open() ) {
                double freq = util.ToDouble(comboBox_freq.Text);
                Apb3.ad9707Write( 3 );  // DA900 DDS1 を出力する
                Apb3.dds1( freq );
                int level = (int)(Math.Pow(10, (util.ToDouble(comboBox_level.Text))/20) * 65535 + 0.5);
mainf.LogPrint("\nlevel=" + level.ToString());
                level = level<<16;
                int am = 0;
                label_levelwarning.Visible = (checkBox_am.Checked || checkBox_dsb.Checked);
                if( checkBox_am.Checked ) {
                    am = (comboBox_ammodlevel.SelectedIndex+1)<<8;
                    Apb3.dds2( util.ToDouble(comboBox_ammodfreq.Text) );
                    Apb3.dds_setting(2, (comboBox_amwave.SelectedIndex));
                }
                int dsb = 0;
                if (checkBox_dsb.Checked) {
                    dsb = 1 << 2;
                    am = (comboBox_ammodlevel.SelectedIndex) << 8;
                    Apb3.dds2( util.ToDouble(comboBox_ammodfreq.Text));
                    Apb3.dds_setting(2, (comboBox_amwave.SelectedIndex));
                } 
                int fm = 0;
                if( checkBox_fm.Checked ) {
                    fm = (comboBox_fmdeviation.SelectedIndex + 1)<<12;
                    Apb3.dds3( util.ToDouble(comboBox_fmmodfreq.Text) );
                    Apb3.dds_setting(3, (comboBox_fmwave.SelectedIndex));
                }

                Apb3.dds_setting( 1, level+fm+am+dsb ); //
                
                Apb3.Close();
            }
        }
        
        
        // チェックボックスAM変調を変更した
        private void checkBox_am_Changed(object sender, EventArgs e) {
            if( checkBox_am.Checked ) checkBox_dsb.Checked = false;
            setting();
        }
        
        
        // チェックボックスDSB変調を変更した
        private void checkBox_dsb_Changed(object sender, EventArgs e) {
            if (checkBox_dsb.Checked) checkBox_am.Checked = false;
            setting();
        }
        
        
        // 周波数マイナス
        private void button_freqminus_Click(object sender, EventArgs e) {
            double freq = util.ToDouble(comboBox_freq.Text);
            double fstep = util.ToDouble(comboBox_freqstep.Text);
            freq -= fstep;
            if (freq <= 0) freq += fstep;
            comboBox_freq.Text = util.Double2String(freq);
        }
        
        
        // 周波数プラス
        private void button_freqplus_Click(object sender, EventArgs e) {
            double freq = util.ToDouble(comboBox_freq.Text);
            double fstep = util.ToDouble(comboBox_freqstep.Text);
            freq += fstep;
            if (freq >= Apb3.nyquistfreq) freq -= fstep;
            comboBox_freq.Text = util.Double2String(freq);
        }
        
        
        // レベルマイナス
        private void button_levelminus_Click(object sender, EventArgs e)
        {
            double level = util.ToDouble(comboBox_level.Text);
            double levelstep = util.ToDouble(comboBox_levelstep.Text);
            level -= levelstep;
            if (level < -60) level += levelstep;
            comboBox_level.Text = util.Double2String(level);
        }
        
        
        // レベルプラス
        private void button_levelplus_Click(object sender, EventArgs e)
        {
            double level = util.ToDouble(comboBox_level.Text);
            double levelstep = util.ToDouble(comboBox_levelstep.Text);
            level += levelstep;
            if (level > 0) level -= levelstep;
            comboBox_level.Text = util.Double2String(level);
        }
        
        
        private void HfSGForm_Click(object sender, EventArgs e) {
            this.Focus();               // どこをクリックされてもフォーカスがくるように
        }
        
    }
}
