﻿//================================================================================
//  
//        ImpedanceView.cs         インピーダンスアナライザ画面
//  
//        (c)2009/10/02  by OjisanKoubou
//  
//================================================================================
//  
//  このプログラムは Ojisankoubou( OGAWA Ichiro ) が著作権を持っています。
//  個人の研究目的には、修正、改造など自由に使用できます。
//  発表される際は著作権表示部分を消さないようにしてください。
//  このソフトウェアの全部もしくは一部を製品に組み込んだり販売したりするなどの
//  商業的利用はできません。
//

using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.IO.Ports;
using System.Linq;
using System.Text;
using System.Threading;
using System.Windows.Forms;

namespace APB_3 {
    //-------------------------------------------------------------------------
    //  インピーダンスアナライザ　view(表示)部分
    //-------------------------------------------------------------------------
    public partial class ImpedanceView : FreqResponseGraphForm {
        private static int wn = 0;          // ウィンドウ番号
        private Thread Tmeasure;
        private volatile bool measuring = false;
        private volatile bool stopmeasuring = false;
        private volatile bool errormeasuring = false;
        private int averaging = 1;          // 平均化回数
        private bool keepresult = false;    // 測定結果を残すか
        private int ntrace = 1;         // トレース保存数
        private int leftaxis = 0;           // 左側軸表示項目
        private int rightaxis = 1;          // 右側軸表示項目
        private double fstart = 0;          // 測定スレッドで使う変数
        private double fstop  = 0;       // 
        private double fstep  = 1;          // 
        private int rbw = 0;                // 
        private int npoint = 1;             // 測定ポイント数
        private bool singlesweep = true;
        private int outlevel = (def.hfsgleveltbl[0])<<16; // 出力レベル 0dB
        
        private ArrayList org      = new ArrayList();   // オリジナル測定結果
        private ArrayList calshort = new ArrayList();   // ショートキャリブレーションデータ
        private ArrayList calopen  = new ArrayList();   // オープンキャリブレーションデータ
        private bool calshortdone  = false;             // ショートキャリブレーション実行済み
        private bool calopendone   = false;             // オープンキャリブレーション実行済み
        
        private const int calmodeopen  = 1;     // キャリブレーションモード
        private const int calmodeshort = 2;     // 
        private const int calmode50    = 3;     // 
        private const int calmodeinit  = 0;     // 初期値設定モード
        
        private const int nmaxwave  = 5;        // 波形最大数
        
        private  Byte[] rdBuffer      = new Byte[Apb3.buffsize];
        private  Byte[] rdBuffer_d32  = new Byte[Apb3.buffsize];
        
        private delegate void DelegateType();   // 別スレッドのコントロールを操作するのに使う
        
        
        
        // コンストラクター
        public ImpedanceView() {
            mainf = Apb3.mainf;
            mainf.LogPrint( "\nImpedanceView.ImpedanceView" );
            InitializeComponent();
            Initialize();
            ReSize();
        }
        
        
        // 初期化
        private void Initialize() {
            wn++;
            base.py = (int)(28 * (MainForm.dpi/96));
            this.Text = "インピーダンスアナライザ " + wn.ToString();
            this.BackColor = clr.IAVPictBG;
            comboBox_freqspan.Items.AddRange(def.freqspanstrtbl);
            comboBox_n.Items.AddRange(def.nstrtbl);
            comboBox_rbw.Items.AddRange(def.rbwstrtbl_netana);
            comboBox_n.Visible = true;
            comboBox_rbw.SelectedIndex = 5;  // 1kHz
            
            ((ToolStripMenuItem)(ToolStripMenuItem_displayitems.DropDownItems[1])).Checked = true;  // 測定日時表示
            ((ToolStripMenuItem)(ToolStripMenuItem_outlevel.DropDownItems[0])).Checked = true;
            ((ToolStripMenuItem)(ToolStripMenuItem_persistence.DropDownItems[8])).Checked = true;   // 50%
            ((ToolStripMenuItem)(ToolStripMenuItem_trace.DropDownItems[3])).Checked = true;         // 10
            ToolStripMenuItem_sweep.Checked = singlesweep;
            ((ToolStripMenuItem)(ToolStripMenuItem_div.DropDownItems[2])).Checked = true;
            ((ToolStripMenuItem)(ToolStripMenuItem_reref.DropDownItems[11])).Checked = true;         // 100
            ((ToolStripMenuItem)(ToolStripMenuItem_rediv.DropDownItems[11])).Checked = true;         // 10
            ((ToolStripMenuItem)(ToolStripMenuItem_imref.DropDownItems[11])).Checked = true;         // 100
            ((ToolStripMenuItem)(ToolStripMenuItem_imdiv.DropDownItems[11])).Checked = true;         // 10
            ((ToolStripMenuItem)(ToolStripMenuItem_absref.DropDownItems[11])).Checked = true;        // 100
            ((ToolStripMenuItem)(ToolStripMenuItem_absdiv.DropDownItems[11])).Checked = true;        // 10
            ((ToolStripMenuItem)(ToolStripMenuItem_pharef.DropDownItems[0])).Checked = true;        // 225
            ((ToolStripMenuItem)(ToolStripMenuItem_phadiv.DropDownItems[0])).Checked = true;        // 45
            ((ToolStripMenuItem)(ToolStripMenuItem_lref.DropDownItems[9])).Checked = true;          // 10u
            ((ToolStripMenuItem)(ToolStripMenuItem_ldiv.DropDownItems[9])).Checked = true;          // 1u
            ((ToolStripMenuItem)(ToolStripMenuItem_cref.DropDownItems[17])).Checked = true;          // 100p
            ((ToolStripMenuItem)(ToolStripMenuItem_cdiv.DropDownItems[17])).Checked = true;          // 10p
            
            wavelist.Add(new FreqResponseGraphWave(0));         // scale=0:Real
            wavelist.Add(new FreqResponseGraphWave(1));         // scale=1:Imag
            wavelist.Add(new FreqResponseGraphWave(2));         // scale=2:ABS
            wavelist.Add(new FreqResponseGraphWave(3, true));   // scale=3:PHA
            wavelist.Add(new FreqResponseGraphWave(4));         // scale=4:L
            wavelist.Add(new FreqResponseGraphWave(5));         // scale=5:C
            ((FreqResponseGraphWave)(base.wavelist[2])).visible = false;
            ((FreqResponseGraphWave)(base.wavelist[3])).visible = false;
            ((FreqResponseGraphWave)(base.wavelist[4])).visible = false;
            ((FreqResponseGraphWave)(base.wavelist[5])).visible = false;
            base.scale.Add(new FreqResponseGraphScale(100, 10, "Real", "Ω", 0));
            base.scale.Add(new FreqResponseGraphScale(100, 10, "Imag", "Ωj", 1));
            base.scale.Add(new FreqResponseGraphScale(100, 10, "ABS", "Ωabs", 1));
            base.scale.Add(new FreqResponseGraphScale(225, 45, "PHA", "deg", 1));
            base.scale.Add(new FreqResponseGraphScale(10e-6, 1e-6, "L", "H", 1));
            base.scale.Add(new FreqResponseGraphScale(100e-12, 10e-12, "C", "F", 1)); 
            ((FreqResponseGraphScale)(base.scale[2])).visible = false;
            ((FreqResponseGraphScale)(base.scale[3])).visible = false;
            ((FreqResponseGraphScale)(base.scale[4])).visible = false;
            ((FreqResponseGraphScale)(base.scale[5])).visible = false;
            cursor.Add(new FreqResponseGraphCursor((FreqResponseGraphWave)(wavelist[0])));
            cursor.Add(new FreqResponseGraphCursor((FreqResponseGraphWave)(wavelist[1])));
            curcursor = ((FreqResponseGraphCursor)cursor[0]);
            ((FreqResponseGraphCursor)cursor[0]).Highlight();
            ((FreqResponseGraphCursor)cursor[0]).pos = 1;
            deltaf = false; // カーソルのデルタ表示はしない
            // 波形初期値
            for (int i = 0; i < 500; i++) {
                ((FreqResponseGraphWave)wavelist[0]).Add(i * 1e6 / 10, 0);
                ((FreqResponseGraphWave)wavelist[1]).Add(i * 1e6 / 10, 0);
                ((FreqResponseGraphWave)wavelist[2]).Add(i * 1e6 / 10, 0);
                ((FreqResponseGraphWave)wavelist[3]).Add(i * 1e6 / 10, 0);
                ((FreqResponseGraphWave)wavelist[4]).Add(i * 1e6 / 10, 0);
                ((FreqResponseGraphWave)wavelist[5]).Add(i * 1e6 / 10, 0);
            }
            foreach (FreqResponseGraphWave frw in wavelist) frw.IncNsweep();  // スイープ数を１増やす
            setcaldata( calmodeinit );
        }
        
        
        // コンテクストメニューをマウスカーソル位置によって切り替える
        private void contextMenuStrip1_Opening(object sender, CancelEventArgs e) {
            contextMenuStrip1.Items.Clear();
            if (isFreqArea() ) { freq_contextMenu();      return; } // 周波数表示部
            if (isLeftArea() ) { leftaxis_contextMenu();  return; } // 左軸
            if (isRightArea()) { rightaxis_contextMenu(); return; } // 右軸
            if (isGraphArea()) { graph_contextMenu();     return; } // グラフ表示部分
            top_contextMenu();                                      // 画面上部
        }
        
        
        // 周波数表示部コンテクストメニュー項目
        private void freq_contextMenu() {
            this.contextMenuStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
                this.ToolStripMenuItem_fullspan,
                this.ToolStripMenuItem_freqcenter,
                this.ToolStripMenuItem_freqaxis,
            });
        }
        
        
        // 左軸コンテクストメニュー項目
        private void leftaxis_contextMenu() {
            switch (leftaxis) {
                case 0:
                    this.contextMenuStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
                        this.ToolStripMenuItem_reref,
                        this.ToolStripMenuItem_rediv,
                        new ToolStripSeparator(),
                    });
                    break;
                case 1:
                    this.contextMenuStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
                        this.ToolStripMenuItem_imref,
                        this.ToolStripMenuItem_imdiv,
                        new ToolStripSeparator(),
                    });
                    break;
                case 2:
                    this.contextMenuStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
                        this.ToolStripMenuItem_absref,
                        this.ToolStripMenuItem_absdiv,
                        new ToolStripSeparator(),
                    });
                    break;
                case 3:
                    this.contextMenuStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
                        this.ToolStripMenuItem_pharef,
                        this.ToolStripMenuItem_phadiv,
                        new ToolStripSeparator(),
                    });
                    break;
                case 4:
                    this.contextMenuStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
                        this.ToolStripMenuItem_lref,
                        this.ToolStripMenuItem_ldiv,
                        new ToolStripSeparator(),
                    });
                    break;
                case 5:
                    this.contextMenuStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
                        this.ToolStripMenuItem_cref,
                        this.ToolStripMenuItem_cdiv,
                        new ToolStripSeparator(),
                    });
                    break;
            }
            this.contextMenuStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
                this.ToolStripMenuItem_axis,
                new ToolStripSeparator(),
                this.ToolStripMenuItem_color,
                this.ToolStripMenuItem_div,
            });
            return;
        }
        
        
        // 右軸コンテクストメニュー項目
        private void rightaxis_contextMenu() {
            switch (rightaxis) {
                case 0:
                    this.contextMenuStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
                        this.ToolStripMenuItem_reref,
                        this.ToolStripMenuItem_rediv,
                        new ToolStripSeparator(),
                    });
                    break;
                case 1:
                    this.contextMenuStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
                        this.ToolStripMenuItem_imref,
                        this.ToolStripMenuItem_imdiv,
                        new ToolStripSeparator(),
                    });
                    break;
                case 2:
                    this.contextMenuStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
                        this.ToolStripMenuItem_absref,
                        this.ToolStripMenuItem_absdiv,
                        new ToolStripSeparator(),
                    });
                    break;
                case 3:
                    this.contextMenuStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
                        this.ToolStripMenuItem_pharef,
                        this.ToolStripMenuItem_phadiv,
                        new ToolStripSeparator(),
                    });
                    break;
                case 4:
                    this.contextMenuStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
                        this.ToolStripMenuItem_lref,
                        this.ToolStripMenuItem_ldiv,
                        new ToolStripSeparator(),
                    });
                    break;
                case 5:
                    this.contextMenuStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
                        this.ToolStripMenuItem_cref,
                        this.ToolStripMenuItem_cdiv,
                        new ToolStripSeparator(),
                    });
                    break;
            }
            this.contextMenuStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
                this.ToolStripMenuItem_axis,
                new ToolStripSeparator(),
                this.ToolStripMenuItem_color,
                this.ToolStripMenuItem_div,
            });
            return;
        }
        
        
        // グラフ表示部分コンテクストメニュー項目
        private void graph_contextMenu() {
            this.contextMenuStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
                ToolStripMenuItem_eraseold,
                ToolStripMenuItem_eraseall,
                new ToolStripSeparator(),
                ToolStripMenuItem_sweep,
                ToolStripMenuItem_trace,
                ToolStripMenuItem_persistence,
                ToolStripMenuItem_outlevel,
                new ToolStripSeparator(),
                this.ToolStripMenuItem_mestocalshort,
                this.ToolStripMenuItem_mestocalopen,
                new ToolStripSeparator(),
                this.ToolStripMenuItem_nozoom,
                this.ToolStripMenuItem_zoom,
                this.ToolStripMenuItem_cursor2freqcenter,
                new ToolStripSeparator(),
            });
            if( ((FreqResponseGraphCursor)(base.cursor[0])).wave == ((FreqResponseGraphCursor)(base.cursor[1])).wave ) {  // 同じ wave のときはデルタ表示可
                contextMenuStrip1.Items.Add( ToolStripMenuItem_delta );
            }
            this.contextMenuStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
                ToolStripMenuItem_cursorline,
                ToolStripMenuItem_cursorcross,
                ToolStripMenuItem_showvaluenearcur,
                new ToolStripSeparator(),
                ToolStripMenuItem_clipboard,
                new ToolStripSeparator(),
                ToolStripMenuItem_displayitems,
                ToolStripMenuItem_backcolor,
            });
            return;
        }
        
        
        // 画面上部コンテクストメニュー項目
        private void top_contextMenu() {
            this.contextMenuStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
                ToolStripMenuItem_eraseold,
                ToolStripMenuItem_eraseall,
                new ToolStripSeparator(),
                ToolStripMenuItem_sweep,
                ToolStripMenuItem_trace,
                ToolStripMenuItem_persistence,
                ToolStripMenuItem_outlevel,
                new ToolStripSeparator(),
                this.ToolStripMenuItem_mestocalshort,
                this.ToolStripMenuItem_mestocalopen,
                new ToolStripSeparator(),
                ToolStripMenuItem_clipboard,
                new ToolStripSeparator(),
                ToolStripMenuItem_displayitems,
            });
        }
        
        
        // コンテクストメニュー　軸表示項目
        private void ToolStripMenuItem_axis_Click(object sender, EventArgs e) {
            int index = ToolStripMenuItem_axis.DropDownItems.IndexOf(((ToolStripMenuItem)sender));
            mainf.LogPrint("\nImpedanceView.ToolStripMenuItem_axis_Click: " + index.ToString());
            if (isLeftArea()) {
                if (leftaxis <= nmaxwave) {
                    ((FreqResponseGraphWave)wavelist[leftaxis]).visible = false;
                    ((FreqResponseGraphScale)scale[leftaxis]).visible = false;
                }
                if (index <= nmaxwave) {
                    ((FreqResponseGraphWave)wavelist[index]).visible = true;
                    ((FreqResponseGraphScale)scale[index]).visible = true;
                    ((FreqResponseGraphScale)scale[index]).pos = 0;  // left axis
                    ((FreqResponseGraphCursor)cursor[0]).wave = ((FreqResponseGraphWave)wavelist[index]);
                }
                if (rightaxis == index) {   // 右軸が左軸と同じなら右軸は
                    rightaxis = nmaxwave+1;          // 表示なしにする。
                }
                leftaxis = index;
            } else if (isRightArea()) {
                if (rightaxis <= nmaxwave) {
                    ((FreqResponseGraphWave)wavelist[rightaxis]).visible = false;
                    ((FreqResponseGraphScale)scale[rightaxis]).visible = false;
                }
                if (index <= nmaxwave) {
                    ((FreqResponseGraphWave)wavelist[index]).visible = true;
                    ((FreqResponseGraphScale)scale[index]).visible = true;
                    ((FreqResponseGraphScale)scale[index]).pos = 1;  // right axis
                    ((FreqResponseGraphCursor)cursor[1]).wave = ((FreqResponseGraphWave)wavelist[index]);
                }
                if (leftaxis == index) {
                    leftaxis = nmaxwave+1; // 非表示
                }
                rightaxis = index;
            }
            if ((rightaxis == nmaxwave+1) && (leftaxis <= nmaxwave)) {
                ((FreqResponseGraphCursor)cursor[0]).wave = ((FreqResponseGraphWave)wavelist[leftaxis]);
                ((FreqResponseGraphCursor)cursor[1]).wave = ((FreqResponseGraphWave)wavelist[leftaxis]);
            }
            if ((leftaxis == nmaxwave+1) && (rightaxis <= nmaxwave)) {
                ((FreqResponseGraphCursor)cursor[0]).wave = ((FreqResponseGraphWave)wavelist[rightaxis]);
                ((FreqResponseGraphCursor)cursor[1]).wave = ((FreqResponseGraphWave)wavelist[rightaxis]);
            }
            if ((leftaxis != nmaxwave+1) && (rightaxis != nmaxwave+1)) {
                deltaf = false; // カーソルのデルタ表示はしない
            }
            ReSize();
        }
        
        
        // コンテクストメニュー　実数部　Ref
        private void ToolStripMenuItem_reref_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nImpedanceView.ToolStripMenuItem_reref_Click:" );
            foreach (ToolStripMenuItem item in ToolStripMenuItem_reref.DropDownItems) {
              item.Checked = (item.Equals(sender))?true :false;
            }
            foreach (ToolStripMenuItem item in ToolStripMenuItem_rediv.DropDownItems) {
              item.Checked = false;
            }
            int index = ToolStripMenuItem_reref.DropDownItems.IndexOf(((ToolStripMenuItem)sender));
            ((ToolStripMenuItem)(ToolStripMenuItem_rediv.DropDownItems[index])).Checked = true;
            ((FreqResponseGraphScale)scale[0]).top = util.ToDouble(((ToolStripMenuItem)sender).Text);
            ((FreqResponseGraphScale)scale[0]).perdiv = util.ToDouble(((ToolStripMenuItem)sender).Text)/10;
            UpdateBitmap();
        }
        
        
        // コンテクストメニュー　実数部　/div
        private void ToolStripMenuItem_rediv_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nImpedanceView.ToolStripMenuItem_rediv_Click:" );
            foreach (ToolStripMenuItem item in ToolStripMenuItem_rediv.DropDownItems) {
              item.Checked = (item.Equals(sender))?true :false;
            }
            ((FreqResponseGraphScale)scale[0]).perdiv = util.ToDouble(((ToolStripMenuItem)sender).Text);
            UpdateBitmap();
        }
        
        
        // コンテクストメニュー　虚数部　Ref
        private void ToolStripMenuItem_imref_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nImpedanceView.ToolStripMenuItem_imref_Click:" );
            foreach (ToolStripMenuItem item in ToolStripMenuItem_imref.DropDownItems) {
              item.Checked = (item.Equals(sender))?true :false;
            }
            foreach (ToolStripMenuItem item in ToolStripMenuItem_imdiv.DropDownItems) {
              item.Checked = false;
            }
            int index = ToolStripMenuItem_imref.DropDownItems.IndexOf(((ToolStripMenuItem)sender));
            ((ToolStripMenuItem)(ToolStripMenuItem_imdiv.DropDownItems[index])).Checked = true;
            ((FreqResponseGraphScale)scale[1]).top = util.ToDouble(((ToolStripMenuItem)sender).Text);
            ((FreqResponseGraphScale)scale[1]).perdiv = util.ToDouble(((ToolStripMenuItem)sender).Text)/10;
            UpdateBitmap();
        }
        
        
        // コンテクストメニュー　虚数部　/div
        private void ToolStripMenuItem_imdiv_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nImpedanceView.ToolStripMenuItem_imdiv_Click:" );
            foreach (ToolStripMenuItem item in ToolStripMenuItem_imdiv.DropDownItems) {
              item.Checked = (item.Equals(sender))?true :false;
            }
            ((FreqResponseGraphScale)scale[1]).perdiv = util.ToDouble(((ToolStripMenuItem)sender).Text);
            UpdateBitmap();
        }
        
        
        // コンテクストメニュー　絶対値　Ref
        private void ToolStripMenuItem_absref_Click(object sender, EventArgs e) {
            foreach (ToolStripMenuItem item in ToolStripMenuItem_absref.DropDownItems) {
              item.Checked = (item.Equals(sender))?true :false;
            }
            foreach (ToolStripMenuItem item in ToolStripMenuItem_absdiv.DropDownItems) {
              item.Checked = false;
            }
            int index = ToolStripMenuItem_absref.DropDownItems.IndexOf(((ToolStripMenuItem)sender));
            ((ToolStripMenuItem)(ToolStripMenuItem_absdiv.DropDownItems[index])).Checked = true;
            mainf.LogPrint( "\nImpedanceView.ToolStripMenuItem_absref_Click:" );
            ((FreqResponseGraphScale)scale[2]).top = util.ToDouble(((ToolStripMenuItem)sender).Text);
            ((FreqResponseGraphScale)scale[2]).perdiv = util.ToDouble(((ToolStripMenuItem)sender).Text)/10;
            UpdateBitmap();
        }
        
        
        // コンテクストメニュー　絶対値　/div
        private void ToolStripMenuItem_absdiv_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nImpedanceView.ToolStripMenuItem_absdiv_Click:" );
            foreach (ToolStripMenuItem item in ToolStripMenuItem_absdiv.DropDownItems) {
              item.Checked = (item.Equals(sender))?true :false;
            }
            ((FreqResponseGraphScale)scale[2]).perdiv = util.ToDouble(((ToolStripMenuItem)sender).Text);
            UpdateBitmap();
        }
        
        
        // コンテクストメニュー　位相　Ref
        private void ToolStripMenuItem_pharef_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nImpedanceView.ToolStripMenuItem_pharef_Click:" );
            foreach (ToolStripMenuItem item in ToolStripMenuItem_pharef.DropDownItems) {
              item.Checked = (item.Equals(sender))?true :false;
            }
            ((FreqResponseGraphScale)scale[3]).top = util.ToDouble(((ToolStripMenuItem)sender).Text);
            UpdateBitmap();
        }
        
        
        // コンテクストメニュー　位相　/div
        private void ToolStripMenuItem_phadiv_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nImpedanceView.ToolStripMenuItem_phadiv_Click:" );
            foreach (ToolStripMenuItem item in ToolStripMenuItem_phadiv.DropDownItems) {
              item.Checked = (item.Equals(sender))?true :false;
            }
            ((FreqResponseGraphScale)scale[3]).perdiv = util.ToDouble(((ToolStripMenuItem)sender).Text);
            UpdateBitmap();
        }
        
        
        // コンテクストメニュー　Ｌ　Ref
        private void ToolStripMenuItem_lref_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nImpedanceView.ToolStripMenuItem_lref_Click:" );
            foreach (ToolStripMenuItem item in ToolStripMenuItem_lref.DropDownItems) {
              item.Checked = (item.Equals(sender))?true :false;
            }
            foreach (ToolStripMenuItem item in ToolStripMenuItem_ldiv.DropDownItems) {
              item.Checked = false;
            }
            int index = ToolStripMenuItem_lref.DropDownItems.IndexOf(((ToolStripMenuItem)sender));
            ((ToolStripMenuItem)(ToolStripMenuItem_ldiv.DropDownItems[index])).Checked = true;
            ((FreqResponseGraphScale)scale[4]).top = util.ToDouble(((ToolStripMenuItem)sender).Text);
            ((FreqResponseGraphScale)scale[4]).perdiv = util.ToDouble(((ToolStripMenuItem)sender).Text)/10;
            UpdateBitmap();
        }
        
        
        // コンテクストメニュー　Ｌ　/div
        private void ToolStripMenuItem_ldiv_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nImpedanceView.ToolStripMenuItem_ldiv_Click:" );
            foreach (ToolStripMenuItem item in ToolStripMenuItem_ldiv.DropDownItems) {
              item.Checked = (item.Equals(sender))?true :false;
            }
            ((FreqResponseGraphScale)scale[4]).perdiv = util.ToDouble(((ToolStripMenuItem)sender).Text);
            UpdateBitmap();
        }
        
        
        // コンテクストメニュー　Ｃ　Ref
        private void ToolStripMenuItem_cref_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nImpedanceView.ToolStripMenuItem_cref_Click:" );
            foreach (ToolStripMenuItem item in ToolStripMenuItem_cref.DropDownItems) {
              item.Checked = (item.Equals(sender))?true :false;
            }
            foreach (ToolStripMenuItem item in ToolStripMenuItem_cdiv.DropDownItems) {
              item.Checked = false;
            }
            int index = ToolStripMenuItem_cref.DropDownItems.IndexOf(((ToolStripMenuItem)sender));
            ((ToolStripMenuItem)(ToolStripMenuItem_cdiv.DropDownItems[index])).Checked = true;
            ((FreqResponseGraphScale)scale[5]).top = util.ToDouble(((ToolStripMenuItem)sender).Text);
            ((FreqResponseGraphScale)scale[5]).perdiv = util.ToDouble(((ToolStripMenuItem)sender).Text)/10;
            UpdateBitmap();
        }
        
        
        // コンテクストメニュー　Ｃ　/div
        private void ToolStripMenuItem_cdiv_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nImpedanceView.ToolStripMenuItem_cdiv_Click:" );
            foreach (ToolStripMenuItem item in ToolStripMenuItem_cdiv.DropDownItems) {
              item.Checked = (item.Equals(sender))?true :false;
            }
            ((FreqResponseGraphScale)scale[5]).perdiv = util.ToDouble(((ToolStripMenuItem)sender).Text);
            UpdateBitmap();
        }
        
        
        // コンテクストメニュー　波形色
        private void ToolStripMenuItem_color_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nImpedanceView.ToolStripMenuItem_color_Click:" );
            int waveno = isLeftArea()? leftaxis:rightaxis;
            ((FreqResponseGraphWave)(wavelist[waveno])).SetColor(
                  clr.colordialog(((FreqResponseGraphWave)(wavelist[waveno])).GetColor())
                  );
            UpdateBitmap();
        }
        
        
        // コンテクストメニュー　背景色
        private void ToolStripMenuItem_backcolor_Click(object sender, EventArgs e) {
            mainf.LogPrint("\nImpedanceView.ToolStripMenuItem_backcolor_Click:");
            FRGGraphBG = clr.colordialog( FRGGraphBG );
            UpdateBitmap();
        }
        
        
        // コンテクストメニュー　縦軸分割数
        private void ToolStripMenuItem_div_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nImpedanceView.ToolStripMenuItem_div_Click:" );
            foreach (ToolStripMenuItem item in ToolStripMenuItem_div.DropDownItems) {
              item.Checked = (item.Equals(sender))?true :false;
            }
            base.vdiv = Convert.ToInt32(((ToolStripMenuItem)sender).Text);
            UpdateBitmap();
        }
        
        
        // コンテクストメニュー　周波数軸　ログ
        private void ToolStripMenuItem_freqaxis_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nImpedanceView.ToolStripMenuItem_freqaxis_Click:" );
            logscale = !logscale;
            centerspan = false;
            freqcenterstartstop();    // 画面表示をセンター周波数表示⇔開始終了周波数表示で切り替える
            UpdateBitmap();
        }
        
        
        // コンテクストメニュー　開始周波数
        private void ToolStripMenuItem_startfreq_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nImpedanceView.ToolStripMenuItem_startfreq_Click:" );
            
        }
        
        
        // コンテクストメニュー　終了周波数
        private void ToolStripMenuItem_stopfreq_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nImpedanceView.ToolStripMenuItem_stopfreq_Click:" );
            
        }
        
        
        // コンテクストメニュー　フルスパン表示
        private void ToolStripMenuItem_fullspan_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nImpedanceView.nToolStripMenuItem_fullspan_Click:" );
            base.menuFullSpan();
        }
        
        
        // コンテクストメニュー　センター周波数⇔開始終了周波数
        private void ToolStripMenuItem_freqcenter_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nImpedanceView.ToolStripMenuItem_freqcenter_Click:" );
            centerspan = !centerspan;
            logscale = false;
            freqcenterstartstop();    // 画面表示をセンター周波数表示⇔開始終了周波数表示で切り替える
            UpdateBitmap();
        }
        
        
        // コンテクストメニュー　カーソルをセンター周波数へ
        private void ToolStripMenuItem_cursor2freqcenter_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nImpedanceView.ToolStripMenuItem_cursor2freqcenter_Click:" );
            base.menuCur2Center();
            freqcenterstartstop();    // 画面表示をセンター周波数表示⇔開始終了周波数表示で切り替える
            UpdateBitmap();
        }
        
        
        // コンテクストメニュー　カーソル間をズーム
        private void ToolStripMenuItem_zoom_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nImpedanceView.ToolStripMenuItem_zoom_Click:" );
            base.menuZoom();
        }
        
        
        // コンテクストメニュー　カーソル間のズームをやめる
        private void ToolStripMenuItem_nozoom_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nImpedanceView.ToolStripMenuItem_nozoom_Click:" );
            base.fstart = fstart;
            base.fstop  = fstop;
            UpdateBitmap();
        }
        
        
        // コンテクストメニュー　カーソルを中心に拡大
        private void ToolStripMenuItem_expand_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nImpedanceView.ToolStripMenuItem_expand_Click:" );
            menuExpand();
        }
        
        
        // コンテクストメニュー　カーソルを中心に縮小
        private void ToolStripMenuItem_shrink_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nImpedanceView.ToolStripMenuItem_shrink_Click:" );
            menuShrink();
        }
        
        
        // コンテクストメニュー　カーソル高調波表示
        private void ToolStripMenuItem_harmonics_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nImpedanceView.ToolStripMenuItem_harmonics_Click:" );
            base.harmonics = !base.harmonics;
            ToolStripMenuItem_harmonics.Checked = base.harmonics;
            PaintAll();
        }
        
        
        // コンテクストメニュー　カーソルを基準値補正する
        private void ToolStripMenuItem_toOffset_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nImpedanceView.ToolStripMenuItem_to0db_Click: "+  curcursor.data.ToString());
            base.SetOffset(curcursor.wave, (( curcursor.wave.offset == 0 )?curcursor.data:0));
        }
        
        
        // コンテクストメニュー　基準値補正をやめる
        private void ToolStripMenuItem_cancelOffset_Click(object sender, EventArgs e) {
            base.SetOffset(curcursor.wave, 0);
        }
        
        
        // コンテクストメニュー　次のピークへ
        private void ToolStripMenuItem_nextpeak_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nImpedanceView.ToolStripMenuItem_nextpeak_Click:" );
            base.menuNextPeak();
        }
        
        
        // コンテクストメニュー　デルタ表示
        private void ToolStripMenuItem_delta_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nImpedanceView.ToolStripMenuItem_delta_Click:" );
            if( ((FreqResponseGraphCursor)(base.cursor[0])).wave != ((FreqResponseGraphCursor)(base.cursor[1])).wave ) return;  // 同じ wave でないときはデルタ表示にしない
            deltaf = !deltaf;
            ToolStripMenuItem_delta.Checked = deltaf;
            PaintAll();
        }
        
        
        // コンテクストメニュー　カーソルライン表示
        private void ToolStripMenuItem_cursorline_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nImpedanceView.ToolStripMenuItem_cursorline_Click:" );
            cursorline = !cursorline;
            ToolStripMenuItem_cursorline.Checked = cursorline;
            PaintAll();
        }
        
        
        // コンテクストメニュー　カーソル×表示
        private void ToolStripMenuItem_cursorcross_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nImpedanceView.ToolStripMenuItem_cursorcross_Click:" );
            cursorcross = !cursorcross;
            ToolStripMenuItem_cursorcross.Checked = cursorcross;
            PaintAll();
        }
        
        
        // コンテクストメニュー　測定結果を残す
        private void ToolStripMenuItem_keepresult_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nImpedanceView.ToolStripMenuItem_keepresult_Click:" );
            keepresult = !keepresult;
            ToolStripMenuItem_keepresult.Checked = keepresult;
        }
        
        
        // コンテクストメニュー　トレースをすべて消す
        private void ToolStripMenuItem_eraseall_Click(object sender, EventArgs e) {
            if( !measuring ) {
                foreach (FreqResponseGraphWave frw in wavelist) frw.WaveClear();
                UpdateBitmap();
            }else{
                error.ErrorMessageBox(error.op_err);
            }
        }
        
        
        // コンテクストメニュー　古いトレースを消す
        private void ToolStripMenuItem_eraseold_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nImpedanceView.ToolStripMenuItem_eraseold_Click:" );
            if( !measuring ) {
                DelOldTraces(1);
                UpdateBitmap();
            }else{
                error.ErrorMessageBox(error.op_err);
            }
        }
        
        
        // コンテクストメニュー　現測定値でショートキャリブレーション
        private void ToolStripMenuItem_mestocalshort_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nImpedanceView.ToolStripMenuItem_mestocalshort_Click:" );
            ToolStripMenuItem_mestocalshort.Checked = true;
            setcaldata( calmodeshort );
        }
        
        
        // コンテクストメニュー　現測定値でオープンキャリブレーション
        private void ToolStripMenuItem_mestocalopen_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nImpedanceView.ToolStripMenuItem_mestocalopen_Click:" );
            ToolStripMenuItem_mestocalopen.Checked = true;
            setcaldata( calmodeopen );
        }
        
        
        // コンテクストメニュー　測定ポイント数
        private void ToolStripMenuItem_mespoint_Click(object sender, EventArgs e) {
            foreach (ToolStripMenuItem item in ToolStripMenuItem_mespoint.DropDownItems) {
              item.Checked = (item.Equals(sender))?true :false;
            }
            
            
            
        }
        
        
        // コンテクストメニュー　アベレージング
        private void ToolStripMenuItem_averaging_Click(object sender, EventArgs e) {
            foreach (ToolStripMenuItem item in ToolStripMenuItem_averaging.DropDownItems) {
              item.Checked = (item.Equals(sender))?true :false;
            }
            
            
            
        }
        
        
        // コンテクストメニュー　出力レベル
        private void ToolStripMenuItem_outlevel_Click(object sender, EventArgs e) {
            foreach (ToolStripMenuItem item in ToolStripMenuItem_outlevel.DropDownItems) {
              item.Checked = (item.Equals(sender))?true :false;
            }
            outlevel = (def.hfsgleveltbl[ ToolStripMenuItem_outlevel.DropDownItems.IndexOf(((ToolStripMenuItem)sender)) ])<<16;
        }
        
        
        // コンテクストメニュー　シングルスイープ
        private void ToolStripMenuItem_sweep_Click(object sender, EventArgs e) {
            singlesweep = !singlesweep;
            ToolStripMenuItem_sweep.Checked = singlesweep;
        }
        
        
        // コンテクストメニュー　パーシスタンス
        private void ToolStripMenuItem_persistence_Click(object sender, EventArgs e) {
            foreach (ToolStripMenuItem item in ToolStripMenuItem_persistence.DropDownItems) {
              item.Checked = (item.Equals(sender))?true :false;
            }
            base.persistence = util.ToDouble(((ToolStripMenuItem)sender).Text)/100;
            if( base.persistence < 0.01 ) base.persistence = 0.01;
            if( !measuring ) UpdateBitmap();
        }
        
        
        // コンテクストメニュー　トレース保存数
        private void ToolStripMenuItem_trace_Click(object sender, EventArgs e) {
            foreach (ToolStripMenuItem item in ToolStripMenuItem_trace.DropDownItems) {
              item.Checked = (item.Equals(sender))?true :false;
            }
            ntrace = util.ToInt32(((ToolStripMenuItem)sender).Text);
            if( !measuring ) {
                DelOldTraces( ntrace );
                UpdateBitmap();
            }
        }
        
        
        // コンテクストメニュー　表示項目
        private void ToolStripMenuItem_displayitems_Click(object sender, EventArgs e) {
            int index = ToolStripMenuItem_displayitems.DropDownItems.IndexOf((ToolStripMenuItem)sender);
            switch( index ) {
            case 0:
                CommentForm f = new CommentForm(this);
                f.ShowDialog();
                ((ToolStripMenuItem)(ToolStripMenuItem_displayitems.DropDownItems[index])).Checked = base.commentvisible;
                break;
            case 1:
                base.datetimevisible = !base.datetimevisible;
                ((ToolStripMenuItem)(ToolStripMenuItem_displayitems.DropDownItems[index])).Checked = base.datetimevisible;
                break;
            case 2:
                base.conditionvisible = !base.conditionvisible;
                ((ToolStripMenuItem)(ToolStripMenuItem_displayitems.DropDownItems[index])).Checked = base.conditionvisible;
                break;
            }
            if( !measuring ) UpdateBitmap();
        }
        
        
        // コンテクストメニュー　コメント表示
        private void ToolStripMenuItem_comment_Click(object sender, EventArgs e) {
        }
        
        
        // コンテクストメニュー　測定日時表示
        private void ToolStripMenuItem_datetime_Click(object sender, EventArgs e) {
        }
        
        
        // コンテクストメニュー　設定
        private void toolStripMenuItem_setting_Click(object sender, EventArgs e) {
        }
        
        
        // コンテクストメニュー　カーソルの下に値表示
        private void ToolStripMenuItem_showvaluenearcur_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nImpedanceView.ToolStripMenuItem_showvaluenearcur_Click:" );
            showlevelnearcur = !showlevelnearcur;
            ToolStripMenuItem_showvaluenearcur.Checked = showlevelnearcur;
        }
        
        
        // コンテクストメニュー　測定値をクリップボードにコピー
        private void ToolStripMenuItem_clipboard_Click(object sender, EventArgs e) {
            copy2clipborad();
        }
        
        
        // センター周波数変更イベント
        private void textBox_centerfreq_TextChanged(object sender, EventArgs e) {
            mainf.LogPrint( "\nImpedanceView.textBox_centerfreq_TextChanged: " + textBox_centerfreq.Text  );
            double centerfreq = util.ToDouble(textBox_centerfreq.Text);
            double freqspan = util.ToDouble(comboBox_freqspan.Text);
            base.fstart = centerfreq - freqspan/2;
            base.fstop  = centerfreq + freqspan/2;
            UpdateBitmap();
        }
        
        
        // 周波数スパン変更イベント
        private void comboBox_freqspan_SelectedIndexChanged(object sender, EventArgs e) {
            mainf.LogPrint( "\nImpedanceView.comboBox_freqspan_SelectedIndexChanged: " + comboBox_freqspan.Text );
            double centerfreq = util.ToDouble(textBox_centerfreq.Text);
            double freqspan = util.ToDouble(comboBox_freqspan.Text);
            mainf.LogPrint( "\nfreqspan=" + freqspan.ToString());
            if( freqspan==0 ) freqspan=Apb3.clockfreq/2;
            base.fstart = centerfreq - freqspan/2;
            base.fstop  = centerfreq + freqspan/2;
            UpdateBitmap();
        }
        
        
        // 開始周波数変更イベント
        private void textBox_startfreq_TextChanged(object sender, EventArgs e) {
            mainf.LogPrint( "\nImpedanceView.textBox_startfreq_TextChanged: " + textBox_startfreq.Text );
            base.fstart = util.ToDouble(textBox_startfreq.Text);
            UpdateBitmap();
        }
        
        
        // 終了周波数変更イベント
        private void textBox_stopfreq_TextChanged(object sender, EventArgs e) {
            mainf.LogPrint( "\nImpedanceView.textBox_stopfreq_TextChanged: " + textBox_stopfreq.Text );
            base.fstop = util.ToDouble(textBox_stopfreq.Text);
            UpdateBitmap();
        }
        
        
        // ボタン　測定開始
        private void measureButton_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\n\nImpedanceView.btnMeasureStartClick:" );
            if (measuring) {
                stopmeasuring = true;  // 中断
            } else if( base.fstart>base.fstop ) {
                error.ErrorMessageBox(error.mes_noexec);
            } else {
                if( !Apb3.Open() ) {
                    mainf.LogPrint( "\nERROR: can't open Apb3 ");
                    return;
                }
                Apb3.Close();
                
                measuring = true;
                stopmeasuring = false;
                errormeasuring = false;
                base.dt = DateTime.Now;
                rbw = comboBox_rbw.SelectedIndex;
                fstart = base.fstart;   // コントロールに値を設定するとイベント処理が入り
                fstop  = base.fstop;    // 変更されてしまうため保存しておく
                textBox_centerfreq.Text = util.Double2String_000((fstart + fstop)/2);
                comboBox_freqspan.Text  = util.Double2String_000((fstop  - fstart));
                textBox_startfreq.Text  = util.Double2String_000(fstart);
                textBox_stopfreq.Text   = util.Double2String_000(fstop);
                npoint = Convert.ToInt16(comboBox_n.Text);
                fstep  = (fstop - fstart) / npoint;
                
                Tmeasure = new Thread(new ThreadStart(tMeasure));
                Tmeasure.Start();                 // 測定スレッドを開始する
            }
        }
        
        
        // 測定スレッド
        private void tMeasure() {
            if( Apb3.Open() ) {
                Stopwatch sw = new Stopwatch();
                sw.Start();
                
                try {
                    int adinput=0;
                    
                    btnStartText( def.stop, clr.MeasBtnExec );
                    FFT.WindowFunc(Apb3.nFFT, 4, 4);         // ガウス窓作成

                    Apb3.CICWrite( rbw );               // RBW によってデシメーション設定
                    Apb3.MemwriteSetting( 0 );         // メモリー書き込み入力番号０、即書き込み開始モード

                    Apb3.dds_setting( 1, outlevel );    // 出力レベル設定
                    Apb3.ad9707Write( 2 );               // DDS0 出力する
                    org.Clear();
                    double fmulti=1;
                    double freq;
                    do {
                        foreach (FreqResponseGraphWave frw in wavelist) frw.IncNsweep();  // スイープ数を１増やす
                        if( logscale ) {
                            if (fstart < 1) fstart = 1;
                            fmulti = Math.Pow( 10, (Math.Log10( fstop ) - Math.Log10( fstart ))/npoint );
                            freq = fstart;
                        } else {
                            freq = fstart;
                            if (freq <= 0) freq = 0.1;
                        }
                        
                        for( int n=0; n < npoint+1; n++ ) { 
                            
                            Apb3.dds0( freq );
                            Apb3.Fpga.FpgaWrite(Apb3.Fpga.memcontrol+1, (0<<7));  // D36=0
                            Apb3.MemwriteStart();           // 測定開始
                            if( Apb3.MemoryReadready( rdBuffer )<0 ) {
                                mainf.LogPrint("\nImpedanceView.MeasureStart: 通信エラー");
                                errormeasuring = true;      // 通信エラーなので中断し
                                Apb3.CloseConnection();     // 一度クローズする
                                break;
                            }
                            Apb3.Fpga.FpgaWrite(Apb3.Fpga.memcontrol+1, (1<<7));  // D36=1
                            Apb3.MemoryRead(rdBuffer_d32);
                            
                            int in1, in2;
                            double isig = 0, qsig = 0;
                            for (int i = 0; i < Apb3.buffsize/4; i++) {
                                in1  = rdBuffer_d32[4*i] & 0x0F;
                                in1 += rdBuffer[4*i]<<4;
                                in1 += (((int)rdBuffer[4*i+1]) << 12);
                                in2  = (rdBuffer_d32[4*i] & 0xF0)>>4;
                                in2 += rdBuffer[4*i+2]<<4;
                                in2 += (((int)rdBuffer[4*i+3]) << 12);
                                if ((in1 & 0x00080000) != 0) in1 -= 1048576;
                                if ((in2 & 0x00080000) != 0) in2 -= 1048576;
                                isig += ((double)in1) * FFT.window[i];
                                qsig += ((double)in2) * FFT.window[i];
                            }
                            Complex vx = new Complex( isig, -qsig );
                            org.Add( new FreqResponseComplexData(freq, vx) );
                            
                            Complex vshort = interpolation( freq, calshort );
                            Complex vopen  = interpolation( freq, calopen );
                            Complex x = 50*vopen*(vx-vshort)/(vshort*(vx-vopen)); // X  = (50*V8*VD-50*V0*V8)/(V0*VD-V0*V8)   ←インピーダンス測定結果
                            double w = 2*Math.PI*freq;
                            
                            ((FreqResponseGraphWave)wavelist[0]).Add(freq, x.Re);    // Real part
                            ((FreqResponseGraphWave)wavelist[1]).Add(freq, x.Im);    // Imaginally part
                            ((FreqResponseGraphWave)wavelist[2]).Add(freq, x.Abs);   // Abs
                            ((FreqResponseGraphWave)wavelist[3]).Add(freq, x.Arg);   // Pha
                            ((FreqResponseGraphWave)wavelist[4]).Add(freq, x.Im/w);   // L
                            ((FreqResponseGraphWave)wavelist[5]).Add(freq, -1/x.Im/w);   // C
                            
                            Invoke((DelegateType)delegate { UpdateBitmap(); });  // 画面再描画
                            
                            if( stopmeasuring ) break;   // 測定中に中断した
                            
                            if( logscale ) {
                                freq = freq*fmulti;
                            } else {
                                freq = freq+fstep;
                            }
                            if( freq >= Apb3.clockfreq/2 ) freq = Apb3.clockfreq/2 - 0.1;
                        }
                        if( errormeasuring ) {
                            error.ErrorMessageBox(error.usb_disconnect); // 測定中に通信エラー
                            break;
                        }
                        
                        if( stopmeasuring || errormeasuring ) {   // 測定中に中断した
                            foreach (FreqResponseGraphWave frw in wavelist) frw.tracecount(); // nsweep の整合性を取る
                            Invoke((DelegateType)delegate { UpdateBitmap(); });  // 画面再描画
                            Apb3.ad9707Write(3);    // DDS1 出力する
                            break;
                        }
                        DelOldTraces( ntrace );
                        Invoke((DelegateType)delegate { UpdateBitmap(); });  // 画面再描画
                    } while( !singlesweep );
                    
                }
                catch (Exception ex) {
                    mainf.LogPrint("\nImpedanceView.MeasureStart: exception=" + ex.Message);
                }
                finally {
//                    Apb3.ad9707Write(3);    // DDS1 出力する
                    Apb3.Close();
                    btnStartText( def.measure, clr.MeasBtnNor );
                    mainf.LogPrint("\nImpedanceView: ", sw);
                    sw.Stop();
                }
            }
            measuring=false;
        }
        
        
        // キャリブレーションデータの設定
        private void setcaldata( int calmode ) {
            ArrayList cal = new ArrayList();
            cal = (ArrayList)(org.Clone());
            if( cal.Count != 0 ) {
                int count=1;
                int countborder=0;
                double f1 = ((FreqResponseComplexData)(cal[0])).freq;
                for( ; count< cal.Count; count++ ) {           // 次のトレースまでのデータ個数
                    double f2 = ((FreqResponseComplexData)(cal[count])).freq;
                    if( f1 > f2 ) {
                        countborder = count;
                        break;
                    }
                    f1 = f2;
                }
                cal.RemoveRange( 0, countborder ); // 最新の１トレースを残して削除
            }
            
            switch( calmode ) {
            case calmodeopen:
                calopen = (ArrayList)(cal.Clone());
                if( !calshortdone ) {
                    calshort.Clear();
                    foreach( FreqResponseComplexData frcd in org) {
                        calshort.Add( new FreqResponseComplexData(frcd.freq, ( -frcd.data )) );
                    }
                }
                calopendone = true;
                break;
            case calmodeshort:
                calshort = (ArrayList)(cal.Clone());
                if( !calopendone ) {
                    calopen.Clear();
                    foreach( FreqResponseComplexData frcd in org) {
                        calopen.Add( new FreqResponseComplexData(frcd.freq, ( -frcd.data )) );
                    }
                }
                calshortdone = true;
                break;
            case calmode50:
                
                break;
            case calmodeinit:   // 初期値
                initcaldata();
                calopendone = false;
                calshortdone = false;
                break;
            }
        }
        
        
        // 補間（ freq での値を補間計算して返す ）
        private Complex interpolation( double freq, ArrayList al ) {
            Complex d1 = new Complex( 0, 0 );
            Complex d2 = new Complex( 0, 0 );
            foreach (FreqResponseComplexData frcdshort in al) {
                if (frcdshort.freq < freq) {
                    d1 = frcdshort.data;
                } else if (frcdshort.freq == freq) {
                    return frcdshort.data;
                } else if (frcdshort.freq > freq) {
                    return frcdshort.data;
                }
            }
            return d1;
        }
        
        
        // 古いトレースを削除
        private void DelOldTraces( int n ) {
            foreach (FreqResponseGraphWave frw in wavelist) frw.DelOldTraces( n );
        }
        
        
        // 測定開始ボタンのテキスト、色を変更
        public void btnStartText( string str, Color clr ) {
            Invoke((DelegateType)delegate {
                measureButton.Text = str;
                measureButton.BackColor = clr;
            });
        }
        
        
        // 画面表示をセンター周波数表示⇔開始終了周波数表示で切り替える
        private void freqcenterstartstop() {
            if( base.centerspan ) {
              label_start_center.Text = "center";
              label_stop_span.Text = "span";
              textBox_startfreq.Visible = false;
              textBox_stopfreq.Visible = false;
              textBox_centerfreq.Visible = true;
              comboBox_freqspan.Visible = true;
            } else {
              label_start_center.Text = "start";
              label_stop_span.Text = "stop";
              textBox_startfreq.Visible = true;
              textBox_stopfreq.Visible = true;
              textBox_centerfreq.Visible = false;
              comboBox_freqspan.Visible = false;
            }
        }
        
        
        public override void MeasureButtonFocus() {
            measureButton.Focus();
        }
        
        
        
        private void initcaldata() {
            calopen.Add(new FreqResponseComplexData(0.1, new Complex(-189387.949369851, -165780.818816109) ));
            calopen.Add(new FreqResponseComplexData(400000.1, new Complex(1737172.52813978, -381353.171263568) ));
            calopen.Add(new FreqResponseComplexData(800000.1, new Complex(1150788.54297033, -1367875.35050466) ));
            calopen.Add(new FreqResponseComplexData(1200000.1, new Complex(245774.255452649, -1745572.68738627) ));
            calopen.Add(new FreqResponseComplexData(1600000.1, new Complex(-655599.405447739, -1601874.93467777) ));
            calopen.Add(new FreqResponseComplexData(2000000.1, new Complex(-1332605.45915663, -1049016.20302509) ));
            calopen.Add(new FreqResponseComplexData(2400000.1, new Complex(-1638869.90717286, -266401.287095367) ));
            calopen.Add(new FreqResponseComplexData(2800000.1, new Complex(-1530111.06478514, 540281.175386476) ));
            calopen.Add(new FreqResponseComplexData(3200000.1, new Complex(-1063134.36670665, 1174218.84836914) ));
            calopen.Add(new FreqResponseComplexData(3600000.1, new Complex(-373986.170602424, 1499934.00517339) ));
            calopen.Add(new FreqResponseComplexData(4000000.1, new Complex(362888.836099212, 1462102.55482577) ));
            calopen.Add(new FreqResponseComplexData(4400000.1, new Complex(976501.699842894, 1098924.31124495) ));
            calopen.Add(new FreqResponseComplexData(4800000.1, new Complex(1341622.67330593, 510638.158226525) ));
            calopen.Add(new FreqResponseComplexData(5200000.1, new Complex(1393446.32610872, -158969.443724092) ));
            calopen.Add(new FreqResponseComplexData(5600000.1, new Complex(1141261.35394992, -761210.205649485) ));
            calopen.Add(new FreqResponseComplexData(6000000.1, new Complex(656782.258351036, -1172474.75927011) ));
            calopen.Add(new FreqResponseComplexData(6400000.1, new Complex(53314.2759197757, -1318138.76430156) ));
            calopen.Add(new FreqResponseComplexData(6800000.1, new Complex(-538261.89710599, -1180578.47121469) ));
            calopen.Add(new FreqResponseComplexData(7200000.1, new Complex(-997681.02466114, -800212.307835422) ));
            calopen.Add(new FreqResponseComplexData(7600000.1, new Complex(-1235881.41869825, -263459.498710698) ));
            calopen.Add(new FreqResponseComplexData(8000000.1, new Complex(-1211383.98030929, 316427.144718388) ));
            calopen.Add(new FreqResponseComplexData(8400000.1, new Complex(-934448.343890998, 819098.231214847) ));
            calopen.Add(new FreqResponseComplexData(8800000.1, new Complex(-465598.86923112, 1146795.00607987) ));
            calopen.Add(new FreqResponseComplexData(9200000.1, new Complex(97741.0298465201, 1230975.46978254) ));
            calopen.Add(new FreqResponseComplexData(9600000.1, new Complex(641443.723348058, 1056821.40112726) ));
            calopen.Add(new FreqResponseComplexData(10000000.1, new Complex(1051992.44710822, 658018.366248492) ));
            calopen.Add(new FreqResponseComplexData(10400000.1, new Complex(1241027.36542668, 111680.571554284) ));
            calopen.Add(new FreqResponseComplexData(10800000.1, new Complex(1165220.08707792, -466424.731015043) ));
            calopen.Add(new FreqResponseComplexData(11200000.1, new Complex(833581.600929068, -955183.905879373) ));
            calopen.Add(new FreqResponseComplexData(11600000.1, new Complex(306968.704073315, -1244466.68973947) ));
            calopen.Add(new FreqResponseComplexData(12000000.1, new Complex(-303436.10744231, -1262468.10376567) ));
            calopen.Add(new FreqResponseComplexData(12400000.1, new Complex(-865418.04299333, -991210.523723647) ));
            calopen.Add(new FreqResponseComplexData(12800000.1, new Complex(-1246032.1912248, -481325.014485396) ));
            calopen.Add(new FreqResponseComplexData(13200000.1, new Complex(-1345178.2648169, 160752.957880369) ));
            calopen.Add(new FreqResponseComplexData(13600000.1, new Complex(-1126765.60378661, 787741.792352386) ));
            calopen.Add(new FreqResponseComplexData(14000000.1, new Complex(-625249.455915904, 1245442.41471448) ));
            calopen.Add(new FreqResponseComplexData(14400000.1, new Complex(45717.1855091877, 1411642.94902631) ));
            calopen.Add(new FreqResponseComplexData(14800000.1, new Complex(727627.0999541, 1229059.17439246) ));
            calopen.Add(new FreqResponseComplexData(15200000.1, new Complex(1242815.57504004, 730246.657005287) ));
            calopen.Add(new FreqResponseComplexData(15600000.1, new Complex(1450780.12912879, 33729.6614180036) ));
            calopen.Add(new FreqResponseComplexData(16000000.1, new Complex(1285438.13854861, -682948.803662705) ));
            calopen.Add(new FreqResponseComplexData(16400000.1, new Complex(787427.177149294, -1225337.89985113) ));
            calopen.Add(new FreqResponseComplexData(16800000.1, new Complex(84004.7210139741, -1449560.75097694) ));
            calopen.Add(new FreqResponseComplexData(17200000.1, new Complex(-635752.062990867, -1294651.67843448) ));
            calopen.Add(new FreqResponseComplexData(17600000.1, new Complex(-1178728.69673838, -806910.909143507) ));
            calopen.Add(new FreqResponseComplexData(18000000.1, new Complex(-1405003.79598444, -121257.026289187) ));
            calopen.Add(new FreqResponseComplexData(18400000.1, new Complex(-1262584.7983924, 575166.681822346) ));
            calopen.Add(new FreqResponseComplexData(18800000.1, new Complex(-804170.398350111, 1100981.70385635) ));
            calopen.Add(new FreqResponseComplexData(19200000.1, new Complex(-159595.522924489, 1327292.33578458) ));
            calopen.Add(new FreqResponseComplexData(19600000.1, new Complex(497861.209057183, 1209754.78581017) ));
            calopen.Add(new FreqResponseComplexData(20000000.1, new Complex(1002500.48912388, 799143.32879871) ));
            calopen.Add(new FreqResponseComplexData(20400000.1, new Complex(1232701.34705799, 203379.025078952) ));
            calopen.Add(new FreqResponseComplexData(20800000.1, new Complex(1149756.58203351, -409990.103081084) ));
            calopen.Add(new FreqResponseComplexData(21200000.1, new Complex(787735.766600424, -892645.773568311) ));
            calopen.Add(new FreqResponseComplexData(21600000.1, new Complex(253008.155975644, -1134896.93856933) ));
            calopen.Add(new FreqResponseComplexData(22000000.1, new Complex(-317986.950879884, -1091118.66433994) ));
            calopen.Add(new FreqResponseComplexData(22400000.1, new Complex(-785966.10016055, -785032.88062338) ));
            calopen.Add(new FreqResponseComplexData(22800000.1, new Complex(-1044856.68101259, -303016.155288127) ));
            calopen.Add(new FreqResponseComplexData(23200000.1, new Complex(-1040983.86196673, 231895.522261482) ));
            calopen.Add(new FreqResponseComplexData(23600000.1, new Complex(-785882.042657435, 691822.305132465) ));
            calopen.Add(new FreqResponseComplexData(24000000.1, new Complex(-349247.1769719, 969657.43748494) ));
            calopen.Add(new FreqResponseComplexData(24400000.1, new Complex(159217.38150442, 1002544.13134022) ));
            calopen.Add(new FreqResponseComplexData(24800000.1, new Complex(616264.626470463, 790093.064183316) ));
            calopen.Add(new FreqResponseComplexData(25200000.1, new Complex(912404.045533524, 387337.039940876) ));
            calopen.Add(new FreqResponseComplexData(25600000.1, new Complex(976873.463791121, -105783.730640319) ));
            calopen.Add(new FreqResponseComplexData(26000000.1, new Complex(792921.528403137, -566683.694016516) ));
            calopen.Add(new FreqResponseComplexData(26400000.1, new Complex(408884.322033172, -878834.173190192) ));
            calopen.Add(new FreqResponseComplexData(26800000.1, new Complex(-79629.008176823, -961498.504162226) ));
            calopen.Add(new FreqResponseComplexData(27200000.1, new Complex(-547882.168326063, -791795.912301002) ));
            calopen.Add(new FreqResponseComplexData(27600000.1, new Complex(-868763.239221313, -408140.802067742) ));
            calopen.Add(new FreqResponseComplexData(28000000.1, new Complex(-954841.693815503, 89707.6160487892) ));
            calopen.Add(new FreqResponseComplexData(28400000.1, new Complex(-774188.392350031, 563083.875472237) ));
            calopen.Add(new FreqResponseComplexData(28800000.1, new Complex(-372175.349308053, 880041.261262651) ));
            calopen.Add(new FreqResponseComplexData(29200000.1, new Complex(139899.869472698, 943079.593832445) ));
            calopen.Add(new FreqResponseComplexData(29600000.1, new Complex(612439.095257438, 727472.207439952) ));
            calopen.Add(new FreqResponseComplexData(30000000.1, new Complex(902626.60745001, 291074.413544978) ));
            calopen.Add(new FreqResponseComplexData(30400000.1, new Complex(913869.685181515, -234703.297109777) ));
            calopen.Add(new FreqResponseComplexData(30800000.1, new Complex(638382.77477888, -687001.559132722) ));
            calopen.Add(new FreqResponseComplexData(31200000.1, new Complex(158285.420682965, -917273.799437845) ));
            calopen.Add(new FreqResponseComplexData(31600000.1, new Complex(-371371.492580211, -843805.901618144) ));
            calopen.Add(new FreqResponseComplexData(32000000.1, new Complex(-770215.496119216, -486090.61761488) ));
            calopen.Add(new FreqResponseComplexData(32400000.1, new Complex(-899141.460720619, 37091.1922716443) ));
            calopen.Add(new FreqResponseComplexData(32800000.1, new Complex(-701055.912173587, 539240.35630815) ));
            calopen.Add(new FreqResponseComplexData(33200000.1, new Complex(-243886.37001765, 829025.153874699) ));
            calopen.Add(new FreqResponseComplexData(33600000.1, new Complex(291992.177981421, 785714.346593428) ));
            calopen.Add(new FreqResponseComplexData(34000000.1, new Complex(680311.861609926, 426013.456960135) ));
            calopen.Add(new FreqResponseComplexData(34400000.1, new Complex(753825.216879142, -82449.0929175071) ));
            calopen.Add(new FreqResponseComplexData(34800000.1, new Complex(494147.172273376, -499245.597755058) ));
            calopen.Add(new FreqResponseComplexData(35200000.1, new Complex(57132.7927931812, -634379.804943779) ));
            calopen.Add(new FreqResponseComplexData(35600000.1, new Complex(-319488.962080175, -462423.327601195) ));
            calopen.Add(new FreqResponseComplexData(36000000.1, new Complex(-466767.988269647, -129074.134184652) ));
            calopen.Add(new FreqResponseComplexData(36400000.1, new Complex(-371871.58282109, 166628.61970069) ));
            calopen.Add(new FreqResponseComplexData(36800000.1, new Complex(-150559.228689359, 298858.916853903) ));
            calopen.Add(new FreqResponseComplexData(37200000.1, new Complex(56580.0050807464, 264327.315822912) ));
            calopen.Add(new FreqResponseComplexData(37600000.1, new Complex(165179.980064066, 137435.764674739) ));
            calopen.Add(new FreqResponseComplexData(38000000.1, new Complex(169768.627768101, 7119.79936071599) ));
            calopen.Add(new FreqResponseComplexData(38400000.1, new Complex(111366.195449957, -76399.3986095563) ));
            calopen.Add(new FreqResponseComplexData(38800000.1, new Complex(37039.8018586692, -103457.526216474) ));
            calopen.Add(new FreqResponseComplexData(39200000.1, new Complex(-26272.837833295, -88156.3493567539) ));
            calopen.Add(new FreqResponseComplexData(39600000.1, new Complex(-65424.9721491153, -49787.65473218) ));
            calopen.Add(new FreqResponseComplexData(39999999.9, new Complex(-3734.4883059116, 22455.5504891676) ));
            calopen.Add(new FreqResponseComplexData(49999999.9, new Complex(-3734.4883059116, 22455.5504891676) ));
            
            calshort.Add(new FreqResponseComplexData(0.1, new Complex(-110722.829556354, -227648.149701225) ));
            calshort.Add(new FreqResponseComplexData(400000.1, new Complex(-1684879.10061779, 582693.07888998) ));
            calshort.Add(new FreqResponseComplexData(800000.1, new Complex(-1104283.21259174, 1392707.76031694) ));
            calshort.Add(new FreqResponseComplexData(1200000.1, new Complex(-276435.403584211, 1749035.51629529) ));
            calshort.Add(new FreqResponseComplexData(1600000.1, new Complex(592118.132130933, 1665295.34583939) ));
            calshort.Add(new FreqResponseComplexData(2000000.1, new Complex(1307656.37475719, 1184503.81262429) ));
            calshort.Add(new FreqResponseComplexData(2400000.1, new Complex(1711714.89583459, 427753.88713306) ));
            calshort.Add(new FreqResponseComplexData(2800000.1, new Complex(1712375.37411221, -428748.318448351) ));
            calshort.Add(new FreqResponseComplexData(3200000.1, new Complex(1309222.67089912, -1185731.1092227) ));
            calshort.Add(new FreqResponseComplexData(3600000.1, new Complex(596658.674913595, -1663805.55015744) ));
            calshort.Add(new FreqResponseComplexData(4000000.1, new Complex(-259983.295440109, -1748429.87083971) ));
            calshort.Add(new FreqResponseComplexData(4400000.1, new Complex(-1055551.13449252, -1418546.30957481) ));
            calshort.Add(new FreqResponseComplexData(4800000.1, new Complex(-1600808.68037588, -748689.018668581) ));
            calshort.Add(new FreqResponseComplexData(5200000.1, new Complex(-1759259.67056642, 99401.7686309476) ));
            calshort.Add(new FreqResponseComplexData(5600000.1, new Complex(-1495709.80643254, 920667.551913337) ));
            calshort.Add(new FreqResponseComplexData(6000000.1, new Complex(-874166.956223236, 1512950.87790588) ));
            calshort.Add(new FreqResponseComplexData(6400000.1, new Complex(-48424.8880201721, 1735092.15145437) ));
            calshort.Add(new FreqResponseComplexData(6800000.1, new Complex(775341.35435821, 1535707.22251776) ));
            calshort.Add(new FreqResponseComplexData(7200000.1, new Complex(1398101.46062801, 972561.625835861) ));
            calshort.Add(new FreqResponseComplexData(7600000.1, new Complex(1671884.50384417, 190115.089996579) ));
            calshort.Add(new FreqResponseComplexData(8000000.1, new Complex(1541758.37293554, -616435.629723936) ));
            calshort.Add(new FreqResponseComplexData(8400000.1, new Complex(1050667.61891924, -1253119.58078169) ));
            calshort.Add(new FreqResponseComplexData(8800000.1, new Complex(330702.602488944, -1576047.50016435) ));
            calshort.Add(new FreqResponseComplexData(9200000.1, new Complex(-441487.119888153, -1520324.5146301) ));
            calshort.Add(new FreqResponseComplexData(9600000.1, new Complex(-1083663.10056759, -1117283.95705624) ));
            calshort.Add(new FreqResponseComplexData(10000000.1, new Complex(-1457237.23467977, -473101.457334994) ));
            calshort.Add(new FreqResponseComplexData(10400000.1, new Complex(-1482961.12667413, 255798.399551835) ));
            calshort.Add(new FreqResponseComplexData(10800000.1, new Complex(-1175802.54162922, 901122.157661839) ));
            calshort.Add(new FreqResponseComplexData(11200000.1, new Complex(-613871.186524399, 1323744.95721392) ));
            calshort.Add(new FreqResponseComplexData(11600000.1, new Complex(67080.3094231848, 1437429.09334724) ));
            calshort.Add(new FreqResponseComplexData(12000000.1, new Complex(715013.351995287, 1228815.08097476) ));
            calshort.Add(new FreqResponseComplexData(12400000.1, new Complex(1188131.98950141, 750161.652894833) ));
            calshort.Add(new FreqResponseComplexData(12800000.1, new Complex(1387021.42963795, 115901.529285801) ));
            calshort.Add(new FreqResponseComplexData(13200000.1, new Complex(1273835.40673515, -532636.315624219) ));
            calshort.Add(new FreqResponseComplexData(13600000.1, new Complex(878722.168656496, -1054952.42383345) ));
            calshort.Add(new FreqResponseComplexData(14000000.1, new Complex(289747.309092619, -1337567.62397721) ));
            calshort.Add(new FreqResponseComplexData(14400000.1, new Complex(-363779.471102599, -1316636.94474909) ));
            calshort.Add(new FreqResponseComplexData(14800000.1, new Complex(-934828.004174569, -996031.397694687) ));
            calshort.Add(new FreqResponseComplexData(15200000.1, new Complex(-1294369.87749493, -443748.021860761) ));
            calshort.Add(new FreqResponseComplexData(15600000.1, new Complex(-1354831.31552029, 217660.500559657) ));
            calshort.Add(new FreqResponseComplexData(16000000.1, new Complex(-1093538.00088638, 835386.113516993) ));
            calshort.Add(new FreqResponseComplexData(16400000.1, new Complex(-568133.807723771, 1259506.29283024) ));
            calshort.Add(new FreqResponseComplexData(16800000.1, new Complex(102797.782032134, 1383545.64848441) ));
            calshort.Add(new FreqResponseComplexData(17200000.1, new Complex(757391.088894297, 1167445.81100302) ));
            calshort.Add(new FreqResponseComplexData(17600000.1, new Complex(1231137.89156051, 657540.116173712) ));
            calshort.Add(new FreqResponseComplexData(18000000.1, new Complex(1397140.47141658, -22868.6252632905) ));
            calshort.Add(new FreqResponseComplexData(18400000.1, new Complex(1206709.6251635, -703156.207778596) ));
            calshort.Add(new FreqResponseComplexData(18800000.1, new Complex(705154.974659461, -1201392.44081694) ));
            calshort.Add(new FreqResponseComplexData(19200000.1, new Complex(20438.4838440199, -1384908.78028846) ));
            calshort.Add(new FreqResponseComplexData(19600000.1, new Complex(-662853.123076021, -1205011.37229774) ));
            calshort.Add(new FreqResponseComplexData(20000000.1, new Complex(-1162436.5702587, -708129.727670024) ));
            calshort.Add(new FreqResponseComplexData(20400000.1, new Complex(-1341496.86266279, -36913.5622239958) ));
            calshort.Add(new FreqResponseComplexData(20800000.1, new Complex(-1161854.676331, 624854.714737242) ));
            calshort.Add(new FreqResponseComplexData(21200000.1, new Complex(-682607.211323149, 1097878.82774307) ));
            calshort.Add(new FreqResponseComplexData(21600000.1, new Complex(-44706.113568426, 1262453.71191139) ));
            calshort.Add(new FreqResponseComplexData(22000000.1, new Complex(574450.370320205, 1090250.49380037) ));
            calshort.Add(new FreqResponseComplexData(22400000.1, new Complex(1010989.40569443, 643918.214732758) ));
            calshort.Add(new FreqResponseComplexData(22800000.1, new Complex(1162291.34037857, 57077.7047956598) ));
            calshort.Add(new FreqResponseComplexData(23200000.1, new Complex(1008164.37748057, -508072.30241203) ));
            calshort.Add(new FreqResponseComplexData(23600000.1, new Complex(608002.939014906, -909687.231624539) ));
            calshort.Add(new FreqResponseComplexData(24000000.1, new Complex(80454.3366863768, -1055509.87751272) ));
            calshort.Add(new FreqResponseComplexData(24400000.1, new Complex(-434345.16845193, -931898.625269696) ));
            calshort.Add(new FreqResponseComplexData(24800000.1, new Complex(-809959.782359767, -582717.781343472) ));
            calshort.Add(new FreqResponseComplexData(25200000.1, new Complex(-963151.274970128, -109609.81132239) ));
            calshort.Add(new FreqResponseComplexData(25600000.1, new Complex(-869977.444264955, 365086.494409131) ));
            calshort.Add(new FreqResponseComplexData(26000000.1, new Complex(-565506.454264206, 724603.30923914) ));
            calshort.Add(new FreqResponseComplexData(26400000.1, new Complex(-136463.460302335, 887438.8779392) ));
            calshort.Add(new FreqResponseComplexData(26800000.1, new Complex(309373.839545661, 823117.12631492) ));
            calshort.Add(new FreqResponseComplexData(27200000.1, new Complex(661153.114382133, 554432.780192097) ));
            calshort.Add(new FreqResponseComplexData(27600000.1, new Complex(836535.851541405, 153215.219888413) ));
            calshort.Add(new FreqResponseComplexData(28000000.1, new Complex(791822.564992182, -280080.992165956) ));
            calshort.Add(new FreqResponseComplexData(28400000.1, new Complex(540384.411650184, -631653.346730426) ));
            calshort.Add(new FreqResponseComplexData(28800000.1, new Complex(145609.682775814, -811411.681229582) ));
            calshort.Add(new FreqResponseComplexData(29200000.1, new Complex(-286012.088654909, -767722.75032731) ));
            calshort.Add(new FreqResponseComplexData(29600000.1, new Complex(-636719.271314892, -508053.755832486) ));
            calshort.Add(new FreqResponseComplexData(30000000.1, new Complex(-804172.005559724, -100279.086343702) ));
            calshort.Add(new FreqResponseComplexData(30400000.1, new Complex(-733167.936537021, 336785.315452489) ));
            calshort.Add(new FreqResponseComplexData(30800000.1, new Complex(-437003.926799519, 671947.213636666) ));
            calshort.Add(new FreqResponseComplexData(31200000.1, new Complex(-3871.03427456803, 793703.921321707) ));
            calshort.Add(new FreqResponseComplexData(31600000.1, new Complex(425782.295138632, 657572.364334077) ));
            calshort.Add(new FreqResponseComplexData(32000000.1, new Complex(706060.667081274, 306558.146232406) ));
            calshort.Add(new FreqResponseComplexData(32400000.1, new Complex(740338.006173017, -139261.198917357) ));
            calshort.Add(new FreqResponseComplexData(32800000.1, new Complex(516362.417110154, -520226.164731913) ));
            calshort.Add(new FreqResponseComplexData(33200000.1, new Complex(121119.606824578, -697528.287188057) ));
            calshort.Add(new FreqResponseComplexData(33600000.1, new Complex(-292701.78440861, -613147.292474583) ));
            calshort.Add(new FreqResponseComplexData(34000000.1, new Complex(-568041.158438542, -311100.325940425) ));
            calshort.Add(new FreqResponseComplexData(34400000.1, new Complex(-608391.993141762, 79949.1494035751) ));
            calshort.Add(new FreqResponseComplexData(34800000.1, new Complex(-412304.785318394, 401806.345095014) ));
            calshort.Add(new FreqResponseComplexData(35200000.1, new Complex(-78154.7291815975, 530022.527777682) ));
            calshort.Add(new FreqResponseComplexData(35600000.1, new Complex(242473.937604513, 427588.645436882) ));
            calshort.Add(new FreqResponseComplexData(36000000.1, new Complex(413018.764326233, 164636.668191243) ));
            calshort.Add(new FreqResponseComplexData(36400000.1, new Complex(376260.633295786, -117683.908760777) ));
            calshort.Add(new FreqResponseComplexData(36800000.1, new Complex(187217.223143435, -286480.417593081) ));
            calshort.Add(new FreqResponseComplexData(37200000.1, new Complex(-33709.8356882943, -288686.046065169) ));
            calshort.Add(new FreqResponseComplexData(37600000.1, new Complex(-174231.439337381, -166299.018719293) ));
            calshort.Add(new FreqResponseComplexData(38000000.1, new Complex(-196517.547013091, -15058.0947648501) ));
            calshort.Add(new FreqResponseComplexData(38400000.1, new Complex(-132687.116547358, 89055.170908247) ));
            calshort.Add(new FreqResponseComplexData(38800000.1, new Complex(-42356.3285215736, 124760.013194598) ));
            calshort.Add(new FreqResponseComplexData(39200000.1, new Complex(34063.4709984348, 107297.304583825) ));
            calshort.Add(new FreqResponseComplexData(39600000.1, new Complex(81971.4683947931, 60915.9120839375) ));
            calshort.Add(new FreqResponseComplexData(39999999.9, new Complex(39280.5705247694, 77507.0413221844) ));
            calshort.Add(new FreqResponseComplexData(49999999.9, new Complex(39280.5705247694, 77507.0413221844) ));
        }

    }
}
