﻿//================================================================================
//  
//        OscilloView.cs           オシロ画面
//  
//        (c)2009/07/26  by OjisanKoubou
//  
//================================================================================
//  
//  このプログラムは Ojisankoubou( OGAWA Ichiro ) が著作権を持っています。
//  個人の研究目的には、修正、改造など自由に使用できます。
//  発表される際は著作権表示部分を消さないようにしてください。
//  このソフトウェアの全部もしくは一部を製品に組み込んだり販売したりするなど
//  商業的利用はおやめください。
//

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Collections;
using System.IO;

namespace APB_3 {
    public partial class OscilloView : TimeGraphForm {
        private static int wn=0;
        private Thread Tmeasure;
        private volatile bool measuring = false;
        private volatile bool stopmeasuring = false;
        private volatile bool errormeasuring = false;
        private int samplingtime, samplingmode;
        private int triggermode, triggeredge;
        private int triggerpos;
        private int threshold;
        private int newtriggerpoint = -1;

        private double freq;
        private bool isDispTime = true;       // 時間表示 or 周波数表示
        
        private delegate void DelegateType();     // 別スレッドのコントロールを操作するのに使う
        
        
        //  コンストラクタ
        public OscilloView() {
            mainf = Apb3.mainf;
            InitializeComponent();
            base.gx = this.button_minus.Location.X; // グラフパネルの位置  dpi から計算するとずれるのでマイナスボタン位置に合わせた
            base.gy = this.button_minus.Location.Y + this.button_minus.Size.Height + 5;   
            
            wn++;
            this.Text = "オシロスコープ "+wn.ToString();
            mainf.LogPrint("\nOscilloView"+wn.ToString());
//            this.BackColor = clr.OSVPictBG;
            comboBox_samplingtime.Items.AddRange(def.ossamplingtimestrtbl);
//            comboBox_samplingmode.Items.AddRange(def.ossamplingmodestrtbl);
//            comboBox_freq.Items.AddRange(def.osfreqstrtbl);
            comboBox_triggermode.Items.AddRange(def.ossamplingmodestrtbl);
            comboBox_triggerpos.Items.AddRange(def.triggerposstrtbl);
            comboBox_threshold.Items.AddRange(def.osthresholdstrtbl);
            
            comboBox_samplingtime.SelectedIndex = 0;    // 10ns
//            comboBox_samplingmode.SelectedIndex = 0;    // ABS+PHA
//            comboBox_freq.SelectedIndex = 0;            // 変換しない
            comboBox_triggermode.SelectedIndex = 0;     // フリーラン
            comboBox_triggerpos.SelectedIndex = 2;      // 中央
            comboBox_threshold.SelectedIndex = 2;       // 128
            
            clkperiod = def.ossamplingtimetbl[0];       // 10ns サンプリングタイム
            wavelength = Apb3.buffsize/4;
            initwave();
            
//            wavelist.Add(new TimeGraphWave(this, 0x3FFFFFFF, 19, ClientSize.Height - (this.button_minus.Size.Height + 5) - base.gy - 18)); // 符号付アナログ　30ビット
            wavelist.Add( new TimeGraphWave( this, 0x0000FFFF, 19, ClientSize.Height - (this.button_minus.Size.Height + 5) - base.gy - 18) ); // 符号付アナログ　16ビット
            ((TimeGraphWave)wavelist[0]).nameedited = true;
            ((TimeGraphWave)wavelist[0]).name = "アナログ信号";
            ((TimeGraphWave)wavelist[0]).dso_perdivtbl = def.osperdivtbl;
            ((TimeGraphWave)wavelist[0]).dso_perdiv = 5000;
            ((TimeGraphWave)wavelist[0]).dso_perbit = 1;
            ((TimeGraphWave)wavelist[0]).dso_top    = 40000;
            ((TimeGraphWave)wavelist[0]).dso_ndiv   = 16;
            
//            wavelist.Add( new TimeGraphWave( this, 0x0FFF0000, 17, 180 ) ); // 符号なしアナログ　12ビット
//            ((TimeGraphWave)wavelist[1]).nameedited = true;
//            ((TimeGraphWave)wavelist[1]).name = "ヒストグラム";
//            ((TimeGraphWave)wavelist[1]).dso_perdiv = 20;
//            ((TimeGraphWave)wavelist[1]).dso_perdivindex = 5;    // 垂直軸感度インデックス
//            ((TimeGraphWave)wavelist[1]).dso_perbit = 1;
//            ((TimeGraphWave)wavelist[1]).dso_top    = 319;
//            ((TimeGraphWave)wavelist[1]).dso_ndiv   = 16;
//            ((TimeGraphWave)wavelist[1]).dso_perdivtbl= new double[] {1000, 500, 200, 100, 50, 20, 10, 5, 2, 1};      // 垂直軸感度テーブル
            
            curlist.Add( new TimeCursor( this, "R", 0 ) );
            curlist.Add( new TimeCursor( this, "M", 1 ) );
            curcursor = ((TimeCursor)curlist[0]);
            CursorHighlight( curcursor );
            
            WaveRefresh();
            Multi( 1 );
            
            ReSize();
            
        }
        
        
        // ボタン　設定
        private void button_setting_Click(object sender, EventArgs e) {
            
        }
        
        
        // ボタン　縮小
        private void button_minus_Click(object sender, EventArgs e) {
          MultiCursorFix( imulti-1 );
          UpdatePaint();
        }
        
        
        // ボタン　拡大
        private void button_plus_Click(object sender, EventArgs e) {
          MultiCursorFix( imulti+1 );
          UpdatePaint();
        }
        
        
        // ボタン　全体⇔縮小
        private void button_full_Click(object sender, EventArgs e) {
          base.MagFull();
        }
        
        
        // ボタン　Refカーソル
        private void button_ref_Click(object sender, EventArgs e) {
            curcursor = (TimeCursor)curlist[0];
            CursorHighlight(curcursor);
            movetoCursor( (TimeCursor)curcursor );
        }
        
        
        // ボタン　Mesカーソル
        private void button_mes_Click(object sender, EventArgs e) {
            curcursor = (TimeCursor)curlist[1];
            CursorHighlight(curcursor);
            movetoCursor( (TimeCursor)curcursor );
        }
        
        
        // ボタン　Mes-Refカーソル
        private void button_mesref_Click(object sender, EventArgs e) {
          int time0 = ((TimeCursor)curlist[0]).time;
          int time1 = ((TimeCursor)curlist[1]).time;
          ZoomBetween( time0, time1 );
          UpdatePaint();
        }
        
        
        // コンテクストメニューOpeningイベントハンドラー
        private void contextMenuStrip1_Opening(object sender, CancelEventArgs e) {
            contextMenuStrip1.Items.Clear();
            int waveno = ClickedWave(mousex, mousey);
            if (waveno >= 0) {
                cmenuWaveno = waveno;
                this.contextMenuStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
                        this.ToolStripMenuItem_mag,
                        this.ToolStripMenuItem_wavecolor,
                        this.ToolStripMenuItem_fullspan,
                        this.ToolStripMenuItem_refcursor,
                        this.ToolStripMenuItem_mescursor,
                        this.ToolStripMenuItem_freqc
                });
                foreach (ToolStripMenuItem item in ToolStripMenuItem_mag.DropDownItems) {
                  item.Checked = false;
                }
                ((ToolStripMenuItem)(ToolStripMenuItem_mag.DropDownItems[((TimeGraphWave)wavelist[cmenuWaveno]).dso_perdivindex])).Checked = true;

            } else {
                this.contextMenuStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
                        this.ToolStripMenuItem_fullspan,
                        this.ToolStripMenuItem_refcursor,
                        this.ToolStripMenuItem_mescursor,
                        this.ToolStripMenuItem_freqc
                });
            }
        }
        
        
        // コンテキストメニュー　拡大率変更
        private void ToolStripMenuItem_mag_Clicked(object sender, EventArgs e) {
            int index = ToolStripMenuItem_mag.DropDownItems.IndexOf(((ToolStripMenuItem)sender));
            ((TimeGraphWave)wavelist[cmenuWaveno]).dso_perdivindex = index;
            UpdatePaint();
            
            
        }
        
        
        // コンテキストメニュー　波形色
        private void ToolStripMenuItem_wavecolor_Click(object sender, EventArgs e) {
            WaveColor(cmenuWaveno);
        }
        
        
        // コンテキストメニュー　拡大⇔縮小
        private void ToolStripMenuItem_fullspan_Click(object sender, EventArgs e) {
            base.MagFull();
        }
        
        
        // コンテキストメニュー　ref カーソル
        private void ToolStripMenuItem_refcursor_Click(object sender, EventArgs e) {
//            base.sttime = ((TimeCursor)curlist[0]).time - (int)((gw-wavex-18)/multi/2);
//            base.UpdatePaint();
            curcursor = (TimeCursor)curlist[0];
            CursorHighlight(curcursor);
            movetoCursor( (TimeCursor)curcursor );
        }
        
        
        // コンテキストメニュー　mes カーソル
        private void ToolStripMenuItem_mescursor_Click(object sender, EventArgs e) {
//            base.sttime = ((TimeCursor)curlist[1]).time - (int)((gw-wavex-18)/multi/2);
//            base.UpdatePaint();
            curcursor = (TimeCursor)curlist[1];
            CursorHighlight(curcursor);
            movetoCursor( (TimeCursor)curcursor );
        }
        
        
        // コンテキストメニュー　時間表示モード⇔周波数表示モード
        private void ToolStripMenuItem_freqmode_Click(object sender, EventArgs e) {
            if( isDispTime ) {
              ToolStripMenuItem_freqmode.Text = "時間表示";
              isDispTime = false;
            }else{
              ToolStripMenuItem_freqmode.Text = "周波数表示";
              isDispTime = true;
            }
            UpdateBtnCursorText();
        }
        
        
        // コンテキストメニュー　周波数カウンター
        private void ToolStripMenuItem_freqc_Click(object sender, EventArgs e) {
            samplingtime = comboBox_samplingtime.SelectedIndex;
//            samplingmode = comboBox_samplingmode.SelectedIndex;
//            freq = util.ToDouble(comboBox_freq.Text);
            if (Apb3.Open()) {
                Apb3.CICWrite(samplingtime);      // デシメーション設定
                Apb3.dds0( freq );                       // freq
                Apb3.Close();
            }
//            if (samplingmode == 0) {             // ADC 出力を測定するモード
//                //freq = FreqCounter.Measure(16, 1);       // ADC出力
//            } else {
//                //freq = FreqCounter.Measure(17, 1);      // I+Q出力
//            }
            MessageBox.Show(freq.ToString("#0. 000 Hz"), "周波数");
        }
        
        
        
        // ボタン　測定開始／終了
        private void button_startstop_Click(object sender, EventArgs e) { 
            mainf.LogPrint("\nOscilloView.button_startstop_Click: measuring="+measuring.ToString());
            if (measuring) {
                stopmeasuring = true;  // 中断
            }
            else {
                measuring = true;
                stopmeasuring = false;
                errormeasuring = false;
                
                samplingtime = comboBox_samplingtime.SelectedIndex;
                triggermode   = comboBox_triggermode.SelectedIndex;
                triggerpos    = comboBox_triggerpos.SelectedIndex;
                threshold = comboBox_threshold.SelectedIndex;
                if (triggermode==2) {
                  triggermode = 1;
                  triggeredge = 1;
                }else if( triggermode==1) {
                  triggeredge = 0;
                }
                if (triggermode == 1)
                {
                    switch (triggerpos)
                    {
                        case 0: newtriggerpoint =    8; break;
                        case 1: newtriggerpoint = 1024; break;
                        case 2: newtriggerpoint = 4096; break;
                        case 3: newtriggerpoint = 7168; break;
                    }
                }
                else
                {
                    newtriggerpoint = -10000;
                }
//mainf.LogPrint("\ntriggermode="+triggermode.ToString() + " triggerpos="+triggerpos.ToString()  + " triggeredge="+triggeredge.ToString() + " threshold="+threshold.ToString());
//                samplingmode = comboBox_samplingmode.SelectedIndex;
//                freq = util.ToDouble( comboBox_freq.Text );
                
                clkperiod = def.ossamplingtimetbl[samplingtime];  // サンプリングタイム
                Tmeasure = new Thread(new ThreadStart(tMeasure));
                Tmeasure.Start();                 // 測定スレッドを開始する
            }
        }
        
        
        // 測定スレッド
        private void tMeasure() {
            if( Apb3.Open() ) {
                try {
                    measuring=true;
                    btnStartText( def.sweep, clr.MeasBtnExec );
                    Apb3.Adc1610.AdcReInit();                            // ADC 再設定
                    Apb3.CICWrite( samplingtime );          // デシメーション設定
                    Apb3.dds0( 0 );                         // 変換周波数
                    Apb3.Fpga.FpgaWrite(Apb3.Fpga.oscilloscope, (triggeredge<<7)+threshold );  // 
                    Apb3.MemwriteSetting( 0, triggerpos, triggermode ); // メモリー書き込み入力番号０
                    Apb3.MemwriteStart();                   // 測定開始
                    
                    if( StartAq() )
                    {
                        if( ReadData() >= 0 ) {
                            Invoke((DelegateType)delegate
                            {
                              triggerpoint = newtriggerpoint;
                              UpdatePaint();
                            });
                        }else{
                            errormeasuring = true;
                        }
                    }
                    
                    if( errormeasuring ) {
                        mainf.LogPrint("\nOscilloView.tMeasure: 通信エラー");
                        error.ErrorMessageBox(error.usb_disconnect); // 測定中に通信エラー
                        Apb3.CloseConnection();     // 一度クローズする
                    }
                    
                }
                catch (Exception ex) {
                    mainf.LogPrint("\nOscilloView.MeasureStart: ex=" + ex.Message);
                }
                finally {
                    Apb3.Close();
                    measuring=false;
                    btnStartText( def.measure, clr.MeasBtnNor );
                }
                
                
            }
        }
        
        
        // 
        private bool StartAq()
        {
            while (true)
            {
                int status = Apb3.Fpga.FpgaRead(Apb3.Fpga.memcontrol);
                if (status < 0) {
                    errormeasuring = true;
                    return false;  // read status ERROR ?
                }
                if ((status & 0x1) == 0) return true; // 測定終了した
                if ((status & 0x4) != 0)
                {
                    btnStartText(def.trgd, clr.MeasBtnExec);
                }
                else if ((status & 0x2) != 0)
                {
                    btnStartText(def.wait, clr.MeasBtnExec);
                }
                else if ((status & 0x1) != 0)
                {
                    btnStartText(def.pre, clr.MeasBtnExec);
                }
                
                if (stopmeasuring) return false;         // 測定中に中断した？
                
                Thread.Sleep(100);
            }
        }
        
        
        // FPGA からデータを読んで wavedata に格納する
        private int ReadData()
        {
            int ramadrs=0;
            ramadrs = Apb3.Fpga.FpgaRead(Apb3.Fpga.memcontrol+1, 2); // 最終書き込みアドレス+1
            
            Byte[] RBuf = new Byte[Apb3.buffsize];
            if (Apb3.Fpga.FpgaRead(Apb3.Fpga.memory, Apb3.buffsize, RBuf) != Apb3.buffsize) 
            {
                Apb3.mainf.LogPrint("\nOscilloView.ReadData: USB Read Error");
                return error.usb_readerr;
            }
            
            int in1, in2;
            int sum=0;
            for (int i = 0; i < Apb3.buffsize/4; i++) {
                ramadrs = ramadrs & 0x1FFF;
                in1  = RBuf[4*ramadrs];
                in1 += (((int)RBuf[4*ramadrs+1]) << 8);
                in2  = RBuf[4*ramadrs+2];
                in2 += (((int)RBuf[4*ramadrs+3]) << 8);
                
                bool sign = ((in1 & 0x8000) != 0 );
                if( (in1 & 0x8000) != 0 ) in1 -= 65536;
                if( (in2 & 0x8000) != 0 ) in2 -= 65536;
                
                wavedata[i] = (uint)(in1 & 0x0000FFFF);
                ramadrs++;
            }
            return 0;
        }
        
        
        
        // 測定開始ボタンのテキストを変更
        public void btnStartText(string str, Color clr ) {
            Invoke((DelegateType)delegate {
                button_startstop.Text = str;
                button_startstop.BackColor = clr;
            });
        }
        
        
        
        // TimeGraphForm から呼ばれる
        public override void ViewPaintAll() {
          UpdateBtnCursorText();
        }
        
        
        
        // カーソルボタンのテキスト変更
        public void UpdateBtnCursorText() {
          double r = ((TimeCursor)curlist[0]).time*base.clkperiod;
          double m = ((TimeCursor)curlist[1]).time*base.clkperiod;
          double mr = m-r;
          button_cursorref.Text = "R: " + util.Time2Str( r, base.clkperiod );
          button_cursormes.Text = "M: " + util.Time2Str( m, base.clkperiod );
          if( isDispTime) {             // 時間表示
            button_cursormesref.Text = "M-R: " + util.Time2Str( mr, base.clkperiod );
          }else{                        // 周波数表示
            button_cursormesref.Text = "M-R: " + util.Time2FreqStr( mr, base.clkperiod );
          }
        }
        
        
        
        private void initwave() {
        for( int i=0; i<wavelength; i++ ) {
          wavedata[i] = 0;
          }
        }
        
    }
}
