﻿//================================================================================
//  
//        SpeanaView.cs          スペアナ画面
//  
//        (c)2009/07/26  by OjisanKoubou
//  
//================================================================================
//  
//  このプログラムは Ojisankoubou( OGAWA Ichiro ) が著作権を持っています。
//  個人の研究目的には、修正、改造など自由に使用できます。
//  発表される際は著作権表示部分を消さないようにしてください。
//  このソフトウェアの全部もしくは一部を製品に組み込んだり販売したりするなどの
//  商業的利用はできません。
//

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.IO.Ports;
using System.Linq;
using System.Text;
using System.Threading;
using System.Windows.Forms;

namespace APB_3 {
    //-------------------------------------------------------------------------
    //  スペクトラムアナライザー　view(表示)部分
    //-------------------------------------------------------------------------
    public partial class SpeanaView : FreqResponseGraphForm {
        private static int wn_speana = 0;
        private static int wn_netana = 0;
        private Thread Tmeasure;
        private volatile bool measuring = false;
        private volatile bool stopmeasuring = false;
        private volatile bool errormeasuring = false;
        private int rbw = 0;
        private int validbin = 256;     // FFT した結果のうち何本のデータを有効とするか   8192 FFT なら 256
        private int averaging = 1;      // 平均化回数
        private bool keepresult = false;
        private int ntrace = 1;         // トレース保存数
        private int leftaxis = 0;
        private int rightaxis = 3;
        private double fstart = 0;          // 測定スレッドで使う変数
        private double fstop  = 0; //Apb3.nyquistfreq*0.8;       // 
        private double fstep  = 1;          // 
        
        private int npoint = 1;
        private bool singlesweep = true;
        private int outlevel = (def.hfsgleveltbl[0])<<16; // 出力レベル 0dB
        
        private double[] fftdata = new double[Apb3.buffsize / 2];  // MeasSpeana で使う配列、メモリー節約のためここで宣言しておく
        private  Byte[] rdBuffer = new Byte[Apb3.buffsize];
        private  Byte[] rdBuffer_d32 = new Byte[Apb3.buffsize];
        
        private delegate void DelegateType();     // 別スレッドのコントロールを操作するのに使う
        
        
        // コンストラクター
        public SpeanaView( bool netanamode ) {
            mainf = Apb3.mainf;
            mainf.LogPrint( "\nSpeanaView.SpeanaView: netanamode=" + netanamode.ToString() );
            base.netanamode = netanamode;
            InitializeComponent();
            Initialize();
            ReSize();
        }
        
        
        // 初期化
        private void Initialize() {
            comboBox_freqspan.Items.AddRange(def.freqspanstrtbl);
            
            ((ToolStripMenuItem)(ToolStripMenuItem_displayitems.DropDownItems[1])).Checked = true;  // 測定日時表示
            ((ToolStripMenuItem)(ToolStripMenuItem_outlevel.DropDownItems[0])).Checked = true;
            ((ToolStripMenuItem)(ToolStripMenuItem_persistence.DropDownItems[4])).Checked = true;   // 80%
            ((ToolStripMenuItem)(ToolStripMenuItem_trace.DropDownItems[0])).Checked = true;         // 1
            ((ToolStripMenuItem)(ToolStripMenuItem_div.DropDownItems[3])).Checked = true;
            ((ToolStripMenuItem)(ToolStripMenuItem_dbref.DropDownItems[4])).Checked = true;         // 0dB
            ((ToolStripMenuItem)(ToolStripMenuItem_dbdiv.DropDownItems[0])).Checked = true;
            ((ToolStripMenuItem)(ToolStripMenuItem_degref.DropDownItems[0])).Checked = true;
            ((ToolStripMenuItem)(ToolStripMenuItem_degdiv.DropDownItems[0])).Checked = true;
            ((ToolStripMenuItem)(ToolStripMenuItem_secref.DropDownItems[15])).Checked = true;       // 1000ns
            ((ToolStripMenuItem)(ToolStripMenuItem_secdiv.DropDownItems[15])).Checked = true;       // 100ns
            
            base.py = (int)(28 * (MainForm.dpi/96));
            
            if( netanamode ) {  // ネットアナモード
                wn_netana++;
                this.Text = "ネットワークアナライザ " + wn_netana.ToString();
                this.BackColor = clr.NAVPictBG;
                comboBox_n.Items.AddRange(def.nstrtbl);
                comboBox_n.Visible = true;
                singlesweep = true;
                label_n.Visible = true;
                comboBox_rbw.Items.AddRange(def.rbwstrtbl_netana);
                comboBox_rbw.SelectedIndex = 5;     // 1kHz このRBWまではスイープ速度が変わらないので
                rightaxis = 1; // 位相
                wavelist.Add(new FreqResponseGraphWave(0));  // scale=0:振幅
                wavelist.Add(new FreqResponseGraphWave(1, true));  // scale=1:位相
                wavelist.Add(new FreqResponseGraphWave(2));  // scale=2:群遅延
                ((FreqResponseGraphWave)(base.wavelist[2])).visible = false;
                base.scale.Add(new FreqResponseGraphScale(0, 10, "MAG", "dB", 0));
                base.scale.Add(new FreqResponseGraphScale(225, 45, "PHA", "deg", 1));
                base.scale.Add(new FreqResponseGraphScale(1000e-9, 100e-9, "DLY", "s", 1));
                ((FreqResponseGraphScale)(base.scale[2])).visible = false;
                base.cursor.Add(new FreqResponseGraphCursor((FreqResponseGraphWave)(wavelist[0])));
                base.cursor.Add(new FreqResponseGraphCursor((FreqResponseGraphWave)(wavelist[1])));
                base.curcursor = ((FreqResponseGraphCursor)(base.cursor[0]));
                ((FreqResponseGraphCursor)(base.cursor[0])).Highlight();
                ((FreqResponseGraphCursor)(base.cursor[0])).pos = 1;
                deltaf = false; // カーソルのデルタ表示はしない
                ToolStripMenuItem_delta.Checked = false;
                // 波形初期値
                for (int i = 0; i < 500; i++) {
                    ((FreqResponseGraphWave)wavelist[0]).Add(i * 1e6 / 10, -200);
                    ((FreqResponseGraphWave)wavelist[1]).Add(i * 1e6 / 10, 0);
                    ((FreqResponseGraphWave)wavelist[2]).Add(i * 1e6 / 10, 0);
                }
            }
            else {  // スペアナモード
                wn_speana++;
                this.Text = "スペクトラムアナライザ " + wn_speana.ToString();
                this.BackColor = clr.SAVPictBG;
                comboBox_n.Visible = false;
                singlesweep = false;
                label_n.Visible = false;
                comboBox_rbw.Items.AddRange(def.rbwstrtbl);
                comboBox_rbw.SelectedIndex = 0;  // 60kHz
                wavelist.Add(new FreqResponseGraphWave(0));  // scale=0:振幅
                base.scale.Add(new FreqResponseGraphScale(0, 10, "MAG", "dB", 0));
                base.cursor.Add(new FreqResponseGraphCursor((FreqResponseGraphWave)(wavelist[0])));
                base.cursor.Add(new FreqResponseGraphCursor((FreqResponseGraphWave)(wavelist[0])));
                base.curcursor = ((FreqResponseGraphCursor)(base.cursor[0]));
                ((FreqResponseGraphCursor)(base.cursor[0])).Highlight();
                ((FreqResponseGraphCursor)(base.cursor[1])).pos = 1;
                centerspan = true;
                // 波形初期値
                for (int i = 0; i < 500; i++) {
                    ((FreqResponseGraphWave)wavelist[0]).Add(i * 1e6 / 10, -200);
                }
            }
//            ToolStripMenuItem_sweep.Checked = singlesweep;
            checkBox_singlesweep.Checked = singlesweep;
            foreach (FreqResponseGraphWave frw in wavelist) frw.IncNsweep();  // スイープ数を１増やす
            freqcenterstartstop();    // 画面表示をセンター周波数表示、開始終了周波数表示に応じて切り替える
        }
        
        
        // コンテクストメニューをマウスクリック位置によって切り替える
        private void contextMenuStrip1_Opening(object sender, CancelEventArgs e) {
            contextMenuStrip1.Items.Clear();
            if (isFreqArea() ) { freq_contextMenu();      return; } // 周波数表示部
            if (isLeftArea() ) { leftaxis_contextMenu();  return; } // 左軸
            if (isRightArea()) { rightaxis_contextMenu(); return; } // 右軸
            if (isGraphArea()) { graph_contextMenu();     return; } // グラフ表示部分
            top_contextMenu();                                      // 画面上部
        }
        
        
        // 周波数表示部コンテクストメニュー項目
        private void freq_contextMenu() {
            contextMenuStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
                    ToolStripMenuItem_fullspan,
                    ToolStripMenuItem_freqcenter,
                    ToolStripMenuItem_freqaxis,
            } );
        }
        
        
        // 左軸コンテクストメニュー項目
        private void leftaxis_contextMenu() {
            switch( leftaxis ) {
            case 0:
                contextMenuStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
                    ToolStripMenuItem_dbref,
                    ToolStripMenuItem_dbdiv,
                    new ToolStripSeparator(),
                } );
                break;
            case 1:
                contextMenuStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
                    ToolStripMenuItem_degref,
                    ToolStripMenuItem_degdiv,
                    new ToolStripSeparator(),
                } );
                break;
            case 2:
                contextMenuStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
                    ToolStripMenuItem_secref,
                    ToolStripMenuItem_secdiv,
                    new ToolStripSeparator(),
                } );
                break;
            }
            if( netanamode ) {
                contextMenuStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
                    ToolStripMenuItem_axis,
                    new ToolStripSeparator(),
                } );
                foreach (ToolStripMenuItem item in ToolStripMenuItem_axis.DropDownItems) {
                  item.Checked = false;
                }
                ((ToolStripMenuItem)(ToolStripMenuItem_axis.DropDownItems[leftaxis])).Checked = true;
            }
            contextMenuStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
                ToolStripMenuItem_color,
                ToolStripMenuItem_div,
            } );
        }
        
        
        // 右軸コンテクストメニュー項目
        private void rightaxis_contextMenu() {
            switch( rightaxis ) {
            case 0:
                contextMenuStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
                    ToolStripMenuItem_dbref,
                    ToolStripMenuItem_dbdiv,
                    new ToolStripSeparator(),
                } );
                break;
            case 1:
                contextMenuStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
                    ToolStripMenuItem_degref,
                    ToolStripMenuItem_degdiv,
                    new ToolStripSeparator(),
                } );
                break;
            case 2:
                contextMenuStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
                    ToolStripMenuItem_secref,
                    ToolStripMenuItem_secdiv,
                    new ToolStripSeparator(),
                } );
                break;
            }
            if( netanamode ) {
                contextMenuStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
                    ToolStripMenuItem_axis,
                    new ToolStripSeparator(),
                } );
                foreach (ToolStripMenuItem item in ToolStripMenuItem_axis.DropDownItems) {
                  item.Checked = false;
                }
                ((ToolStripMenuItem)(ToolStripMenuItem_axis.DropDownItems[rightaxis])).Checked = true;
            }
            contextMenuStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
                ToolStripMenuItem_color,
                ToolStripMenuItem_div,
            } );
        }
        
        
        // グラフ表示部分コンテクストメニュー項目
        private void graph_contextMenu() {
            ToolStripMenuItem_toOffset.Checked = ( curcursor.wave.offset != 0 );
            contextMenuStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
                    ToolStripMenuItem_eraseold,
                    ToolStripMenuItem_eraseall,
                    new ToolStripSeparator(),
//                    ToolStripMenuItem_sweep,
                    ToolStripMenuItem_trace,
                    ToolStripMenuItem_persistence,
            } );
            if( netanamode ) {
                contextMenuStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
                    ToolStripMenuItem_outlevel,
                    new ToolStripSeparator(),
                    ToolStripMenuItem_mestocal,
                } );
            }
            contextMenuStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
                new ToolStripSeparator(),
                ToolStripMenuItem_toOffset,
                new ToolStripSeparator(),
                ToolStripMenuItem_nozoom,
                ToolStripMenuItem_zoom,
                ToolStripMenuItem_cursor2freqcenter,
                new ToolStripSeparator(),
                ToolStripMenuItem_harmonics,
            } );
            if( ((FreqResponseGraphCursor)(base.cursor[0])).wave == ((FreqResponseGraphCursor)(base.cursor[1])).wave ) {  // 同じ wave のときはデルタ表示可
                contextMenuStrip1.Items.Add( ToolStripMenuItem_delta );
            }
            contextMenuStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
                ToolStripMenuItem_cursorline,
                ToolStripMenuItem_cursorcross,
                ToolStripMenuItem_showvaluenearcur,
                new ToolStripSeparator(),
                ToolStripMenuItem_clipboard,
                new ToolStripSeparator(),
                ToolStripMenuItem_displayitems,
                ToolStripMenuItem_backcolor,
            } );
        }
        
        
        // 画面上部コンテクストメニュー項目
        private void top_contextMenu() {
            contextMenuStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
                    ToolStripMenuItem_fullspan,
                    ToolStripMenuItem_freqcenter,
                    ToolStripMenuItem_freqaxis,
                    new ToolStripSeparator(),
                    ToolStripMenuItem_eraseold,
                    ToolStripMenuItem_eraseall,
                    new ToolStripSeparator(),
//                    ToolStripMenuItem_sweep,
                    ToolStripMenuItem_trace,
                    ToolStripMenuItem_persistence,
            } );
            if( netanamode ) {
                contextMenuStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
                    ToolStripMenuItem_outlevel,
                    new ToolStripSeparator(),
                    ToolStripMenuItem_mestocal,
                } );
            }
            contextMenuStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
                new ToolStripSeparator(),
                ToolStripMenuItem_clipboard,
                new ToolStripSeparator(),
                ToolStripMenuItem_displayitems,
            } );
        }
        
        
        // コンテクストメニュー　軸表示項目選択
        private void ToolStripMenuItem_axis_Click(object sender, EventArgs e) {
            int index = ToolStripMenuItem_axis.DropDownItems.IndexOf(((ToolStripMenuItem)sender));
            mainf.LogPrint("\nSpeanaView.ToolStripMenuItem_axis_Click: " + index.ToString());
            if (isLeftArea()) {
                if (leftaxis <= 2) {
                    ((FreqResponseGraphWave)wavelist[leftaxis]).visible = false;
                    ((FreqResponseGraphScale)scale[leftaxis]).visible = false;
                }
                if (index <= 2) {
                    ((FreqResponseGraphWave)wavelist[index]).visible = true;
                    ((FreqResponseGraphScale)scale[index]).visible = true;
                    ((FreqResponseGraphScale)scale[index]).pos = 0;  // left axis
                    ((FreqResponseGraphCursor)(base.cursor[0])).wave = ((FreqResponseGraphWave)wavelist[index]);
                }
                if (rightaxis == index) {
                    rightaxis = 3;
                }
                leftaxis = index;
            } else if (isRightArea()) {
                if (rightaxis <= 2) {
                    ((FreqResponseGraphWave)wavelist[rightaxis]).visible = false;
                    ((FreqResponseGraphScale)scale[rightaxis]).visible = false;
                }
                if (index <= 2) {
                    ((FreqResponseGraphWave)wavelist[index]).visible = true;
                    ((FreqResponseGraphScale)scale[index]).visible = true;
                    ((FreqResponseGraphScale)scale[index]).pos = 1;  // right axis
                    ((FreqResponseGraphCursor)(base.cursor[1])).wave = ((FreqResponseGraphWave)wavelist[index]);
                }
                if (leftaxis == index) {
                    leftaxis = 3; // 非表示
                }
                rightaxis = index;
            }
            if ((rightaxis == 3) && (leftaxis <= 2)) {
                ((FreqResponseGraphCursor)(base.cursor[0])).wave = ((FreqResponseGraphWave)wavelist[leftaxis]);
                ((FreqResponseGraphCursor)(base.cursor[1])).wave = ((FreqResponseGraphWave)wavelist[leftaxis]);
            }
            if ((leftaxis == 3) && (rightaxis <= 2)) {
                ((FreqResponseGraphCursor)(base.cursor[0])).wave = ((FreqResponseGraphWave)wavelist[rightaxis]);
                ((FreqResponseGraphCursor)(base.cursor[1])).wave = ((FreqResponseGraphWave)wavelist[rightaxis]);
            }
            if ((leftaxis != 3) && (rightaxis != 3)) {
                deltaf = false; // カーソルのデルタ表示はしない
                ToolStripMenuItem_delta.Checked = false;
            }
            ReSize();
        }
        
        
        // コンテクストメニュー　dB Ref
        private void ToolStripMenuItem_dbref_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nSpeanaView.ToolStripMenuItem_dbref_Click:" );
            foreach (ToolStripMenuItem item in ToolStripMenuItem_dbref.DropDownItems) {
              item.Checked = (item.Equals(sender))?true :false;
            }
            double magref = util.ToDouble(((ToolStripMenuItem)sender).Text);
            ((FreqResponseGraphScale)scale[0]).top = magref;
            UpdateBitmap();
        }
        
        
        // コンテクストメニュー　dB/div
        private void ToolStripMenuItem_dbdiv_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nSpeanaView.ToolStripMenuItem_dbdiv_Click:" );
            foreach (ToolStripMenuItem item in ToolStripMenuItem_dbdiv.DropDownItems) {
              item.Checked = (item.Equals(sender))?true :false;
            }
            double perdiv = util.ToDouble(((ToolStripMenuItem)sender).Text);
            ((FreqResponseGraphScale)scale[0]).perdiv = perdiv;
            UpdateBitmap();
        }
        
        
        // コンテクストメニュー　deg Ref
        private void ToolStripMenuItem_degref_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nSpeanaView.ToolStripMenuItem_degref_Click:" );
            foreach (ToolStripMenuItem item in ToolStripMenuItem_degref.DropDownItems) {
              item.Checked = (item.Equals(sender))?true :false;
            }
            double degref = util.ToDouble(((ToolStripMenuItem)sender).Text);
            ((FreqResponseGraphScale)scale[1]).top = degref;
            UpdateBitmap();
        }
        
        
        // コンテクストメニュー　deg/div
        private void ToolStripMenuItem_degdiv_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nSpeanaView.ToolStripMenuItem_degdiv_Click:" );
            foreach (ToolStripMenuItem item in ToolStripMenuItem_degdiv.DropDownItems) {
              item.Checked = (item.Equals(sender))?true :false;
            }
            double degdiv = util.ToDouble(((ToolStripMenuItem)sender).Text);
            ((FreqResponseGraphScale)scale[1]).perdiv = degdiv;
            UpdateBitmap();
        }
        
        
        // コンテクストメニュー　sec Ref
        private void ToolStripMenuItem_secref_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nSpeanaView.ToolStripMenuItem_secref_Click:" );
            foreach (ToolStripMenuItem item in ToolStripMenuItem_secref.DropDownItems) {
              item.Checked = (item.Equals(sender))?true :false;
            }
            foreach (ToolStripMenuItem item in ToolStripMenuItem_secdiv.DropDownItems) {
              item.Checked = false;
            }
            int index = ToolStripMenuItem_secref.DropDownItems.IndexOf(((ToolStripMenuItem)sender));
            ((ToolStripMenuItem)(ToolStripMenuItem_secdiv.DropDownItems[index])).Checked = true;
            double secref = util.ToDouble(((ToolStripMenuItem)sender).Text);
            ((FreqResponseGraphScale)scale[2]).top = secref;
            ((FreqResponseGraphScale)scale[2]).perdiv = secref/10;
            UpdateBitmap();
        }
        
        
        // コンテクストメニュー　sec/div
        private void ToolStripMenuItem_secdiv_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nSpeanaView.ToolStripMenuItem_secdiv_Click:" );
            foreach (ToolStripMenuItem item in ToolStripMenuItem_secdiv.DropDownItems) {
              item.Checked = (item.Equals(sender))?true :false;
            }
            double secdiv = util.ToDouble(((ToolStripMenuItem)sender).Text);
            ((FreqResponseGraphScale)scale[2]).perdiv = secdiv;
            UpdateBitmap();
        }
        
        
        // コンテクストメニュー　波形色
        private void ToolStripMenuItem_color_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nSpeanaView.ToolStripMenuItem_color_Click:" );
            int waveno = isLeftArea()? leftaxis:rightaxis;
            ((FreqResponseGraphWave)(wavelist[waveno])).SetColor(
                  clr.colordialog(((FreqResponseGraphWave)(wavelist[waveno])).GetColor())
                  );
            UpdateBitmap();
        }
        
        
        // コンテクストメニュー　背景色
        private void ToolStripMenuItem_backcolor_Click(object sender, EventArgs e) {
            mainf.LogPrint("\nSpeanaView.ToolStripMenuItem_backcolor_Click:");
            FRGGraphBG = clr.colordialog( FRGGraphBG );
            UpdateBitmap();
        }
        
        
        // コンテクストメニュー　縦軸分割数
        private void ToolStripMenuItem_div_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nSpeanaView.ToolStripMenuItem_div_Click:" );
            foreach (ToolStripMenuItem item in ToolStripMenuItem_div.DropDownItems) {
              item.Checked = (item.Equals(sender))?true :false;
            }
            base.vdiv = Convert.ToInt32(((ToolStripMenuItem)sender).Text);
            UpdateBitmap();
        }
        
        
        // コンテクストメニュー　周波数軸　ログ
        private void ToolStripMenuItem_freqaxis_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nSpeanaView.ToolStripMenuItem_freqaxis_Click:" );
            base.logscale = !base.logscale;
            base.centerspan = false;
            freqcenterstartstop();    // 画面表示をセンター周波数表示⇔開始終了周波数表示で切り替える
            UpdateBitmap();
        }
        
        
        // コンテクストメニュー　開始周波数
        private void ToolStripMenuItem_startfreq_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nSpeanaView.ToolStripMenuItem_startfreq_Click:" );
            
        }
        
        
        // コンテクストメニュー　終了周波数
        private void ToolStripMenuItem_stopfreq_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nSpeanaView.ToolStripMenuItem_stopfreq_Click:" );
            
        }
        
        
        // コンテクストメニュー　周波数スパン
        private void ToolStripMenuItem_freqspan_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nSpeanaView.ToolStripMenuItem_freqspan_Click:" );
            
        }
        
        
        // コンテクストメニュー　フルスパン表示
        private void ToolStripMenuItem_fullspan_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nSpeanaView.nToolStripMenuItem_fullspan_Click:" );
            base.menuFullSpan();
        }
        
        
        // コンテクストメニュー　センター周波数⇔開始終了周波数
        private void ToolStripMenuItem_freqcenter_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nSpeanaView.ToolStripMenuItem_freqcenter_Click:" );
            base.centerspan = !base.centerspan;
            logscale = false;
            freqcenterstartstop();    // 画面表示をセンター周波数表示⇔開始終了周波数表示で切り替える
            UpdateBitmap();
        }
        
        
        // コンテクストメニュー　カーソルをセンター周波数へ
        private void ToolStripMenuItem_cursor2freqcenter_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nSpeanaView.ToolStripMenuItem_cursor2freqcenter_Click:" );
            base.menuCur2Center();
            freqcenterstartstop();    // 画面表示をセンター周波数表示⇔開始終了周波数表示で切り替える
            UpdateBitmap();
        }
        
        
        // コンテクストメニュー　カーソル間をズーム
        private void ToolStripMenuItem_zoom_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nSpeanaView.ToolStripMenuItem_zoom_Click:" );
            base.menuZoom();
        }
        
        
        // コンテクストメニュー　カーソル間のズームをやめる
        private void ToolStripMenuItem_nozoom_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nSpeanaView.ToolStripMenuItem_nozoom_Click:" );
            base.fstart = util.ToDouble(textBox_startfreq.Text);
            base.fstop = util.ToDouble(textBox_stopfreq.Text);
            UpdateBitmap();
        }
        
        
        // コンテクストメニュー　カーソルを中心に拡大
        private void ToolStripMenuItem_expand_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nSpeanaView.ToolStripMenuItem_expand_Click:" );
            menuExpand();
        }
        
        
        // コンテクストメニュー　カーソルを中心に縮小
        private void ToolStripMenuItem_shrink_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nSpeanaView.ToolStripMenuItem_shrink_Click:" );
            menuShrink();
        }
        
        
        // コンテクストメニュー　カーソル高調波表示
        private void ToolStripMenuItem_harmonics_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nSpeanaView.ToolStripMenuItem_harmonics_Click:" );
            base.harmonics = !base.harmonics;
            ToolStripMenuItem_harmonics.Checked = base.harmonics;
            PaintAll();
        }
        
        
        // コンテクストメニュー　カーソルを基準値補正する
        private void ToolStripMenuItem_toOffset_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nSpeanaView.ToolStripMenuItem_to0db_Click: "+  curcursor.data.ToString());
            base.SetOffset(curcursor.wave, (( curcursor.wave.offset == 0 )?curcursor.data:0));
        }
        
        
        // コンテクストメニュー　基準値補正をやめる
        private void ToolStripMenuItem_cancelOffset_Click(object sender, EventArgs e) {
            base.SetOffset(curcursor.wave, 0);
        }
        
        
        // コンテクストメニュー　次のピークへ
        private void ToolStripMenuItem_nextpeak_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nSpeanaView.ToolStripMenuItem_nextpeak_Click:" );
            base.menuNextPeak();
        }
        
        
        // コンテクストメニュー　デルタ表示
        private void ToolStripMenuItem_delta_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nSpeanaView.ToolStripMenuItem_delta_Click:" );
            if( ((FreqResponseGraphCursor)(base.cursor[0])).wave != ((FreqResponseGraphCursor)(base.cursor[1])).wave ) return;  // 同じ wave でないときはデルタ表示にしない
            deltaf = !deltaf;
            ToolStripMenuItem_delta.Checked = deltaf;
            PaintAll();
        }
        
        
        // コンテクストメニュー　カーソルライン表示
        private void ToolStripMenuItem_cursorline_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nSpeanaView.ToolStripMenuItem_cursorline_Click:" );
            cursorline = !cursorline;
            ToolStripMenuItem_cursorline.Checked = cursorline;
            PaintAll();
        }
        
        
        // コンテクストメニュー　カーソル×表示
        private void ToolStripMenuItem_cursorcross_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nSpeanaView.ToolStripMenuItem_cursorcross_Click:" );
            cursorcross = !cursorcross;
            ToolStripMenuItem_cursorcross.Checked = cursorcross;
            PaintAll();
        }
        
        
        // コンテクストメニュー　測定結果を残す
        private void ToolStripMenuItem_keepresult_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nSpeanaView.ToolStripMenuItem_keepresult_Click:" );
            keepresult = !keepresult;
            ToolStripMenuItem_keepresult.Checked = keepresult;
        }
        
        
        // コンテクストメニュー　トレースをすべて消す
        private void ToolStripMenuItem_eraseall_Click(object sender, EventArgs e) {
            if( !measuring ) {
                foreach (FreqResponseGraphWave frw in wavelist) frw.WaveClear();
                UpdateBitmap();
            }else{
                error.ErrorMessageBox(error.op_err);
            }
        }
        
        
        // コンテクストメニュー　古いトレースを消す
        private void ToolStripMenuItem_eraseold_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nSpeanaView.ToolStripMenuItem_eraseold_Click:" );
            if( !measuring ) {
                DelOldTraces(1);
                UpdateBitmap();
            }else{
                error.ErrorMessageBox(error.op_err);
            }
        }
        
        
        // コンテクストメニュー　現測定値で正規化キャリブレーション
        private void ToolStripMenuItem_mestocal_Click(object sender, EventArgs e) {
            if( !measuring ) {
                if( ToolStripMenuItem_mestocal.Checked ) {
                    ToolStripMenuItem_mestocal.Checked = false;
                    foreach (FreqResponseGraphWave frw in wavelist) frw.CalClear();
                }else{
                    ToolStripMenuItem_mestocal.Checked = true;
                    foreach (FreqResponseGraphWave frw in wavelist) frw.ToCal();
                }
                UpdateBitmap();
            }
        }
        
        
        // コンテクストメニュー　正規化キャリブレーションやめる
        private void ToolStripMenuItem_nocal_Click(object sender, EventArgs e) {
            if( !measuring ) {
                ToolStripMenuItem_mestocal.Checked = false;
                foreach (FreqResponseGraphWave frw in wavelist) frw.CalClear();
                UpdateBitmap();
            }
        }
        
        
        // コンテクストメニュー　測定ポイント数
        private void ToolStripMenuItem_mespoint_Click(object sender, EventArgs e) {
            foreach (ToolStripMenuItem item in ToolStripMenuItem_mespoint.DropDownItems) {
              item.Checked = (item.Equals(sender))?true :false;
            }
            
            
            
        }
        
        
        // コンテクストメニュー　アベレージング
        private void ToolStripMenuItem_averaging_Click(object sender, EventArgs e) {
            foreach (ToolStripMenuItem item in ToolStripMenuItem_averaging.DropDownItems) {
              item.Checked = (item.Equals(sender))?true :false;
            }
            
            
            
        }
        
        
        // コンテクストメニュー　出力レベル
        private void ToolStripMenuItem_outlevel_Click(object sender, EventArgs e) {
            foreach (ToolStripMenuItem item in ToolStripMenuItem_outlevel.DropDownItems) {
              item.Checked = (item.Equals(sender))?true :false;
            }
            outlevel = (def.hfsgleveltbl[ ToolStripMenuItem_outlevel.DropDownItems.IndexOf(((ToolStripMenuItem)sender)) ])<<16;
        }
        
        
        // コンテクストメニュー　シングルスイープ
        private void ToolStripMenuItem_sweep_Click(object sender, EventArgs e) {
            singlesweep = !singlesweep;
            ToolStripMenuItem_sweep.Checked = singlesweep;
        }
        
        
        // コンテクストメニュー　パーシスタンス
        private void ToolStripMenuItem_persistence_Click(object sender, EventArgs e) {
            foreach (ToolStripMenuItem item in ToolStripMenuItem_persistence.DropDownItems) {
              item.Checked = (item.Equals(sender))?true :false;
            }
            base.persistence = util.ToDouble(((ToolStripMenuItem)sender).Text)/100;
            if( base.persistence < 0.01 ) base.persistence = 0.01;
            if( !measuring ) UpdateBitmap();
        }
        
        
        // コンテクストメニュー　トレース保存数
        private void ToolStripMenuItem_trace_Click(object sender, EventArgs e) {
            foreach (ToolStripMenuItem item in ToolStripMenuItem_trace.DropDownItems) {
              item.Checked = (item.Equals(sender))?true :false;
            }
            ntrace = util.ToInt32(((ToolStripMenuItem)sender).Text);
            if( !measuring ) {
                DelOldTraces( ntrace );
                UpdateBitmap();
            }
        }
        
        
        // コンテクストメニュー　表示項目
        private void ToolStripMenuItem_displayitems_Click(object sender, EventArgs e) {
            int index = ToolStripMenuItem_displayitems.DropDownItems.IndexOf((ToolStripMenuItem)sender);
            switch( index ) {
            case 0:
                    CommentForm f = new CommentForm(this);
                    f.ShowDialog();
                ((ToolStripMenuItem)(ToolStripMenuItem_displayitems.DropDownItems[index])).Checked = base.commentvisible;
                break;
            case 1:
                base.datetimevisible = !base.datetimevisible;
                ((ToolStripMenuItem)(ToolStripMenuItem_displayitems.DropDownItems[index])).Checked = base.datetimevisible;
                break;
            case 2:
                base.conditionvisible = !base.conditionvisible;
                ((ToolStripMenuItem)(ToolStripMenuItem_displayitems.DropDownItems[index])).Checked = base.conditionvisible;
                break;
            }
            if( !measuring ) UpdateBitmap();
        }
        
        
        // コンテクストメニュー　コメント表示
        private void ToolStripMenuItem_comment_Click(object sender, EventArgs e) {
        }
        
        
        // コンテクストメニュー　測定日時表示
        private void ToolStripMenuItem_datetime_Click(object sender, EventArgs e) {
        }
        
        
        // コンテクストメニュー　設定
        private void toolStripMenuItem_setting_Click(object sender, EventArgs e) {
        }
        
        
        // コンテクストメニュー　カーソルの下に値表示
        private void ToolStripMenuItem_showvaluenearcur_Click(object sender, EventArgs e) {
            mainf.LogPrint( "\nSpeanaView.ToolStripMenuItem_showvaluenearcur_Click:" );
            showlevelnearcur = !showlevelnearcur;
            ToolStripMenuItem_showvaluenearcur.Checked = showlevelnearcur;
        }
        
        
        // コンテクストメニュー　測定値をクリップボードにコピー
        private void ToolStripMenuItem_clipboard_Click(object sender, EventArgs e) {
            copy2clipborad();
        }
        
        
        // センター・スパン テキスト変更イベント
        private void comboBox_centerspan_Changed(object sender, EventArgs e) {
            mainf.LogPrint( "\nSpeanaView.comboBox_centerspan_TextChanged: " + textBox_centerfreq.Text+ " " + comboBox_freqspan.Text );
            double centerfreq = util.ToDouble(textBox_centerfreq.Text);
            double freqspan = util.ToDouble(comboBox_freqspan.Text)+0.1;
            if( Math.Floor(freqspan/2+0.5) > Math.Floor(freqspan/2) ) freqspan += 1; // 奇数→偶数
            freqspan = Math.Floor(freqspan);
            if( freqspan==0 ) freqspan=Apb3.clockfreq/2;
            base.fstart = centerfreq - freqspan/2;
            base.fstop  = centerfreq + freqspan/2;
            if( !measuring ) UpdateBitmap();
        }
        
        
        // 開始周波数変更イベント
        private void textBox_startfreq_TextChanged(object sender, EventArgs e) {
            mainf.LogPrint( "\nSpeanaView.textBox_startfreq_TextChanged: " + textBox_startfreq.Text );
            base.fstart = util.ToDouble(textBox_startfreq.Text);
            if( base.fstart>base.fstop ) return;
            if( !measuring ) UpdateBitmap();
        }
        
        
        // 終了周波数変更イベント
        private void textBox_stopfreq_TextChanged(object sender, EventArgs e) {
            mainf.LogPrint( "\nSpeanaView.textBox_stopfreq_TextChanged: " + textBox_stopfreq.Text );
            base.fstop = util.ToDouble(textBox_stopfreq.Text);
            if( base.fstart>base.fstop ) return;
            if( !measuring ) UpdateBitmap();
        }
        
        
        // ＲＢＷ変更イベント
        private void comboBox_rbw_SelectedIndexChanged(object sender, EventArgs e) {
            rbw = comboBox_rbw.SelectedIndex;
        }
        
        
        // シングルスイープ変更イベント
        private void checkBox_singlesweep_CheckedChanged(object sender, EventArgs e)
        {
            singlesweep = checkBox_singlesweep.Checked;
        }
        
        
        // ボタン　測定開始
        private void btnMeasureStartClick(object sender, System.EventArgs e) {
            mainf.LogPrint( "\n\nSpeanaView.btnMeasureStartClick:" + DateTime.Now.ToString() );
            if (measuring) {
                stopmeasuring = true;  // 中断
            } else if( base.fstart>base.fstop ) {
                error.ErrorMessageBox(error.mes_noexec);
            } else {
                if( !Apb3.Open() ) {
                    mainf.LogPrint( "\nERROR: can't open Apb3 ");
                    return;
                }
                Apb3.Close();
                
                measuring = true;
                stopmeasuring = false;
                errormeasuring = false;
                base.dt = DateTime.Now;
                rbw = comboBox_rbw.SelectedIndex;
                fstart = base.fstart;  // コントロールに値を設定するとイベント処理が入り
                fstop = base.fstop;    // 変更されてしまうため保存しておく
                textBox_centerfreq.Text = util.Double2String_000((fstart + fstop)/2);
                comboBox_freqspan.Text  = util.Double2String_000((fstop  - fstart));
                textBox_startfreq.Text  = util.Double2String_000(fstart);
                textBox_stopfreq.Text   = util.Double2String_000(fstop);
                npoint = Convert.ToInt16(comboBox_n.Text);
                fstep = (fstop - fstart) / npoint;
                
                Tmeasure = new Thread(new ThreadStart(tMeasure));
                Tmeasure.Start();                 // 測定スレッドを開始する
                
                
            }
        }
        
        
        // 測定スレッド
        private void tMeasure() {
            if( Apb3.Open() ) {
                Stopwatch sw = new Stopwatch();
                sw.Start();
                
                try {
                    btnStartText( def.stop, clr.MeasBtnExec );
                    Apb3.Adc1610.AdcReInit();                            // ADC 再設定
                    if( netanamode ) {                                // ネットアナのときは
                        FFT.n = Apb3.nFFT;                            // 
                        Apb3.ad9707Write( 2 );                        // DDS0 出力に切り替える
                    }else{                                            // スペアナのときは
                        FFT.n = (rbw == 13)? Apb3.nFFT: Apb3.nFFT/2;  // RBW=7Hz 以外はFFT幅を半分にしてオーバーラップさせる
                        Apb3.ad9707Write( 3 );                        // DDS1 出力に切り替える
                    }
                    FFT.WindowFunc(FFT.n, 4, 4.5);         // ガウス窓作成
                    
                    Apb3.MemwriteSetting( 0 );              // メモリー書き込み入力番号０、即書き込み開始モード
                    
                    do {
                        foreach (FreqResponseGraphWave frw in wavelist) frw.IncNsweep();  // スイープ数を１増やす
                        if( netanamode ) MeasNetana();  // ----- ネットアナ測定 -----
                        else             MeasSpeana();  // ----- スペアナ測定 -----
                        
                        if( errormeasuring ) {              // 測定中に通信エラー
                            error.ErrorMessageBox(error.usb_disconnect);
                            break;
                        }
                        
                        if( stopmeasuring ) {   // 測定中に中断した
                            foreach (FreqResponseGraphWave frw in wavelist) frw.tracecount(); // nsweep の整合性を取る
                            DelOldTraces( ntrace );
                            Invoke((DelegateType)delegate {
                                UpdateBitmap();
                            });
                            Apb3.ad9707Write( 3 );                        // DDS1 出力に切り替える
                            break;
                        }
                        DelOldTraces( ntrace );
                        Invoke((DelegateType)delegate {
                            UpdateBitmap();
                        });
                    } while( !singlesweep );
                    
                }
                catch (Exception ex) {
                    mainf.LogPrint("\nSpeanaView.tMeasure: exception=" + ex.Message);
                }
                finally {
                    Apb3.Close();
                    btnStartText( def.measure, clr.MeasBtnNor );
                    mainf.LogPrint("\nSpeanaView: " + ((double)sw.ElapsedTicks / (double)Stopwatch.Frequency * 1000).ToString("0.0") + " msec");
                    sw.Stop();
                }
            }
            measuring=false;
        }
        
        
        // ネットアナ測定
        private void MeasNetana() {
            try {
                Apb3.CICWrite( rbw );               // RBW によってデシメーション設定
                Apb3.dds_setting( 1, outlevel );    // 出力レベル設定
                double fmulti=1;
                double freq;
                if( logscale ) {
                    if (fstart < 1) fstart = 1;
                    fmulti = Math.Pow( 10, (Math.Log10( fstop ) - Math.Log10( fstart ))/npoint );
                    freq = fstart/fmulti;
                } else {
                    freq = fstart-fstep;
                    if (freq < 0) freq = 0;
                }
                int adinput=0;
                int in1, in2;
                
                double prevpha = 0;
                double isig = 0, qsig = 0;
                double abs = 0;
                double mag = 0;
                double pha = 0;
                double diffpha = 0;
                double gdl = 0;
                
                for( int n=0; n < npoint+2; n++ ) {
                    Apb3.dds0( freq );
                    Apb3.Fpga.FpgaWrite(Apb3.Fpga.memcontrol+1, (0<<7));  // D36=0
                    Apb3.MemwriteStart();           // 測定開始
                    if( Apb3.MemoryReadready( rdBuffer )<0 ) {
                        mainf.LogPrint("\nSpeanaView:MeasNetana: 通信エラー");
                        errormeasuring = true;   // 通信エラーなので中断し
                        Apb3.CloseConnection();    // 一度クローズする
                        break;
                    }
                    Apb3.Fpga.FpgaWrite(Apb3.Fpga.memcontrol+1, (1<<7));  // D36=1
                    Apb3.MemoryRead(rdBuffer_d32);
                    
                    isig = 0; qsig = 0;
                    for (int i = 0; i < Apb3.buffsize/4; i++) {
                        in1  = rdBuffer_d32[4*i] & 0x0F;
                        in1 += rdBuffer[4*i]<<4;
                        in1 += (((int)rdBuffer[4*i+1]) << 12);
                        in2  = (rdBuffer_d32[4*i] & 0xF0)>>4;
                        in2 += rdBuffer[4*i+2]<<4;
                        in2 += (((int)rdBuffer[4*i+3]) << 12);
                        if ((in1 & 0x00080000) != 0) in1 -= 1048576;
                        if ((in2 & 0x00080000) != 0) in2 -= 1048576;
                        
                            isig += ((double)in1) * FFT.window[i];  // ノイズが入ってきたときにその影響を小さくできる
                            qsig += ((double)in2) * FFT.window[i];
                    }
                    abs = isig * isig + qsig * qsig;
//                    mag = ((abs==0)? -200: 10 * Math.Log10(abs) ) - 173.6853; // 20*log(524288*8192/2/3.1416/1.4142)
                    mag = ((abs==0)? -200: 10 * Math.Log10(abs) ) - 175.43; // 20*log(524288*8192/2/3.1416/1.4142)
                    pha = ((qsig==0)&&(isig==0))? 0: Math.Atan2(-qsig, isig)*180.0/Math.PI;
                    diffpha = prevpha-pha;
                    if( diffpha < -180 ) diffpha += 360;
                    gdl = diffpha/360/fstep;  // s
                    prevpha = pha;
                    
                    ((FreqResponseGraphWave)wavelist[0]).Add(freq, mag);
                    ((FreqResponseGraphWave)wavelist[1]).Add(freq, pha);
                    ((FreqResponseGraphWave)wavelist[2]).Add(freq, gdl);
                    Invoke((DelegateType)delegate {
                        UpdateBitmap();
                    });
                    if( logscale ) {
                        freq = freq*fmulti;
                    } else {
                        freq = freq+fstep;
                    }
                    if( freq > Apb3.clockfreq/2 ) break;
                    
                    if( stopmeasuring ) break;         // 測定中に中断した？
                }
            }
            catch (Exception ex) {
                errormeasuring = true;
                mainf.LogPrint("\nMeasSpeana: exception=" + ex.Message);
            }
        }
        
        
        // スペアナ測定　　RBW=7Hz 以外は７５％オーバーラップ
        private void MeasSpeana() {
            try {
                Apb3.CICWrite( rbw );             // RBW によってデシメーション設定
                FFT.n = (rbw == 13)? Apb3.nFFT: Apb3.nFFT/2;     // RBW=7Hz 以外はオーバーラップさせるためFFT幅を半分にする
                int nfbin = (rbw == 0)? FFT.n/2 : (rbw == 13)? validbin : validbin/2;// デシメーションしないときは全FFT 結果を有効とする
                double df = Apb3.clockfreq / def.decimationtbl[rbw] * nfbin / FFT.n; // 一回のFFTで得られるスペクトラム幅
                int n = (int)(((fstop-fstart)/2/df+1)/2);   // センター周波数をFFTのゼロ周波数にしたときに必要な計算回数
                double freq = (rbw == 0)? 0 : ((fstop + fstart)/2-n*2*df); // 全帯域のときは周波数変換しない
                int in1, in2;
                double[] indata = new double[Apb3.buffsize / 2];
                double[] fftabs = new double[Apb3.buffsize / 2];
                do {
                    Apb3.dds0( freq );
                    Apb3.Fpga.FpgaWrite(Apb3.Fpga.memcontrol+1, (0<<7));  // D32=0　下位３２ビットを読む
                    Apb3.MemwriteStart();         // 測定開始
                    if( Apb3.MemoryReadready(rdBuffer)<0 ) {
                        mainf.LogPrint("\nSpeanaView:MeasSpeana: 通信エラー");
                        errormeasuring = true;   // 通信エラーなので中断し
                        Apb3.CloseConnection();    // 一度クローズする
                        break;
                    }
                    Apb3.Fpga.FpgaWrite(Apb3.Fpga.memcontrol+1, (1<<7));  // D32=1　上位８ビットを読む
                    Apb3.MemoryRead(rdBuffer_d32);
                    for (int i = 0; i < Apb3.buffsize/4; i++) {
                        in1  = rdBuffer_d32[4*i] & 0x0F;
                        in1 += rdBuffer[4*i]<<4;
                        in1 += (((int)rdBuffer[4*i+1]) << 12);
                        in2  = (rdBuffer_d32[4*i] & 0xF0)>>4;
                        in2 += rdBuffer[4*i+2]<<4;
                        in2 += (((int)rdBuffer[4*i+3]) << 12);
                        if ((in1 & 0x00080000) != 0) in1 -= 1048576;
                        if ((in2 & 0x00080000) != 0) in2 -= 1048576;
                        indata[2*i    ] = ((double)in1);
                        indata[2*i + 1] = ((double)in2);
                    }
                    
                    for (int i = 0; i < nfbin*2; i++) {
                        fftabs[i] = 0;
                    }
                    
                    for( int ov = 0; ov < 5; ov++ ) {   // ７５％オーバーラップ繰り返し
                        for( int i = 0; i < FFT.n; i++ ) {  // FFT するためのデータコピー
                            fftdata[2*i  ] = indata[2*i   + Apb3.nFFT/8*ov] * FFT.window[i];
                            fftdata[2*i+1] = indata[2*i+1 + Apb3.nFFT/8*ov] * FFT.window[i];
                        }
                        FFT.exec(FFT.n, fftdata);
                        for (int i = 0; i < nfbin*2; i++) {
                            int j = (i < nfbin) ? 2 * (i + FFT.n - nfbin) : 2 * (i - nfbin);
                            fftabs[i] += fftdata[j] * fftdata[j] + fftdata[j + 1] * fftdata[j + 1];
                        }
                        if( rbw==13 ) break;  // RBW=7Hz
                    }
                    
                    for (int i = 0; i < nfbin*2; i++) {
                        double f = freq + (i - nfbin) * df / nfbin;
                        if( (f >= 0)&&(f < Apb3.clockfreq/2) ) {
//                            double mag = ((fftabs[i]==0)? -200: 10 * Math.Log10(fftabs[i]) - 174.6544); // 20*log(524288*4096/2/pai/sqrt(2)*sqrt(5))
                            double mag = ((fftabs[i]==0)? -200: 10 * Math.Log10(fftabs[i]) - 176.4); // 20*log(524288*4096/2/pai/sqrt(2)*sqrt(5))
                            if( rbw==13 ) mag += 0.9691;  // 20*log(sqrt(5)) - 20*log(2)  オーバーラップ回数とFFT数差
                            ((FreqResponseGraphWave)wavelist[0]).Add(f, mag);
                        }
                    }
                    
                    Invoke((DelegateType)delegate {
                        UpdateBitmap();
                    });
                    if( stopmeasuring ) break;          // 測定中に中断した？
                    freq += 2*df;
                    if( freq >= Apb3.clockfreq/2+df ) break;
                    if( freq >= Apb3.clockfreq/2 ) freq = Apb3.clockfreq/2+df/128; // とりあえず
                } while (freq - df < fstop);
            }
            catch (Exception ex) {
                errormeasuring = true;
                mainf.LogPrint("\nMeasSpeana: exception=" + ex.Message);
            }
        }
        
        
        // 古いトレースを削除
        private void DelOldTraces( int n ) {
            foreach (FreqResponseGraphWave frw in wavelist) frw.DelOldTraces( n );
        }
        
        
        // 測定開始ボタンのテキスト、色を変更
        private void btnStartText( string str, Color clr ) {
            try {
                Invoke((DelegateType)delegate {
                    measureButton.Text = str;
                    measureButton.BackColor = clr;
                });
            }
            catch (Exception ex) {
                mainf.LogPrint("\nbtnStartText: exception=" + ex.Message);
            }
        }
        
        
        // 画面表示をセンター周波数表示⇔開始終了周波数表示で切り替える
        private void freqcenterstartstop() {
            if( base.centerspan ) {
              label_start_center.Text = "center";
              label_stop_span.Text = "span";
              textBox_startfreq.Visible = false;
              textBox_stopfreq.Visible = false;
              textBox_centerfreq.Visible = true;
              comboBox_freqspan.Visible = true;
            } else {
              label_start_center.Text = "start";
              label_stop_span.Text = "stop";
              textBox_startfreq.Visible = true;
              textBox_stopfreq.Visible = true;
              textBox_centerfreq.Visible = false;
              comboBox_freqspan.Visible = false;
            }
        }
        
        
        // 
        public override void MeasureButtonFocus() {
            measureButton.Focus();
        }
        
        
        
        private void SpeanaView_FormClosing(object sender, FormClosingEventArgs e) {
            if (measuring) {
                Tmeasure.Abort();
            }
        }
        
        
        private void test() {
            FreqResponseGraphWave frw = (FreqResponseGraphWave)(wavelist[0]);
            
            mainf.LogPrint("\ntest: nsweep=" + frw.nsweep.ToString() + " wave.Count=" + frw.wave.Count.ToString());
            if( frw.wave.Count == 0 ) {
                return;
            }
            int count=1;
            double f1 = ((FreqResponseGraphData)(frw.wave[0])).freq;
            for( ; count< frw.wave.Count; count++ ) {           // 次のトレースまでのデータ個数
                double f2 = ((FreqResponseGraphData)(frw.wave[count])).freq;
                if( f1 > f2 ) {
                    mainf.LogPrint("\ntest: count=" + count.ToString() + " f1=" + f1.ToString() + " f2=" + f2.ToString());
                }
                f1 = f2;
            }
        }

    }

}
