﻿//================================================================================
//  
//        TimeGraphWave.cs           タイムスケールグラフ波形
//  
//        (c)2009/07/26  by OjisanKoubou
//  
//================================================================================
//  
//  このプログラムは Ojisankoubou( OGAWA Ichiro ) が著作権を持っています。
//  個人の研究目的には、修正、改造など自由に使用できます。
//  発表される際は著作権表示部分を消さないようにしてください。
//  このソフトウェアの全部もしくは一部を製品に組み込んだり販売したりするなど
//  商業的利用はおやめください。
//


using System;
using System.Collections;
using System.Collections.Generic;
using System.IO.Ports;
using System.Linq;
using System.Text;
using System.Threading;
using System.Drawing;
using System.Drawing.Drawing2D;


namespace APB_3
{
    //-------------------------------------------------------------------------
    // TimeGraph用波形表示クラス
    //-------------------------------------------------------------------------
    public class TimeGraphWave
    {
        public TimeGraphForm graph;
        public int y;                    // graph での表示位置
        public int h = 20;               // 表示高さ
        public int wsstr = (int)(68 * (MainForm.dpi/96)); // スケール文字列の幅
        
        public Bitmap bitmap = new Bitmap(1, 1);        // 波形描画用
        public SolidBrush brNameBG = new SolidBrush(clr.NameNormalBG);  // 信号名背景色
        public SolidBrush brWaveBG = new SolidBrush(clr.WaveNormalBG);  // 波形背景色
        public Pen penWave = new Pen(clr.getcolor());                  // 波形色ペン
        public SolidBrush brWaveValue = new SolidBrush(clr.WaveValue); // 波形値色ブラッシュ
        public string name;             // 信号名
        public bool nameedited = false; // 信号名がユーザーによって変更された
        public long mask;               // データビットマスク
        private int shift;              // データビット位置
        private int nbit;               // ビット数
        public int radix = 2;           // 基数( 2:2進  8:8進  10:10進  11:符号付10進  16:16進  17:アナログ  18:符号付アナログ  19:dso)
        public double gain = 1;         // アナログ表示 gain
        public int hokan = 2;           // 補間関数　0:直線　1:ハニング　2:ハミング　3:フラットトップ　4:

        public double dso_perdiv    = 32;   // 1div あたりいくつか
        public double dso_perbit    = 1;    // 1bit あたりいくつか
        public double dso_top       = 128;  // 画面の一番上
        public int dso_ndiv         = 8;    // 分割数
        public double[] dso_perdivtbl= new double[] {      // 垂直軸感度テーブル
            64, 32, 16, 8, 4, 2, 1};
        public int dso_perdivindex  = 0;    // 垂直軸感度インデックス
        public int dso_averaging    = 1;    // 平均化回数
        
        public string comment = "";
        


        // コンストラクタ
        public TimeGraphWave(TimeGraphForm graph, long mask)
        {
            this.graph = graph;
            this.mask = mask;
            SetVariables();
        }

        // コンストラクタ
        public TimeGraphWave(TimeGraphForm graph, long mask, int radix)
        {
            this.radix = radix;
            this.graph = graph;
            this.mask = mask;
            SetVariables();
        }

        // コンストラクタ
        public TimeGraphWave(TimeGraphForm graph, long mask, int radix, int h)
        {
            this.h = h;
            this.radix = radix;
            this.graph = graph;
            this.mask = mask;
            SetVariables();
        }


        // ユーザーによって変更されていなければ自動で名前をつける
        public void setname()
        {
            if (!nameedited)
            {
                if (nbit == 1)
                {
                    this.name = "D " + shift.ToString();
                }
                else
                {
                    this.name = "D " + (shift + nbit - 1).ToString() + ".." + shift.ToString();
                }
            }
        }


        // 内部変数(shift,nbi)の設定
        private void SetVariables()
        {
            long mask = (long)this.mask;
            shift = 0;
            while ((mask & 0x0001L) == 0)
            {
                shift++;
                mask = mask >> 1;
            }
            nbit = 0;
            while (mask != 0)
            {
                nbit++;
                mask = mask >> 1;
            }
            setname();
        }


        // 波形データから表示用データを作る
        public void Update()
        {
            SetVariables();
            bitmap.Dispose();                   // 今までのビットマップを捨てて
            bitmap = new Bitmap(graph.gw - 18, h);        // 新しくサイズ変更したビットマップをつくる
            Graphics gr = Graphics.FromImage(bitmap);
            Pen penWaveScale = new Pen(clr.WaveScale);
            //            penScale1.DashStyle = DashStyle.Dot;
            Pen penWaveSubScale = new Pen(clr.WaveSubScale);
            penWaveSubScale.DashStyle = DashStyle.Dot; //Dash;

            gr.FillRectangle(new SolidBrush(clr.GraphBG), 0, 0, graph.gw - 18, 1);
            gr.FillRectangle(brWaveBG, 0, 1, graph.gw - 18, h - 1);
            // 時間スケール
            int xs2 = graph.wavex - (int)(graph.sttime * graph.multi) + (int)((graph.sttime * graph.multi) / graph.divpix) * graph.divpix;
            while (xs2 < graph.gw - 18)
            {                // スケール表示
                //                gr.DrawLine(new Pen(clr.WaveScale), xs2, 1, xs2, h);
                gr.DrawLine(penWaveScale, xs2, 1, xs2, h);
                for (int i = 1; i < graph.nsubdiv; i++)
                {
                    int xs1 = xs2 + i * graph.divpix / graph.nsubdiv;
                    //                    gr.DrawLine(new Pen(clr.WaveSubScale), xs1, 1, xs1, h);
                    gr.DrawLine(penWaveSubScale, xs1, 1, xs1, h);
                }
                xs2 += graph.divpix;
            }
            // 信号名
            gr.FillRectangle(brNameBG, 0, 1, graph.wavex, h - 1);
            gr.DrawString(name, fnt.TGName, new SolidBrush(clr.Name), 2, ((h > 16) ? 5 : 0));
            // 波形
            if ( radix > 16 )  // アナログ表示もしくはＤＳＯモード？
            {
                UpdateAnalog(gr);
            }
            else if (nbit == 1)
            {
                UpdateDigital(gr);
            }
            else
            {
                UpdateDigitalNbit(gr);
            }
        }

        // 波形データからデジタル１ビット表示用 bitmap データを作る
        private void UpdateDigital(Graphics gr)
        {
            double m = graph.multi;
            int x1 = graph.wavex, x2 = x1, y1 = h - 1, y2 = y1;
            double xf = x2 - graph.multi;
            ArrayList alp = new ArrayList();
            for (int i = graph.sttime; i < graph.wavelength; i++)
            {
                xf += graph.multi;                    // 浮動小数点でのｘ
                x2 = (int)(xf + 0.5);
                if (x2 > graph.gw - 18) break;               // 表示範囲外？
                if (i < 0)
                {                       // 波形データの存在しないところ
                    x1 = x2;
                    continue;
                }
                //            y2 =( ((graph.wavedata[i]&mask)>>shift) == 0 )? h-1 : 4;
                y2 = (((graph.wavedata[i] & mask) >> shift) == 0) ? h - 1 : 5;
                if ((i == 0) || (i == graph.sttime))
                {     // 波形の先頭
                    x1 = x2;
                    y1 = y2;
                }
                if (y1 != y2)
                {
                    alp.Add(new Point(x1, y1));
                    alp.Add(new Point(x2, y1));
                    y1 = y2;
                    x1 = x2;
                }
            }
            x2 = (int)(xf + 0.5);
            alp.Add(new Point(x1, y1));
            alp.Add(new Point(x2, y1));
            Point[] points = new Point[alp.Count];
            for (int i = 0; i < alp.Count; i++) points[i] = (Point)alp[i];
            gr.DrawLines(penWave, points);    // bitmap に波形表示
        }


        // 波形データからデジタル複数ビット表示用 bitmap データを作る
        private void UpdateDigitalNbit(Graphics gr)
        {
            double m = graph.multi;
            int x1 = graph.wavex, x2 = x1, y1 = h - 1, y2 = y1;
            double xf = x2 - graph.multi;
            long d1 = 0, d2 = 0;
            string s;
            for (int i = graph.sttime; i < graph.wavelength; i++)
            {
                xf += graph.multi;                // 浮動小数点での次のｘ位置
                x2 = (int)(xf + 0.5);               // 画面上での次のｘ位置
                if (x2 > graph.gw - 18) break;        // 表示範囲外？
                if (i < 0)
                {                       // 波形データの存在しないところ
                    x1 = x2;
                    continue;
                }
                d2 = (graph.wavedata[i] & mask) >> shift;
                if ((i == 0) || (i == graph.sttime))
                {     // 波形の先頭
                    x1 = x2;
                    d1 = d2;
                    continue;
                }
                if (d1 != d2)
                {
                    if (d1 == 0)
                    {
                        gr.DrawLine(penWave, x1, h - 1, x2, h - 1); // x1,y1,x2,y2
                    }
                    else
                    {
                        gr.DrawRectangle(penWave, x1, 4, x2 - x1, h - 5);  // x,y,w,h
                        s = util.Int2String((int)d1, radix, nbit);
                        if (x2 - x1 > s.Length * 6)
                        {
                            gr.DrawString(s, fnt.TGWaveValue, brWaveValue, new RectangleF(x1, 4, x2 - x1 + 2, h - 5));
                        }
                    }
                    x1 = x2;
                    d1 = d2;
                }
            }
            x2 = (int)(xf + 0.5);
            if (d1 == 0)
            {
                gr.DrawLine(penWave, x1, h - 1, x2, h - 1);
            }
            else
            {
                gr.DrawRectangle(penWave, x1, 4, x2 - x1, h - 5);
                s = util.Int2String((int)d1, radix, nbit);
                if (x2 - x1 > s.Length * 6)
                {
                    gr.DrawString(s, fnt.TGWaveValue, brWaveValue, new RectangleF(x1, 4, x2 - x1 + 2, h - 5));
                }
            }
        }



        // 波形データからアナログ表示用 bitmap データを作る
        private void UpdateAnalog(Graphics gr)
        {
            //graph.mainf.debugprint("\nTimeGraphWave.Update: radix=" + radix.ToString());
            
            // スケール表示
              SetDsoPerdiv( dso_perdivtbl[dso_perdivindex] );
              double top = dso_top/dso_perdiv;
              int   itop = (int)( top + 0.000001 );  // 整数化したときの誤差対策のため十分小さい数を足す
              for( int s=0; s<dso_ndiv+1; s++ ){
                int ys   = (int)( ( top - itop + s  ) * h / dso_ndiv );
                if( (ys > 1)&&( ys < h ) ){
                  gr.DrawLine(new Pen(clr.AnalogWaveScale), graph.wavex - 3, ys, graph.gw - 18, ys);
                  string scalestr = util.Double2String((itop-s)*dso_perdiv);
                  gr.DrawString(scalestr.PadLeft(9), fnt.TGScale, new SolidBrush(clr.Name), graph.wavex - wsstr, ys - 7);
                }
              }
            
            double m = graph.multi;
            int x2 = graph.wavex, y2 = h - 4;
            double x2f = x2 - m;
            int i = graph.sttime;
            if (i < 0)
            {
                x2f += m * (-i);
                i = 0;
            }
            
            int amax = 1 << nbit;   // アナログ最大値
            ArrayList alp = new ArrayList();
            for (; i < graph.wavelength; i++)
            {
                x2f += m;
                x2 = (int)(x2f + 0.5);
                if (x2 > graph.gw - 18) break;      // 表示範囲を超えた
                y2 = (int)(h * (dso_top - AnalogData(i)*dso_perbit)/dso_perdiv/dso_ndiv);
                alp.Add(new Point(x2, y2));
                alp.Add(new Point(x2 + (int)m, y2));     // 補間なし
            }
            if (alp.Count != 0)
            {
                Point[] points = new Point[alp.Count];
                for (int k = 0; k < alp.Count; k++) points[k] = (Point)alp[k];
                gr.DrawLines(penWave, points);    // 波形を描く
            }
            
            // コメント表示
            gr.DrawString( comment, fnt.TGCursor, new SolidBrush(clr.Cursor), graph.wavex+10, 4);
            
        }


        // wavedata からアナログ信号を抜き出す。
//        private long AnalogData(int i)
//        {
//            if      (i < 0 )                i = 0;                // 範囲を超えた場合
//            else if (i >= graph.wavelength) i = graph.wavelength - 1;
//            
//            long val=0;
//            if (  dso_averaging > 1 )
//            {
//                int j = i - dso_averaging + 1;
//                if( j < 0 ) j = 0;
//                long amax = 1 << nbit;
//                for( ; j<= i; j++ )
//                {
//                    long v = (long)((graph.wavedata[j] & mask) >> shift);
//                    if (v >= amax / 2) v = v - amax;   // マイナスの場合
//                    val += v;
//                }
//                val /= Math.Min( i+1, dso_averaging );
//            }
//            else if ( (radix == 18) ||  (radix == 19) )  // 符号付 or DSO
//            {           // 符号付アナログ
//                val = (long)((graph.wavedata[i] & mask) >> shift);
//                long amax = 1 << nbit;
//                if (val >= amax / 2) val = val - amax;   // マイナスの場合
//            }
//            else
//            {                              // 符号なしアナログ
//                val = (long)((graph.wavedata[i] & mask) >> shift);
//            }
//            return val;
//        }
        
        private double AnalogData(int i)
        {
            if      (i < 0 )                i = 0;                // 範囲を超えた場合
            else if (i >= graph.wavelength) i = graph.wavelength - 1;
            
            double val=0;
            if (  dso_averaging > 1 )
            {
                int j = i - dso_averaging + 1;
                if( j < 0 ) j = 0;
                double amax = (double)(1 << nbit);
                for( ; j<= i; j++ )
                {
                    double v = (double)((graph.wavedata[j] & mask) >> shift);
                    if (v >= amax / 2) v = v - amax;   // マイナスの場合
                    val += v;
                }
                val /= Math.Min( i+1, dso_averaging );
            }
            else if ( (radix == 18) ||  (radix == 19) )  // 符号付 or DSO
            {           // 符号付アナログ
                val = (double)((graph.wavedata[i] & mask) >> shift);
                double amax = (double)(1 << nbit);
                if (val >= amax / 2) val = val - amax;   // マイナスの場合
            }
            else
            {                              // 符号なしアナログ
                val = (double)((graph.wavedata[i] & mask) >> shift);
            }
            return val;
        }


        // 表示
        public void Disp(Graphics gr, int x, int y)
        {
            this.y = y;
            gr.DrawImage(bitmap, x, y);  //波形部分
        }


        // ノーマルカラーにする
        public void Normal()
        {
            brNameBG.Color = clr.NameNormalBG;
            brWaveBG.Color = clr.WaveNormalBG;
            Update();
        }


        // ハイライトにする
        public void Highlight()
        {
            if( radix != 19 ) {   // dso モード以外
                brNameBG.Color = clr.NameHighlightBG;
                brWaveBG.Color = clr.WaveHighlightBG;
                Update();
            }
        }


        public bool isInside(int px, int py)
        {
            return ((py >= y) && (py <= y + h));
        }
        
        
        // DSO の perdiv を設定、センターは一定で拡大縮小する
        public void SetDsoPerdiv(double perdiv)
        {
            double center = dso_top - dso_ndiv*dso_perdiv/2;
            dso_perdiv = perdiv;
            dso_top = center + dso_ndiv * dso_perdiv / 2;
        }
        
        
    }
}
