----------------------------------------------------------------------------------
--  
--        cordic.vhd        CORDIC gĐΒlAΊpvZ    ilatency=18j
--  
--        (c)2011/06/10  by OjisanKoubou
--  
----------------------------------------------------------------------------------
--  
--  ̃vO Ojisankoubou( OGAWA Ichiro ) 쌠Ă܂B
--  ľړIɂ́ACAȂǎRɎgpł܂B
--  \ۂ͒쌠\Ȃ悤ɂĂB
--  ̃\tgEFȂS͈ꕔ𐻕iɑgݍ񂾂̔肷Ȃ
--  ƓIp͂߂B
--
--
--Z 18bit
--Selected Device : 3s200avq100-4 
--
-- Number of Slices:                      197  out of   1792    10%  
-- Number of Slice Flip Flops:             94  out of   3584     2%  
-- Number of 4 input LUTs:                371  out of   3584    10%  
-- Number of IOs:                          67
-- Number of bonded IOBs:                  67  out of     68    98%  
-- Number of GCLKs:                         1  out of     24     4%  
--
--   Minimum period: 10.150ns (Maximum Frequency: 98.522MHz)
--   Minimum input arrival time before clock: 7.079ns
--   Maximum output required time after clock: 5.531ns
--   Maximum combinational path delay: No path found
--
--

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

------------------------------------------------------------------------------
entity cordic is 
  port (
    CLK     : in  std_logic;                       -- 
    ND      : in  std_logic;                       -- ̓f[^L
    X_IN    : in  std_logic_vector( 15 downto 0 ); -- 
    Y_IN    : in  std_logic_vector( 15 downto 0 ); -- 
    ABS_OUT : out std_logic_vector( 15 downto 0 ); -- Βló@0.823(=1.647/2){Ă
    ARG_OUT : out std_logic_vector( 15 downto 0 ); -- argumentiΊpjo
    OD      : out std_logic                        -- o̓f[^L
  );
end cordic;


------------------------------------------------------------------------------
architecture RTL of cordic is
  
  subtype D18 is STD_LOGIC_VECTOR (17 downto 0);
  type ATNTABLE is array (0 to 15) of D18;
  constant ATN :ATNTABLE := (    -- Atan(2^-i)
    "001000000000000000", "000100101110010000", "000010011111101101", "000001010001000100", 
    "000000101000101100", "000000010100010111", "000000001010001100", "000000000101000110", 
    "000000000010100011", "000000000001010001", "000000000000101001", "000000000000010100", 
    "000000000000001010", "000000000000000101", "000000000000000011", "000000000000000001"
  );
  
  
  signal QUAD0 : std_logic_vector( 1 downto 0 ); -- ی
  signal QUAD  : std_logic_vector( 1 downto 0 ); -- ی
  signal ARG0  : std_logic_vector( 15 downto 0 ); -- 
  
  signal X_ABS : std_logic_vector( 15 downto 0 ); -- 
  signal Y_ABS : std_logic_vector( 15 downto 0 ); -- 
  signal X0 : std_logic_vector( 15 downto 0 ); -- 
  signal Y0 : std_logic_vector( 15 downto 0 ); -- 
  signal X  : std_logic_vector( 17 downto 0 ); -- 
  signal Y  : std_logic_vector( 17 downto 0 ); -- 
  signal Z  : std_logic_vector( 17 downto 0 ); -- 
  
  signal N  : std_logic_vector(  4 downto 0 ); -- 
  signal XN : std_logic_vector( 17 downto 0 ); -- N bit Vtg
  signal YN : std_logic_vector( 17 downto 0 ); -- N bit Vtg
  
  
  component shiftra18
  port (
    X   : in  std_logic_vector( 17 downto 0 ); -- 
    Y   : out std_logic_vector( 17 downto 0 ); -- o̓f[^
    N   : in  std_logic_vector( 3 downto 0 )   -- Vtg
  );
  end component;
  
  
begin
  
  u0 : shiftra18
  port map (
    X   => X,             -- 
    Y   => XN,            -- o̓f[^
    N   => N(3 downto 0)  -- Vtg
  );
  
  
  u1 : shiftra18
  port map (
    X   => Y,             -- 
    Y   => YN,            -- o̓f[^
    N   => N(3 downto 0)  -- Vtg
  );
  
  
  QUAD0 <= Y_IN(15) & X_IN(15);
  X_ABS <= X_IN when( X_IN(15) = '0' ) else (not X_IN) + 1;
  Y_ABS <= Y_IN when( Y_IN(15) = '0' ) else (not Y_IN) + 1;
  
  X0    <= X_ABS when( QUAD0 = "00" or QUAD0 = "11" ) else Y_ABS;
  Y0    <= Y_ABS when( QUAD0 = "00" or QUAD0 = "11" ) else X_ABS;
  
  ARG0  <= Z(17 downto 2)           when( QUAD = "00" ) else
           Z(17 downto 2) + X"4000" when( QUAD = "01" ) else
           Z(17 downto 2) + X"C000" when( QUAD = "10" ) else
           Z(17 downto 2) + X"8000";
           
  
  
  ------------------------------------------------------------------
  --    CORDIC ŐΒlAΊpvZ
  ------------------------------------------------------------------
  process( CLK ) begin
    if( rising_edge(CLK) ) then
      
      if( ND='1' ) then
        QUAD <= QUAD0;
        X <= "0" & X0 & "0";
        Y <= "0" & Y0 & "0";
        Z <= (others => '0');
        N <= (others => '0');
        
      elsif( N(4)='0' ) then
        if( Y(17)='0' ) then          -- Y >= 0
          X <= X + YN;
          Y <= Y - XN;
          Z <= Z + ATN(CONV_INTEGER(N(3 downto 0)));
        else                          -- Y < 0
          X <= X - YN;
          Y <= Y + XN;
          Z <= Z - ATN(CONV_INTEGER(N(3 downto 0)));
        end if;
        N <= N+1;
        
      else
        if( N(0)='0' ) then
          ABS_OUT <= X(17 downto 2);    -- Βl
          ARG_OUT <= ARG0;              -- Ίp
          OD <= '1';
          N <= N+1;
        else
          OD <= '0';
        end if;
        
      end if;
      
    end if;
  end process;
  
  
end RTL;
----------------------------- end of architecture ----------------------------

