----------------------------------------------------------------------------------
--  
--        fir255_hilbert.vhd        FIR tB^[ ( 2ch, 255 tap, 1024 clk )
--  
--        (c)2010/01/11  by OjisanKoubou
--  
----------------------------------------------------------------------------------
--  
--  ̃vO OjisanKoubou( OGAWA Ichiro ) 쌠Ă܂B
--  ľړIɂ́ACAȂǎRɎgpł܂B
--  \ۂ͒쌠\Ȃ悤ɂĂB
--  ̃\tgEFȂS͈ꕔ𐻕iɑgݍ񂾂̔肷ȂǏƓI
--  p͂߂B
--
--20100204 o16bits
--
-- Number of Slices:                      136  out of   2448     5%  
-- Number of Slice Flip Flops:            192  out of   4896     3%  
-- Number of 4 input LUTs:                164  out of   4896     3%  
-- Number of IOs:                         106
-- Number of bonded IOBs:                 102  out of     66   154% (*) 
-- Number of BRAMs:                         1  out of     12     8%  
-- Number of MULT18X18SIOs:                 1  out of     12     8%  
-- Number of GCLKs:                         1  out of     24     4%  
--
--   Minimum period: 9.527ns (Maximum Frequency: 104.964MHz)
--   Minimum input arrival time before clock: 4.827ns
--   Maximum output required time after clock: 4.935ns
--   Maximum combinational path delay: 7.247ns
--
--
--


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

library UNISIM;
use UNISIM.VComponents.all;

------------------------------------------------------------------------------
entity fir255_hilbert is 
  port (
    RESET   : in  STD_LOGIC;                        -- 
    CLK     : in  STD_LOGIC;                        -- 100MHz
    NDIN    : in  STD_LOGIC;                        -- ͓M
    DIN_I   : in  STD_LOGIC_VECTOR( 17 downto 0 );  -- f[^́@O
    DIN_Q   : in  STD_LOGIC_VECTOR( 17 downto 0 );  -- f[^́@P
    NDOUT   : out STD_LOGIC;                        -- o͓M
    DOUT_I  : out STD_LOGIC_VECTOR( 15 downto 0 );  -- tB^[ó@O
    DOUT_Q  : out STD_LOGIC_VECTOR( 15 downto 0 );  -- tB^[ó@P
    
    MEME    : in  STD_LOGIC;                        -- 
    MWE     : in  STD_LOGIC;                        -- 
    MADR    : in  STD_LOGIC_VECTOR ( 15 downto 0);  -- W[AhX
    DATAIN  : in  STD_LOGIC_VECTOR (  7 downto 0);  -- W[f[^
    DATAOUT : out STD_LOGIC_VECTOR (  7 downto 0)   -- W[f[^o
  );
end fir255_hilbert;


------------------------------------------------------------------------------
architecture RTL of fir255_hilbert is
  
  signal ND_D       : std_logic;                       -- 
  signal MEMADR     : STD_LOGIC_VECTOR( 9 downto 0 )  ; -- [AhX A
  signal MEMDATAOUT : STD_LOGIC_VECTOR( 17 downto 0 ) ; -- [f[^o A
  signal MEMWE      : STD_LOGIC_VECTOR( 0 downto 0 )  ; -- [ A
  signal DIN_Q_D    : STD_LOGIC_VECTOR( 17 downto 0 ) ; -- f[^1 ̃obt@
  signal DOUT_I_D   : STD_LOGIC_VECTOR( 15 downto 0 ) ; -- f[^o0 ̃obt@
  signal DINA       : STD_LOGIC_VECTOR( 17 downto 0 ) ; -- [f[^ B
  signal DINB       : STD_LOGIC_VECTOR( 17 downto 0 ) ; -- [f[^ B
  signal DINB1      : STD_LOGIC_VECTOR( 15 downto 0 ) ; -- [f[^ B
  signal DOUTB      : STD_LOGIC_VECTOR( 17 downto 0 ) ; -- [f[^o B
  signal WEB        : STD_LOGIC_VECTOR( 0 downto 0 )  ; -- [ B
  
  signal CNT        : STD_LOGIC_VECTOR( 9 downto 0 )  ; -- S̃JE^[    512
  signal DATACNT0   : STD_LOGIC_VECTOR( 7 downto 0 )  ; -- f[^JE^[0 128
  signal DATACNT1   : STD_LOGIC_VECTOR( 7 downto 0 )  ; -- f[^JE^[1 128
  
  signal DATA       : STD_LOGIC_VECTOR( 17 downto 0 ) ; -- f[^b`
  signal MAC        : STD_LOGIC_VECTOR( 35 downto 0 ) ; -- ϘaZAL[^[
  
  signal MULOUT     : STD_LOGIC_VECTOR( 35 downto 0 ) ; -- |Zo
  
  
  component RAM1024x18_HILBERT255
    port (
      clka  : IN std_logic;
      dina  : IN std_logic_VECTOR(17 downto 0);
      addra : IN std_logic_VECTOR(9 downto 0);
      wea   : IN std_logic_VECTOR(0 downto 0);
      douta : OUT std_logic_VECTOR(17 downto 0);
      clkb  : IN std_logic;
      dinb  : IN std_logic_VECTOR(17 downto 0);
      addrb : IN std_logic_VECTOR(9 downto 0);
      web   : IN std_logic_VECTOR(0 downto 0);
      doutb : OUT std_logic_VECTOR(17 downto 0)
    );
  end component;
  
  begin
    
    U0 : RAM1024x18_HILBERT255
    port map (
      clka  => CLK,
      dina  => DINA,
      addra => MEMADR,
      wea   => MEMWE,
      douta => MEMDATAOUT,
      
      clkb  => CLK,
      dinb  => DINB,
      addrb => MADR(11 downto 2),
      web   => WEB,
      doutb => DOUTB
    );
    
    
    U1 : MULT18X18
    port map (
      P   => MULOUT,        -- 36-bit multiplier output
      A   => MEMDATAOUT,    -- W
      B   => DATA           -- f[^
    );
    
    
    ------------------------------------------------------------------
    --    ehqtB^[Z
    ------------------------------------------------------------------
    process( RESET, CLK ) begin
      if( RESET = '1' ) then
        CNT       <= (others=>'0');
        DATACNT0  <= (others=>'0');
        DATACNT1  <= (others=>'0');
        MEMWE     <="0";
        MAC       <= (others => '0');
        MEMADR    <= (others=>'0');
      elsif( rising_edge(CLK) ) then
        
        ND_D <= NDIN;
        if( NDIN='1' and ND_D='0' ) then  -- NDIN ̗オ
          CNT <= (others => '0');
        elsif( CNT/="1111111111" ) then
          CNT <= CNT+1;
        end if;
        
        if( CNT(8 downto 0) = "000000000" ) then -- =0@f[^[h
          MEMWE   <= "1";                       -- ̓f[^ԍŌ̃AhXɏ
          if( CNT(9)='0' ) then
            DINA   <= DIN_I;
            DIN_Q_D <= DIN_Q;                    -- 荞݂ĂƂŎg
          else
            DINA   <= DIN_Q_D;
          end if;
        else
          MEMWE   <="0";
        end if;
        
        if( CNT(0) = '0' ) then                 -- ̂Ƃ͒xf[^̓ǂݏo
          if( CNT(9) = '0' ) then               -- ch 0
            MEMADR  <= "10" & DATACNT0;
            DATACNT0<= DATACNT0 + 1;
          else                                  -- ch 1
            MEMADR  <= "11" & DATACNT1;
            DATACNT1<= DATACNT1 + 1;
          end if;
          DATA <= MEMDATAOUT;
          NDOUT   <= '0';
        else                                    -- ̂Ƃ̓tB^[W̓ǂݏo
          MEMADR  <= "0" & CNT( 9 downto 1 );
          if( CNT(8 downto 0) = "000000001" ) then -- =1@ZI
            if( CNT(9) = '0' ) then             -- ch 0
              DATACNT0<= DATACNT0 + 1;
              DOUT_I  <= DOUT_I_D;
              DOUT_Q  <= MAC( 34 downto 19 );
              NDOUT   <= '1';
            else                                -- ch 1
              DATACNT1<= DATACNT1 + 1;
              DOUT_I_D <= MAC( 34 downto 19 );   -- Ƃœo͂邽߃b`Ă
            end if;
            MAC <= (others => '0');
          else
            MAC <= MAC + MULOUT;                -- ϘaZ
          end if;
        end if;
        
      end if;
    end process;
    
    
    ------------------------------------------------------------------
    --    [ǂݏ
    ------------------------------------------------------------------
    
    WEB     <= "1" when( MWE='1' and MEME='1' and MADR(1 downto 0)="10" ) else "0";
    
    DATAOUT <= "00000000"  when(MEME='0') else
               DOUTB( 7 downto 0 ) when( MADR(1 downto 0)="00" ) else
               DOUTB(15 downto 8 ) when( MADR(1 downto 0)="01" ) else
               "000000" & DOUTB(17 downto 16);
    
    DINB    <= DATAIN(1 downto 0) & DINB1;
    
    process( CLK ) begin
      if( rising_edge(CLK) ) then
        if( MADR( 1 downto 0 ) = "00" ) then
          DINB1( 7 downto 0 ) <= DATAIN;    -- 
        elsif( MADR( 1 downto 0 ) = "01" ) then
          DINB1(15 downto 8 ) <= DATAIN;    -- 
        end if;
      end if;
    end process;
    
    
  end RTL;
----------------------------- end of architecture ----------------------------

