// file: test.v
// (c) Copyright 2009 - 2011 Xilinx, Inc. All rights reserved.
// 
// This file contains confidential and proprietary information
// of Xilinx, Inc. and is protected under U.S. and
// international copyright and other intellectual property
// laws.
// 
// DISCLAIMER
// This disclaimer is not a license and does not grant any
// rights to the materials distributed herewith. Except as
// otherwise provided in a valid license issued to you by
// Xilinx, and to the maximum extent permitted by applicable
// law: (1) THESE MATERIALS ARE MADE AVAILABLE "AS IS" AND
// WITH ALL FAULTS, AND XILINX HEREBY DISCLAIMS ALL WARRANTIES
// AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY, INCLUDING
// BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-
// INFRINGEMENT, OR FITNESS FOR ANY PARTICULAR PURPOSE; and
// (2) Xilinx shall not be liable (whether in contract or tort,
// including negligence, or under any other theory of
// liability) for any loss or damage of any kind or nature
// related to, arising under or in connection with these
// materials, including for any direct, or any indirect,
// special, incidental, or consequential loss or damage
// (including loss of data, profits, goodwill, or any type of
// loss or damage suffered as a result of any action brought
// by a third party) even if such damage or loss was
// reasonably foreseeable or Xilinx had been advised of the
// possibility of the same.
// 
// CRITICAL APPLICATIONS
// Xilinx products are not designed or intended to be fail-
// safe, or for use in any application requiring fail-safe
// performance, such as life-support or safety devices or
// systems, Class III medical devices, nuclear facilities,
// applications related to the deployment of airbags, or any
// other applications that could lead to death, personal
// injury, or severe property or environmental damage
// (individually and collectively, "Critical
// Applications"). Customer assumes the sole risk and
// liability of any use of Xilinx products in Critical
// Applications, subject only to applicable laws and
// regulations governing limitations on product liability.
// 
// THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS
// PART OF THIS FILE AT ALL TIMES.
//----------------------------------------------------------------------------
// User entered comments
//----------------------------------------------------------------------------
// None
//----------------------------------------------------------------------------

`timescale 1ps/1ps

(* CORE_GENERATION_INFO = "test,selectio_wiz_v3_1,{component_name=test,bus_dir=INPUTS,bus_sig_type=DIFF,bus_io_std=LVDS_33,use_serialization=false,use_phase_detector=false,serialization_factor=4,enable_bitslip=false,enable_train=false,system_data_width=8,bus_in_delay=VARIABLE,bus_out_delay=NONE,clk_sig_type=DIFF,clk_io_std=LVDS_33,clk_buf=BUFIO2,active_edge=BOTH_RISE_FALL,clk_delay=FIXED,v6_bus_in_delay=NONE,v6_bus_out_delay=NONE,v6_clk_buf=BUFIO,v6_active_edge=NOT_APP,v6_ddr_alignment=SAME_EDGE_PIPELINED,v6_oddr_alignment=SAME_EDGE,ddr_alignment=C0,v6_interface_type=NETWORKING,interface_type=NETWORKING,v6_bus_in_tap=0,v6_bus_out_tap=0,v6_clk_io_std=LVCMOS18,v6_clk_sig_type=DIFF}" *)

module test
   // width of the data for the system
 #(parameter sys_w = 8,
   // width of the data for the device
   parameter dev_w = 16)
 (
  // From the system into the device
  input  [sys_w-1:0] DATA_IN_FROM_PINS_P,
  input  [sys_w-1:0] DATA_IN_FROM_PINS_N,
  output [dev_w-1:0] DATA_IN_TO_DEVICE,
  output             DELAY_BUSY,
  input              DELAY_CLK,
  input              DELAY_DATA_CAL,
  input              DELAY_DATA_CE,                     // Enable signal for delay
  input              DELAY_DATA_INC,                    // Delay increment (high), decrement (low) signal
  input              CLK_IN_P,      // Differential clock from IOB
  input              CLK_IN_N,
  output             CLK_OUT,
  input              CLK_RESET,
  input              IO_RESET);
  // Signal declarations
  ////------------------------------
  wire               clock_enable = 1'b1;
  // After the buffer
  wire   [sys_w-1:0] data_in_from_pins_int;
  // Between the delay and serdes
  wire [sys_w-1:0]  data_in_from_pins_delay;
  wire   [sys_w-1:0] delay_data_busy;
  assign             DELAY_BUSY = |delay_data_busy;
  // Create the clock logic
  IBUFGDS_DIFF_OUT
    #(.IOSTANDARD ("LVDS_33"))
   ibufds_clk_inst
     (.I          (CLK_IN_P),
      .IB         (CLK_IN_N),
      .O          (clk_in_int_p),
      .OB         (clk_in_int_n));

  // delay the input clock
  IODELAY2
   #(
     .DATA_RATE                  ("DDR"),
     .IDELAY_VALUE               (0),
     .IDELAY_TYPE                ("FIXED"),
     .COUNTER_WRAPAROUND         ("STAY_AT_LIMIT"),
     .DELAY_SRC                  ("IDATAIN"),
     .SERDES_MODE                ("NONE"),
     .SIM_TAPDELAY_VALUE         (75))
   iodelay2_clk
    (
     // required datapath
     .IDATAIN                (clk_in_int_p),
     .DATAOUT                (clk_in_int_p_delay),
     .T                      (1'b1),
     // inactive data connections
     .DATAOUT2               (),
     .DOUT                   (),
     .ODATAIN                (1'b0),
     .TOUT                   (),
     // tie off the clocks
     .IOCLK0                 (1'b0),                 // No calibration needed
     .IOCLK1                 (1'b0),                 // No calibration needed
     // Tie of the variable delay programming
     .CLK                    (1'b0),
     .CAL                    (1'b0),
     .INC                    (1'b0),
     .CE                     (1'b0),
     .BUSY                   (),
     .RST                    (1'b0));

  IODELAY2
   #(
     .DATA_RATE                  ("DDR"),
     .IDELAY_VALUE               (0),
     .IDELAY_TYPE                ("FIXED"),
     .COUNTER_WRAPAROUND         ("STAY_AT_LIMIT"),
     .DELAY_SRC                  ("IDATAIN"),
     .SERDES_MODE                ("NONE"),
     .SIM_TAPDELAY_VALUE         (75))
   iodelay2_clk_n
    (
     // required datapath
     .IDATAIN                (clk_in_int_n),
     .DATAOUT                (clk_in_int_n_delay),
     .T                      (1'b1),
     // inactive data connections
     .DATAOUT2               (),
     .DOUT                   (),
     .ODATAIN                (1'b0),
     .TOUT                   (),
     // tie off the clocks
     .IOCLK0                 (1'b0),                 // No calibration needed
     .IOCLK1                 (1'b0),                 // No calibration needed
     // Tie of the variable delay programming
     .CLK                    (1'b0),
     .CAL                    (1'b0),
     .INC                    (1'b0),
     .CE                     (1'b0),
     .BUSY                   (),
     .RST                    (1'b0));

  // Set up the clock for use in the serdes
  BUFIO2 #(
      .DIVIDE_BYPASS ("FALSE"),
      .I_INVERT      ("FALSE"),
      .USE_DOUBLER   ("FALSE"),
      .DIVIDE        (1))
   bufio2_inst
     (.DIVCLK       (clk_div),
      .IOCLK        (clk_in_int_buf),
      .SERDESSTROBE (),
      .I            (clk_in_int_p_delay)
   );

  // also generated the inverted clock
  BUFIO2
    #(.DIVIDE_BYPASS ("FALSE"),
      .I_INVERT      ("FALSE"),
      .USE_DOUBLER   ("FALSE"),
      .DIVIDE        (1))
   bufio2_inv_inst
     (.DIVCLK        (),
      .IOCLK        (clk_in_int_inv),
      .SERDESSTROBE (),
      .I            (clk_in_int_n_delay));


   // Buffer up the "divided" copied version of the input clock
   BUFG clkout_buf_inst
    (.O (CLK_OUT),
     .I (clk_div));
  // We have multiple bits- step over every bit, instantiating the required elements
  genvar pin_count;
  generate for (pin_count = 0; pin_count < sys_w; pin_count = pin_count + 1) begin: pins
    // Instantiate the buffers
    ////------------------------------
    // Instantiate a buffer for every bit of the data bus
    IBUFDS
      #(.DIFF_TERM  ("FALSE"),             // Differential termination
        .IOSTANDARD ("LVDS_33"))
     ibufds_inst
       (.I          (DATA_IN_FROM_PINS_P  [pin_count]),
        .IB         (DATA_IN_FROM_PINS_N  [pin_count]),
        .O          (data_in_from_pins_int[pin_count]));

    // Instantiate the delay primitive
    ////-------------------------------
    IODELAY2
     #(.DATA_RATE                  ("DDR"),
       .IDELAY_VALUE               (0),
       .IDELAY_TYPE                ("VARIABLE_FROM_ZERO"),
       .COUNTER_WRAPAROUND         ("STAY_AT_LIMIT"),
       .DELAY_SRC                  ("IDATAIN"),
       .SERDES_MODE                ("NONE"),
       .SIM_TAPDELAY_VALUE         (75))
     iodelay2_bus
      (
       // required datapath
       .IDATAIN                (data_in_from_pins_int  [pin_count]),
       .DATAOUT                (data_in_from_pins_delay[pin_count]),
       .T                      (1'b1),
       // inactive data connections
       .DATAOUT2               (),
       .DOUT                   (),
       .ODATAIN                (1'b0),
       .TOUT                   (),
       // connect up the clocks
       .IOCLK0                 (clk_in_int_buf),       // High speed clock for calibration for SDR/DDR
       .IOCLK1                 (clk_in_int_inv),       // High speed clock for calibration for DDR
       .CLK                    (DELAY_CLK),
       .CAL                    (DELAY_DATA_CAL),
       .INC                    (DELAY_DATA_INC),
       .CE                     (DELAY_DATA_CE),
       .BUSY                   (delay_data_busy[pin_count]),
       .RST                    (IO_RESET));


    // Connect the delayed data to the fabric
    ////--------------------------------------
   // DDR register instantation
    IDDR2
     #(.DDR_ALIGNMENT  ("C0"),
       .INIT_Q0        (1'b0),
       .INIT_Q1        (1'b0),
       .SRTYPE         ("ASYNC"))
     iddr2_inst
      (.Q0             (DATA_IN_TO_DEVICE[pin_count]),
       .Q1             (DATA_IN_TO_DEVICE[sys_w + pin_count]),
       .C0             (clk_in_int_buf),
       .C1             (clk_in_int_inv),
       .CE             (clock_enable),
       .D              (data_in_from_pins_delay[pin_count]),
       .R              (IO_RESET),
       .S              (1'b0));
  end
  endgenerate
endmodule
